#!/bin/bash

#Constants
DATE=`date +%Y%m%d%H%M%S`

PRODUCTION_VERSION_FN='SignStream3-version.txt'
PRODUCTION_VERSION_URL='https://dai.cs.rutgers.edu/SignStream/download/release/'"$PRODUCTION_VERSION_FN"

NEW_VERSION_URL=$PRODUCTION_VERSION_URL

DISABLE_QUARANTINE_COMMAND='DisableQuarantine.command'

ORIG_DIR=$( pwd )

ORIG_CMD="$0"
ORIG_ARGS+=("$1")
ORIG_ARGS+=("$2")
ORIG_ARGS+=("$3")
ORIG_ARGS+=("$4")
ORIG_ARGS+=("$5")
ORIG_ARGS+=("$6")

# Navigate to root SignStream directory

CMD="$( basename "$0" )"
PROCID="$$"
TERMINAL_NAME="$CMD""$PROCID"
echo -n -e "\033]0;""$TERMINAL_NAME""\007" # changes the running Terminal name for use when closing below

DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cd "${DIR}"

PLIST_FILE='SignStream3.app/Contents/Info.plist'

CURRENT_PLIST_FILE="$DIR"'/'"$PLIST_FILE"


# $1 > $2 : return 1
# $1 = $2 : return 0
# $1 < $2 : return 2
vercomp () {
    if [[ "x""$1" == "x""$2" ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}


# verbetween 1 0 3: return 4
# verbetween 1 1 3: return 2
# verbetween 1 2 3: return 0
# verbetween 1 3 3: return 1 
# verbetween 1 4 3: return 3
verbetween() {
  vercomp "$1" "$3"
  if [[ "$?" -eq 1 ]]; then
    min="$3"
    max="$1"
  else
    min="$1"
    max="$3"
  fi
 vercomp "$min" "$2"
  if [[ "$?" -eq 1 ]]; then
    return 4
  elif [[ "$?" -eq 0 ]]; then
    return 2 
  elif [[ "$?" -eq 2 ]]; then
    vercomp "$2" "$max"
    if [[ "$?" -eq 2 ]]; then
      return 0
    elif [[ "$?" -eq 0 ]]; then
      return 1
    else
      return 3
    fi
  fi
}

close (){
  echo ""
  echo "You can now close this window (click, or press enter, to close)."
  cd "$DIR"
  read junk
  $( while pgrep -f "$CMD" | grep '^'"$PROCID"'$' &>/dev/null; do sleep 1; done; osascript -e 'tell application "Terminal" to close (first window whose name contains "'"$TERMINAL_NAME"'")' &> /dev/null ) &
  exit
}


VERSION=$( cat "$CURRENT_PLIST_FILE" | grep -A 1 '<key>CFBundleVersion</key>' | grep '.*<string>.*</string>.*' | sed -e 's/.*<string>\(.*\)<\/string>.*/\1/' )
PATCH_VERSION="1"

verbetween "3.0.0" $VERSION "3.3.0"
if [[ "$?" -eq 2 || "$?" -eq 0 ]]; then # "3.0.0"<=VERSION<3.3.0
  if [[ -f "$DIR"'/.patch-'"$VERSION"'-'"$PATCH_VERSION" ]]; then
    echo "SignStream3 has already been patched. There is not need for this script."
    close
  else
    SIGNSTREAM_PATCH_FN='SignStream3-'"$VERSION"'-patch-'"$PATCH_VERSION"'.zip'
    SIGNSTREAM_PATCH_URL='https://dai.cs.rutgers.edu/SignStream/download/release/patches/'"$SIGNSTREAM_PATCH_FN"

    SIGNSTREAM_PATCH_REQUEST=$( curl -L --silent --write-out "HTTPSTATUS:%{http_code}" --output "$SIGNSTREAM_PATCH_FN" "$SIGNSTREAM_PATCH_URL" )

    if [ "x"$( echo "$SIGNSTREAM_PATCH_REQUEST" | grep '^HTTPSTATUS:'  | sed -e 's/HTTPSTATUS://' ) != "x200" ]; then
      echo "Error in retrieving patch file."
      close
    else
      echo "Found patch $PATCH_VERSION"
  
      #Unzip Latest Version of SignStream.
      echo "Unzipping the new version..."
      unzip -o -q "$SIGNSTREAM_PATCH_FN"
      if [ "$?" != "0" ]; then
        echo "An error occurred while unzipping a patch for the new version of SignStream3. Please contact the administrator."
        echo "Cleaning up..."
        rm -rf "$SIGNSTREAM_PATCH_FN"
        close
      fi
  
      #Delete the zipped version
      rm -rf "$SIGNSTREAM_PATCH_FN"
          #Delete the zipped version
    
      # In future may need to run script after patch is downloaded.
      PATCH_COMMAND='Patch-'$VERSION'-'$PATCH_VERSION'.command'
      if [ -f "$PATCH_COMMAND" ]; then
        ./"$PATCH_COMMAND"
      fi

  
      echo "Applied patch $PATCH_VERSION"
    fi
  fi
else
  echo "SignStream3 is already at or beyond version 3.3.0. No need for this script."
  close
fi


# Do the update with the SignStream menu command
./"$DISABLE_QUARANTINE_COMMAND" --noautoclose
./Update.command --patch --install latest --force

