#!/usr/bin/python

# This script pads a video to be in sync with the audio

import getopt
import sys
import os
import os.path

import video as v
import video.cut as vc

def usage():
  sys.stderr.write(
"""
Usage: pad_video_start.py
  [--tmpdir=.] [--ffmpegdir=/usr]
  <source video file> <destination video file> <num_frames>
""")
  sys.exit(1)

def prepend_frames(videodir, num_new_frames):
  num_frames = len(os.listdir(videodir))
  if num_new_frames > 0:
    for i in xrange(num_frames, 0, -1):
      old_name = "%05d.ppm" % i
      new_name = "%05d.ppm" % (i + num_new_frames)
      os.rename(os.path.join(videodir, old_name), os.path.join(videodir, new_name))
    # copy start frames via symlink
    first_frame = "%05d.ppm" % (num_new_frames + 1)
    for i in xrange(1, num_new_frames + 1):
      target_frame = "%05d.ppm" % i
      os.symlink(os.path.abspath(os.path.join(videodir, first_frame)), os.path.join(videodir, target_frame))


class MPEG4Encoder(vc.FFMPEGEncoder):
  """Encodes native MPEG4 files"""
  def __init__(self, quality=2, **kw):
    super(MPEG4Encoder, self).__init__(**kw)
    self.quality = quality
  
  def arguments(self):
    args = super(MPEG4Encoder, self).arguments()
    args.extend(["-vcodec", "mpeg4"])
    if self.quality is not None:
      args.extend(["-q:v", str(self.quality)])
    return args

  
if __name__ == '__main__':
  try:
    (options, args) = getopt.getopt(sys.argv[1:], "",
                                    ["tmpdir=", "ffmpegdir="])
  except:
    usage()
  if len(args) != 3:
    usage()
  
  source = args[0]
  destination = args[1]
  num_frames = int(args[2])
  
  ffmpegdir = '/usr'
  tmpdir = '.'
  for (option, value) in options:
    if option == '--tmpdir':
      tmpdir = value
    elif option == '--ffmpegdir':
      ffmpegdir = value
  
  ffprobe = os.path.join(ffmpegdir, 'ffprobe')
  ffmpeg = os.path.join(ffmpegdir, 'ffmpeg')
  tmpdir = os.path.join(tmpdir, os.path.basename(source))
  video = v.Video(source, ffprobe=ffprobe)

  vd = video.get_duration()
  fps = video.get_fps()
  print "V: %s: duration: %d; fps: %5.2f\n" % (source, vd, fps)
  
  try:
    os.mkdir(tmpdir)
  except OSError, e:
    pass
  
  fg = vc.FrameGrabber(format='ppm', type='all', ffmpeg=ffmpeg)
  fg.transcode(source, os.path.join(tmpdir, "%05d.ppm"))

  prepend_frames(tmpdir, num_frames)
  
  mpeg4enc = MPEG4Encoder(ffmpeg=ffmpeg, quality=2, args=['-an'])
  mpeg4enc.transcode(os.path.join(tmpdir, "%05d.ppm"), destination, fps=fps)
  
  newvideo = v.Video(destination, ffprobe=ffprobe)
  nvd = newvideo.get_duration()
  nvfps = newvideo.get_fps()
  
  print "New video duration: %d, fps: %5.2f\n" % (nvd, nvfps)
  
  for file in os.listdir(tmpdir):
    os.remove(os.path.join(tmpdir, file))
  os.rmdir(tmpdir)
  