package edu.bu.signstream.common.db;

import java.util.*;
import edu.bu.signstream.common.db.parser.*;
import java.io.File;
import edu.bu.signstream.common.util.vo.*;

public class SS2Database implements BaseDatabase {
	private SignStreamDocument doc = null;

	public SS2Database(String fileName) throws IllegalArgumentException {
		SS2Parser parser = new SS2Parser();
		doc = parser.parseDBFileData(new File(fileName));
	}

	@Override
	public ArrayList getExcerpts() {
		ArrayList excerpts = new ArrayList();

		HashMap map = doc.getUtterances();
		Set set = map.entrySet();
		Iterator it = set.iterator();
		while (it.hasNext()) {
			Utterance utter = (Utterance) it.next();
			excerpts.add(utter.getExcerpt());
		}

		return excerpts;
	}

	@Override
	public Utterance getUtterance(String id) {
		return doc.getUtterance(id);
	}

	@Override
	public HashMap getMediaFiles() {
		return doc.getMediaFiles();
	}

	@Override
	public HashMap getParticipants() {
		return doc.getParticipants();
	}

	@Override
	public ArrayList getSegments(String uid) {
		Utterance utter = doc.getUtterance(uid);
		return utter.getSegments();
	}

	public HashMap getTracks(String uid, String pid) {
		HashMap tracks = new HashMap();

		ArrayList segments = getSegments(uid);

		for (int i = 0; i < segments.size(); i++) {
			Segment segment = (Segment) segments.get(i);
			if (segment.getParticipantId().equals(pid)) {
				tracks = segment.getTracks();
			}
		}

		return tracks;
	}

	@Override
	public ArrayList getFields(String uid, String pid, String fid) {
		HashMap tracks = getTracks(uid, pid);

		return (ArrayList) tracks.get(fid);
	}
/*
	public ViewSegment getViewSegment(
		String utteranceId,
		String participantId) {

		Utterance utterance = doc.getUtterance(utteranceId);
		HashMap hMediaFiles = utterance.getMediaFiles();

		ArrayList mediaFiles = new ArrayList();
		Iterator it = mediaFiles.iterator();
		while (it.hasNext()) {
			MediaFile mf = (MediaFile) it.next();
			mediaFiles.add(mf.getPath());
		}

		HashMap codingScheme = doc.getCodingScheme();
		Segment segment = null;

		ArrayList segments = utterance.getSegments();
		Iterator sIt = segments.iterator();
		while (sIt.hasNext()) {
			Segment s = (Segment) sIt.next();
			if (s.getParticipantId().equals(participantId)) {
				segment = s;
				break;
			}
		}

		HashMap participants = doc.getParticipants();
		Participant participant = (Participant) participants.get(participantId);

		return new ViewSegment(mediaFiles, codingScheme, segment, participant);
	}
*/	
}
