package edu.bu.signstream.common.db;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.logging.Level;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class UtilityFilesXMLWriter {
	
	public void saveAnnotatorsLibrary(ParticipantsLibrary library) throws Exception {
		saveAnnotatorsLibrary(library.getGlobalParticipantsFileName(), library.getGlobalParticipants());
		saveAnnotatorsLibrary(library.getLocalParticipantsFileName(), library.getLocalParticipants());	
	}
	
	public void saveAnnotatorsLibrary(String fileName, HashMap<String, SS3Participant> participants) throws Exception {
		Document document = XMLWriter.createNewXMLDocument(fileName);

		try {
	         Element root = document.createElement("SIGNSTREAM-DATABASE");
	         root.setAttribute(
	            "SIGNSTREAM-VERSION",
	            "" );
	         root.setAttribute("SOURCE", "" );
	         root.setAttribute("DATABASE-VERSION", "");
	         root.setAttribute("SIGNATURE", "");
	         
	         Element dstrEl = document.createElement("DISTRIBUTOR");
	         dstrEl.appendChild(document.createTextNode("American Sign Language Linguistic Research Project, Boston University, Boston, MA."));

	         root.appendChild(dstrEl);
	         
	         Element participantsEl = document.createElement("ANNOTATORS");
	         root.appendChild(participantsEl);
	         
	         Iterator<Entry<String, SS3Participant>> iter = participants.entrySet().iterator();	
	         while(iter.hasNext()) {
	        	 SS3Participant prt = iter.next().getValue();
	        	 Element participantEl = document.createElement("ANNOTATOR");
	        	 participantsEl.appendChild(participantEl);
	        	 participantEl.setAttribute("ID", prt.getId());
	        	 participantEl.setAttribute("FULL_NAME", prt.getFullName());
	        	 participantEl.setAttribute("LABEL", prt.getLabel());
	        	 participantEl.setAttribute("COMMENTS", prt.getComments());
	         }

	         XMLWriter.saveDocument(root, fileName);

	      } catch (Exception ex) {
	    	  SS3SignStreamApplication.logger.log(Level.INFO, "Annotators library had NOT been saved."  + ex.getStackTrace());

	         throw new Exception("Annotators library had NOT been saved.");
	      }
	   }

	
	
	public void saveParticipantsLibrary(ParticipantsLibrary library) throws Exception{
		saveParticipantsLibrary(library.getGlobalParticipantsFileName(), library.getGlobalParticipants());
		saveParticipantsLibrary(library.getLocalParticipantsFileName(), library.getLocalParticipants());	
	}
	
	public void saveParticipantsLibrary(String fileName, HashMap<String, SS3Participant> participants) throws Exception {
	
		Document document = XMLWriter.createNewXMLDocument(fileName);

		try {
	         Element root = document.createElement("SIGNSTREAM-DATABASE");
	         root.setAttribute(
	            "SIGNSTREAM-VERSION",
	            "" );
	         root.setAttribute("SOURCE", "" );
	         root.setAttribute("DATABASE-VERSION", "");
	         root.setAttribute("SIGNATURE", "");
	         
	         Element dstrEl = document.createElement("DISTRIBUTOR");
	         dstrEl.appendChild(document.createTextNode("American Sign Language Linguistic Research Project, Boston University, Boston, MA."));

	         root.appendChild(dstrEl);
	         
	         Element participantsEl = document.createElement("PARTICIPANTS");
	         root.appendChild(participantsEl);
	         
	         Iterator<Entry<String, SS3Participant>> iter = participants.entrySet().iterator();
	         while(iter.hasNext()) {
	        	 SS3Participant prt = iter.next().getValue();
	        	 Element participantEl = document.createElement("PARTICIPANT");
	        	 participantsEl.appendChild(participantEl);
	        	 participantEl.setAttribute("ID", prt.getId());
	        	 participantEl.setAttribute("FULL_NAME", prt.getFullName());
	        	 participantEl.setAttribute("LABEL", prt.getLabel());
	        	 participantEl.setAttribute("GENDER", ""+prt.getGender());
	        	 participantEl.setAttribute("DOMINANT_HAND", prt.getDomHand());
	        	 participantEl.setAttribute("YEAR_OF_BIRTH", prt.getYearOfBirth());
	        	 participantEl.setAttribute("NATIVE_LANGUAGE", prt.getNativeLanguage());
	        	 participantEl.setAttribute("COMMENTS", prt.getComments());
	        	 participantEl.setAttribute("PARENT_INFO", prt.getParentsInfo());
	        	 participantEl.setAttribute("BACKGROUND", prt.getBackground());
	         }

	         XMLWriter.saveDocument(root, fileName);

	      } catch (Exception ex) {
	    	  SS3SignStreamApplication.logger.log(Level.INFO, "Participants library had NOT been saved."  + ex.getStackTrace());

	         throw new Exception("Participants library had NOT been saved.");
	      }
	   }
	}

