package edu.bu.signstream.common.util;

import edu.bu.signstream.common.SSMath;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

public class CoordinateTimeConverter {
    // How many frames should the max zoom level show per width
    private static final long MAX_ZOOM_FRAMES = 10;

    // When fitting the zoom, pad the width by this amount.
    // The drawing code has a small margin, so fitting the exact width would cause
    // utterances to be slightly larger than the available drawing space.
    // This should really be accounted for in the drawing logic.
    private static final int WIDTH_PAD = -100;

    // LOG_A = 0.99 makes the loglerp value increase slowly near 0, and quickly near 1.
    // See https://www.desmos.com/calculator/ixdtn1t1ap
    private static final double LOG_A = 0.99;

    // Dumb random defaults since the code calls this a million times before it should.
    // These should really be removed.
    // TODO: Remove these.
    public double timebase = 30000;
    public double timestep = 1001;
    public double framerate = timebase / timestep;
    public double duration = timestep * 30 * 60;
    private double width = 1000;
    
    private double zoom = 0.0;

	public static int NUMBER_OF_SLIDER_TICKS = 10000;
    
	public static int getNumberOfSliderTicks() {
		return NUMBER_OF_SLIDER_TICKS;
	}

	public void setNumberOfSliderTicks(int nUMBER_OF_SLIDER_TICKS) {
		NUMBER_OF_SLIDER_TICKS = nUMBER_OF_SLIDER_TICKS;
	}

    public void setTimeScale(double timebase, double timestep, double duration) {
        this.timebase = timebase;
        this.timestep = timestep;
        this.framerate = timebase / timestep;
        this.duration = duration;
    }

    public double timestep() { return timestep; }
    public double timebase() { return timebase; }
    public double duration() { return duration; }
    public double getFramerate() { return framerate; }

    public void resize(long width) {
        this.width = width + WIDTH_PAD;
    }

    private double minFrameWidth() {
        double frames = duration / timestep;
        return width / frames;
    }

    private double maxFrameWidth() {
        return width / MAX_ZOOM_FRAMES;
    }

    public double frameWidth() {
        return SSMath.lerp(minFrameWidth(), maxFrameWidth(), zoom);
    }

    public double getFit(double duration) {
        double frames = duration / timestep;
        var linearFit = SSMath.ilerp(minFrameWidth(), maxFrameWidth(), this.width / frames);
        return SSMath.logilerpc(LOG_A, linearFit);
    }
    public double getFit(Duration duration) { return getFit(duration.as(TimeUnit.TIMESTAMP)); }

    public void fit(double duration) {
        double frames = duration / timestep;
        this.zoom = SSMath.ilerp(minFrameWidth(), maxFrameWidth(), width / frames);
	}
    public void fit(Duration duration) { fit(duration.as(TimeUnit.TIMESTAMP)); }

    public void zoom(double zoom) {
        this.zoom = SSMath.loglerpc(LOG_A, zoom);
    }

    public double zoom() {
        return SSMath.logilerpc(LOG_A, zoom);
    }


    public double convertTimeToCoordinate(double time) {
        return convertTimeInfoToCoordinate(TimeRange.ABSOLUTE.time(Math.round(time), TimeUnit.TIMESTAMP));

//        double frame = time / timestep;
//        return frame * frameWidth();
    }

	public long convertCoordinateToTime(double coordinate) {
        return convertCoordinateToTimeInfo(coordinate).as(TimeUnit.TIMESTAMP, Rounding.NEAREST);

//        double frame = coordinate / frameWidth();
//        var time = Math.round(frame * timestep);
//        return time;
    }
    
    public double convertTimeInfoToCoordinate(Time timeInfo) {
//	    return convertTimeToCoordinate(timeInfo.timestamp());
        return timeInfo.absolute().as(TimeUnit.COORDINATE, Rounding.NEAREST);
    }
    
    public Time convertCoordinateToTimeInfo(double coordinate) {
	    return TimeRange.ABSOLUTE
            .time(Math.round(coordinate), TimeUnit.COORDINATE)
            .in(SS3Singleton.getVideoTimeRange());
//        double frame = coordinate / frameWidth();
//        var time = Math.round(frame * timestep);
//        return SS3Singleton.getCurrentTimeRange().time(Math.round(time)).subtract(SS3Singleton.getCurrentTimeRange().offset());
    }

	/*
	    TODO: REMOVE EVERYTHING BELOW EVENTUALLY
	 */

    @Deprecated
    public double getMovieFrameDuration() {
        return timestep;
    }


    @Deprecated
    public double getPixelsInTimeFrame() {
        return frameWidth();
    }

    @Deprecated
    public int getScreenWidth(double _duration) {
        return (int)width;
    }

    @Deprecated
    public double getAdjustedMovieTime(double coordinate) {
        return convertCoordinateToTime(coordinate);
    }

    @Deprecated
    public double getFrameNumber(double time) {
        return time / timestep;
    }

    @Deprecated
    public double getCorrespondingMovieTime(double frame) {
        return frame * timestep;
    }
}
