package edu.bu.signstream.common.util.https.responses;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.Header;

import java.util.Objects;
// import org.json.JSONObject;

/**
 * This class is used to store the response sending a POST request with a video to the DAI server.
 */
public class DAIResponse extends BasicResponse {

    private final int daiStatusCode;
    private final String message;
    private final DAIData data;

    private final String session;
    private final String resultsLink;
    private final String choiceAPILink;
    private final String resultsJSON;
    private final ObjectMapper mapper;
    private final BaseDAIResponse baseDAIResponse;

    public DAIResponse(int statusCode, long contentLength, Header contentType, String contentJSON) throws JsonProcessingException {
        super(statusCode, contentLength, contentType, contentJSON);

        this.mapper = new ObjectMapper(); // This is a Jackson XML object, use to parse the JSON response

        this.baseDAIResponse = this.mapper.readValue(contentJSON, BaseDAIResponse.class); // Content JSON is the JSON string, BaseDAIResponse.class is the class to map the JSON data to
        this.daiStatusCode = this.baseDAIResponse.getDAIStatusCode();
        this.message = this.baseDAIResponse.getMessage();
        this.data = this.baseDAIResponse.getData(); // Because the DAI response returns a string with a JSON object inside of it, the Data is its own JSON mapper object that BaseDAI class returns with

        this.session = this.data.getSession();
        this.resultsLink = this.data.getResultsDisplay();
        this.choiceAPILink = this.data.getChoiceAPILink();
        this.resultsJSON = this.data.getResultsJSON();

        System.out.println("DAI Status Code: " + this.daiStatusCode);
        System.out.println("DAI Message: " + this.message);
    }

    public int getDAIStatusCode() {
        return this.daiStatusCode;
    }

    public String getMessage() {
        return this.message;
    }

    public DAIData getData() {
        return this.data;
    }

    public String getSession() {
        return this.session;
    }

    public String getResultsLink() {
        return this.resultsLink;
    }

    public String getChoiceAPILink() {
        return this.choiceAPILink;
    }

    @Override
    public String toString() {
        try {
            return this.mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this.baseDAIResponse);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

}

/**
 * This class is used to map the surface level JSON object to.
 */
@JsonIgnoreProperties(ignoreUnknown = true) // If there are any properties in the JSON that are not in this class, ignore them, do not throw exceptions as it breaks SignStream if it does.
class BaseDAIResponse {
    @JsonProperty("status")
    private int daiStatusCode;

    @JsonProperty("message")
    private String message;

    @JsonProperty("data")
    private DAIData data; // This is a JSON object inside of a JSON object, so we need to create a new class to map the data to, JacksonJson will automagically do this for us

    public int getDAIStatusCode() {
        return this.daiStatusCode;
    }

    public String getMessage() {
        return this.message;
    }

    public DAIData getData() {
        return this.data;
    }
}

/**
 * This class is used to store the sub JSON object inside of the DAI response.
 */
@JsonIgnoreProperties(ignoreUnknown = true)
class DAIData {
    @JsonProperty("session")
    private String session;

    @JsonProperty("results-display")
    private String resultsDisplay;

    @JsonProperty("choice-api_link")
    private String choiceAPILink;

    @JsonProperty("results-json")
    private String resultsJSON;

    public String getResultsJSON() {
        return this.resultsJSON;
    }

    public String getSession() {
        return this.session;
    }

    public String getResultsDisplay() {
        return this.resultsDisplay;
    }

    public String getChoiceAPILink() {
        return this.choiceAPILink;
    }
}
