package edu.bu.signstream.common.util.vo.ss3.db;

import java.io.File;
import java.net.URI;
import java.util.*;
import java.util.stream.Collectors;

import javax.swing.tree.DefaultMutableTreeNode;

import edu.bu.signstream.common.db.ss3.SS3Writer;
import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.graph.Graph;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.note.SS3NoteCollection;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.media.Audio;
import edu.bu.signstream.media.Video;

public class SS3Database extends SS3Element {
    private String version;

    // Set to true if critical validation errors are found and ignored
    private boolean broken = false;

	// contains name of the original file + _utteranceID in format 001
	private String fileName = "";
	private String directory = "";
	private String parentDirectoryPath = SSResources.collectionPath;
	private DefaultMutableTreeNode dbNavigationPanelNode;
	
	private double movieDuration = 0;
	private int packetDuration = 0;
	private int movieTimescale = 0;
	
	private double temporalPartitionPixelInTimeFrame = 0; 
	private double temporalPartitionMovieTime = 0;
	private double temporalPartitionOnScreenLocation = 0;
	
	public static String tempDirectoryPrefix = "tmp_";
	public static String copyFilePrefix = "copy_";
	public static String dbFileExtension = ".ss3";
	public static String zipFileExtension = ".zip";
	public static String macroFileExtension = ".3.xml";

	public static final String COLLECTION_EXT = ".ssc";
	public static final String COLLECTION_MACRO_UNIT_DIR = "macro_unit.d";
	
    public static final String MACRO_UNIT_FILENAME = "macrounit1.xml";
    public static final String CONF_FILE = "signstream.conf";
    public static final String CONF_SEPARATOR = "=";
    public static final String CONF_VERSION_KEY = "version";
    public static final String CONF_MACRO_UNIT_KEY = "macro_unit.1.xml";
    
    public static final String COLLECTION_VERSION = "1.0";		// this is the version of the collection package format, not the xml version


	public String getVersion() { return version; }
    public void setVersion(String version) { this.version = version; }

    public void setBroken() { this.broken = true; }
    public boolean isBroken() { return this.broken; }

	public double getTemporalPartitionPixelInTimeFrame() {
		return temporalPartitionPixelInTimeFrame;
	}

	public void setTemporalPartitionPixelInTimeFrame(
			double temporalPartitionPixelInTimeFrame) {
		this.temporalPartitionPixelInTimeFrame = temporalPartitionPixelInTimeFrame;
	}

	public double getTemporalPartitionMovieTime() {
		return temporalPartitionMovieTime;
	}

	public void setTemporalPartitionMovieTime(double temporalPartitionMovieTime) {
		this.temporalPartitionMovieTime = temporalPartitionMovieTime;
	}

	public double getTemporalPartitionOnScreenLocation() {
		return temporalPartitionOnScreenLocation;
	}

	public void setTemporalPartitionOnScreenLocation(
			double temporalPartitionOnScreenLocation) {
		this.temporalPartitionOnScreenLocation = temporalPartitionOnScreenLocation;
	}

	private File readFromXMLFile = null;

	private String excerpt = "";
	private String number = "";

	private SS3DBProfile profile = new SS3DBProfile();

	// GD: 2021-04-21: not sure the purpose of this element - it gets read/written, and that's it
	@Deprecated
	private ArrayList<Reference> cSchemeRef = new ArrayList<Reference>();

	private ArrayList<SS3Participant> selectedParticipants = new ArrayList<SS3Participant>();
	private ArrayList<SS3Participant> selectedAnnotators = new ArrayList<SS3Participant>();

	private final HashMap<String, TimeUnit> timeUnits = new HashMap<>();
	private final HashMap<String, TimeRange> timeRanges = new HashMap<>();

	private final HashMap<Integer, Video> videos = new HashMap<>();
    private final HashMap<Integer, Audio> audios = new HashMap<>();
    private final HashMap<String, Graph> graphs = new HashMap<>();

	private HashMap<String, SS3MediaFile> mFiles = new HashMap<>();

	private final SS3ElementCollection tracks = new SS3ElementCollection();
	private final SS3NoteCollection notes = new SS3NoteCollection();

	private final SS3ElementCollection selections = new SS3ElementCollection();
	
	private SS3GlossWindow glsWindow = new SS3GlossWindow();

	private SS3CodingScheme scheme = null;

	private Time startTime;
	private Time endTime;

	@Deprecated
	private String participantName = null;
	
	private Excerpt excerptObj = null;

	public SS3Database(Time startTime, Time endTime) {
		this.startTime = startTime;
		this.endTime = endTime;
	}

	public SS3Database() {
	    this.startTime = TimeRange.ABSOLUTE.zero();
        this.endTime = TimeRange.ABSOLUTE.zero();
    }
	
	public Excerpt getExcerptObj() {
		if(excerptObj == null && excerpt != null && excerpt.length() == 0) {
			excerptObj = new Excerpt(number, excerpt);
		}
		return excerptObj;
	}
		
	public double getMovieDuration() {
		return movieDuration;
	}

	public void setMovieDuration(double movieDuration) {
		this.movieDuration = movieDuration;
	}

	public int getPacketDuration() {
		return packetDuration;
	}

	public void setPacketDuration(int packetDuration) {
		this.packetDuration = packetDuration;
	}
	
	public int getMovieTimescale() {
		return movieTimescale;
	}
	
	public void setMovieTimescale(int movieTimescale) {
		this.movieTimescale = movieTimescale;
	}
	
	public void setExcerptObj(Excerpt excerptObj) {
		this.excerptObj = excerptObj;
	}
	
	public SS3GlossWindow getSS3GlossWindow() {
		return glsWindow;
	}
	
	public void setSS3GlossWindow(SS3GlossWindow glsWindow) {
		this.glsWindow = glsWindow;
	}

	public boolean addSelectedParticipant(SS3Participant participant) {
		for (SS3Participant part: selectedParticipants) {
			if (part.getId().equals(participant.getId()))
					return false;
		}
		
		selectedParticipants.add(participant);
		return true;
	}

	public ArrayList<SS3Participant> getSelectedParticipants() {
		return (ArrayList<SS3Participant>) selectedParticipants.clone();
	}
	
	public void removeParticipant(SS3Participant participant) {
		selectedParticipants.remove(participant);
	}
	
	public SS3Participant getSelectedParticipant(String participantID) {
		for(SS3Participant participant : selectedParticipants) {
			if(participant.getId().equalsIgnoreCase(participantID))
				return participant;
		}
		return null;
	}
	
	public SS3Participant getSelectedAnnotator(String annotatorID) {
		for(SS3Participant annotator : selectedAnnotators) {
			if(annotator.getId().equalsIgnoreCase(annotatorID))
				return annotator;
		}
		return null;
	}
	
	public void setSelectedParticipants(ArrayList<SS3Participant> participants) {
		this.selectedParticipants = participants;
	}
	
	public void setSelectedAnnotators(ArrayList<SS3Participant> annotators) {
		this.selectedAnnotators = annotators;
	}

	public boolean addSelectedAnnotator(SS3Participant participant) {
		for (SS3Participant part: selectedAnnotators) {
			if (part.getId().equals(participant.getId()))
					return false;
		}
		
		selectedAnnotators.add(participant);
		return true;
	}

	public ArrayList<SS3Participant> getSelectedAnnotators() {
		return (ArrayList<SS3Participant>) selectedAnnotators.clone();
	}
	
	public void removeAnnotator(SS3Participant annotator) {
		selectedAnnotators.remove(annotator);
	}

	public String getParentDirectoryPath() {
		return parentDirectoryPath;
	}

	public void setParentDirectoryPath(String parentDirectoryPath) {
		this.parentDirectoryPath = parentDirectoryPath;
	}

	public DefaultMutableTreeNode getDbNavigationPanelNode() {
		return dbNavigationPanelNode;
	}

	public void setDbNavigationPanelNode(DefaultMutableTreeNode dbNavigationPanelNode) {
		this.dbNavigationPanelNode = dbNavigationPanelNode;
	}

	public void setStartTime(Time startTime) {
		this.startTime = startTime;
	}

	public Time getStartTime() {
		if (startTime.doubleValue() >= 0)
			return startTime;

		if (glsWindow == null && glsWindow.getTemporalPartitions().isEmpty()) {
		    startTime = SS3Singleton.getVideoTimeRange().zero();
		} else {
			double movieTime = -1;
			ArrayList<SS3TemporalPartition> partitions = glsWindow.getTemporalPartitions();

			for (SS3TemporalPartition partition : partitions) {
				ArrayList<SS3GlossWindowSegment> partitionSegments = partition.getTemporalPartitionSegments();
				for (SS3GlossWindowSegment partitionSegment : partitionSegments) {
					ArrayList<SS3GlossWindowItem> items = partitionSegment.getSS3GlossWindowItems();
					for (SS3GlossWindowItem item : items) {
						double newMovieTime = item.getStartTimeInfo().doubleValue();
						if (movieTime == -1)
							movieTime = newMovieTime;

						if (newMovieTime >= 0 && newMovieTime < movieTime)
							movieTime = newMovieTime;
					}
				}
			}
			// TODO: Remove round when `TimeInfo` is `long`
            startTime = SS3Singleton.getVideoTimeRange().time(Math.round(movieTime), TimeUnit.TIMESTAMP);
		}

		return startTime;
	}

	public void setEndTime(Time endTime) {
		this.endTime = endTime;
	}

	public Time getEndTime() {
		if (endTime.doubleValue() >= 0)
			return endTime;

		if (glsWindow == null && glsWindow.getTemporalPartitions().isEmpty()) {
            endTime = SS3Singleton.getVideoTimeRange().zero();
		} else {
			double movieTime = -1;
			ArrayList<SS3TemporalPartition> partitions = glsWindow.getTemporalPartitions();

			for (SS3TemporalPartition partition : partitions) {
				ArrayList<SS3GlossWindowSegment> partitionSegments = partition.getTemporalPartitionSegments();
				for (SS3GlossWindowSegment partitionSegment : partitionSegments) {
					ArrayList<SS3GlossWindowItem> items = partitionSegment.getSS3GlossWindowItems();
					for (SS3GlossWindowItem item : items) {
						double newMovieTime = item.getEndTimeInfo().doubleValue();

						if (movieTime == -1)
							movieTime = newMovieTime;

						if (newMovieTime >= 0 && newMovieTime > movieTime)
							movieTime = newMovieTime;
					}
				}
			}
            // TODO: Remove round when `TimeInfo` is `long`.
            endTime = SS3Singleton.getVideoTimeRange().time(Math.round(movieTime), TimeUnit.TIMESTAMP);
		}
		return endTime;
	}

	public String getDirectory() {
		return directory;
	}

	public void setDirectory(String directory) {
		this.directory = directory;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public String getFileName() {
		return fileName;
	}

	public void addNote(SS3Note note) {
		notes.addNote(note);
	}

	public ArrayList<SS3Note> getNotesList() {
		return notes.getNotesList();
	}

	public SS3Note getNote(String noteID) {
		return notes.getNote(noteID);
	}

	public void addStatement(SS3GlossWindowItem statement) {
	}

	public void addSelection(SS3Selection selection) {
		selections.addElement(selection);
	}

	public void addTrack(SS3Track track) {
		tracks.addElement(track);
	}
	
	public void removeTrackByFieldID(String fieldID) {
		tracks.removeElement(getTrack(fieldID));		
	}

	@Deprecated
	public ArrayList<Object> getTracksList() {
		return tracks.getObjectsList();
	}

	public List<SS3Track> getTracks() {
	    return tracks.getObjectsList().stream()
            .map(SS3Track.class::cast)
            .collect(Collectors.toList());
	}

	public SS3Track getTrack(String trackID) {		
		Object obj = tracks.getElement(trackID);
		if (obj != null) {
			return (SS3Track) obj;
		} else {
			SS3Track track = new SS3Track();
			track.setId(trackID);
			tracks.addElement(track);
			return track;
		}
	}

	public SS3Track getTrack(SS3StatementField field, 
			String fieldID, 
			int startMovieTime, int endMovieTime) {

		if(field != null) {
			fieldID = field.getId();
		}
		
		SS3Track newTrack = new SS3Track();
		newTrack.setId(fieldID);
		
		SS3Track track = getTrack(fieldID);

		if (field == null || track == null)
			return newTrack;

		ArrayList<SS3Entity> ss3Entities = field.getSS3Entities();
		ArrayList<String> entIDs = new ArrayList<String>();
		for (SS3Entity ss3Entity: ss3Entities) {
			entIDs.add(ss3Entity.getID());
		}
		for(String entID : entIDs) {
			SS3Entity entity = track.getSS3Entity(entID);
			
			if(entity == null) {
				System.out.println("HERE!!! entID = " + entID + ", fieldID = " + fieldID);
			} else {
			
			if(fieldID.equalsIgnoreCase(Util.FREE_TEXT_FIELD_ID)) {
				newTrack.addEntity(entity);
			} else if(startMovieTime <= entity.getStartTime() && endMovieTime >= entity.getEndTime()) {
				newTrack.addEntity(entity);
			 }
			}
		}

		return newTrack;
	}
	
	public SS3Track getTrack(String fieldID, SS3GlossWindowSegment segmentTier) {
		
		SS3Track newTrack = new SS3Track();
		newTrack.setId(fieldID);
		
		ArrayList<SS3StatementField> statementFields = new ArrayList<SS3StatementField>();
		if (segmentTier.getSS3StatementField(fieldID) != null) {
			statementFields.add(segmentTier.getSS3StatementField(fieldID));
		}

		for (int i = 0; i < statementFields.size(); i++) {
			SS3StatementField statementField = statementFields.get(i);
			
			ArrayList<SS3Entity> ss3Entities = statementField.getSS3Entities();
			
			for (SS3Entity ss3Entity: ss3Entities) {
				if (ss3Entity.getStartTimeInfo() == null || ss3Entity.getEndTimeInfo() == null) {
					continue;
				}
				
				newTrack.addEntity(ss3Entity);
			}
		}
		
		return newTrack;
	}

	public void addTimeRange(TimeRange range) {
        timeRanges.put(range.id(), range);

        var unit = range.unit();
        if (!timeUnits.containsKey(unit.id()))
            timeUnits.put(unit.id(), unit);
	}
    public Collection<TimeRange> getTimeRanges() { return timeRanges.values(); }
    public TimeRange getTimeRange(String id) {
	    if (TimeRange.ABSOLUTE.id().equals(id))
	        return TimeRange.ABSOLUTE;

	    return timeRanges.get(id);
	}

	public void addTimeUnit(TimeUnit unit) {
	    if (!timeUnits.containsKey(unit.id()))
	        timeUnits.put(unit.id(), unit);
    }
    public Collection<TimeUnit> getTimeUnits() { return timeUnits.values(); }
    public TimeUnit getTimeUnit(String id) {
	    if (TimeUnit.NANOSECOND.id().equals(id))
	        return TimeUnit.NANOSECOND;

	    return timeUnits.get(id);
	}

	public ArrayList<Reference> getCodingSchemeRef() {
		return cSchemeRef;
	}

	public void setCodingSchemeRef(ArrayList<Reference> list) {
		cSchemeRef = list;
	}

	public void addVideo(Video v, int order) {
        videos.put(order, v);
    }
	public Video getVideo(URI uri) { 
		for (Video v: videos.values()) {
			if (v.uri().toString().equals(uri.toString()))
				return v;
		}
		return null;
	}
	public List<Video> getVideos() { 
		ArrayList<Video> ret = new ArrayList<>(videos.size());
		for (Integer order: videos.keySet()) {
			ret.add(order - 1, videos.get(order));
		}
		return ret;
	}
	public void clearVideos() { videos.clear(); }

    public void addAudio(Audio a, int order) {
        audios.put(order, a);
    }
    public Audio getAudio(URI uri) { 
		for (Audio a: audios.values()) {
			if (a.uri().toString().equals(uri.toString()))
				return a;
		}
		return null;
    }
    public List<Audio> getAudios() { 
		ArrayList<Audio> ret = new ArrayList<Audio>(audios.size());
		for (Integer order: audios.keySet()) {
			ret.add(order - 1, audios.get(order));
		}
		return ret;
    }
    public void clearAudios() { audios.clear(); }

    public void addGraph(Graph g) {
	    graphs.put(g.id(), g);
	}
    public Graph getGraph(String id) { return graphs.get(id); }
    public List<Graph> getGraphs() { return new ArrayList<>(graphs.values()); }
    public void clearGraphs() { graphs.clear(); }

	public HashMap<String, SS3MediaFile> getMediaFiles() {
		return mFiles;
	}

	public void setMediaFiles(HashMap<String, SS3MediaFile> map) {
		mFiles = map;
	}

	public void addMediaFile(SS3MediaFile mFile) {
		mFiles.put(mFile.getId(), mFile);
	}
	
	public SS3MediaFile getMediaFile(String id) {
		return mFiles.get(id);
	}

	public ArrayList<SS3MediaFile> getMediaFilesCollection() {
		ArrayList<SS3MediaFile> list = new ArrayList<SS3MediaFile>();
		Set<Map.Entry<String, SS3MediaFile>> set = mFiles.entrySet();
        for (Map.Entry<String, SS3MediaFile> entry : set) {
            list.add(entry.getValue());
        }
		return list;
	}
	
	public SS3DBProfile getProfile() {
		return profile;
	}

	public void deleteFile() {
//		var dbWriter = new SS4Writer();
		var dbWriter = new SS3Writer();
		dbWriter.deleteDatabase(this);
	}

	public boolean save() throws Exception {
		this.movieDuration = SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovieDuration();		
		this.packetDuration = SS3Singleton.getVideoControlManager().getMultipleMovieController().getPacketDuration();	
		this.movieTimescale = SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovieTimescale();
//		var dbWriter = new SS4Writer();
        var dbWriter = new SS3Writer();
		return dbWriter.writeDatabase(this);
	}

	public void setSS3CodingScheme(SS3CodingScheme scheme) {
		  if (scheme.getID() == null) {
			  scheme.setID(Util.getLongUniqueNumber());
		  }
		this.scheme = scheme;
	}

	public SS3CodingScheme getSS3CodingScheme() {
		if (scheme == null) {
			scheme = SS3Singleton.getSignStreamApplication().parseDefaultSS3CodingScheme();
			scheme.setID(Util.getLongUniqueNumber());
		}
		return scheme;
	}

	public void setLocalSS3CodingScheme(SS3CodingScheme localScheme) {
		SS3CodingScheme dbScheme = this.getSS3CodingScheme();

		HashMap<String, SS3Field> localFields = localScheme.getFields();
		Set<Map.Entry<String, SS3Field>> set = localFields.entrySet();

		Iterator<Map.Entry<String, SS3Field>> iter = set.iterator();
		while (iter.hasNext()) {
			Map.Entry<String, SS3Field> me = iter.next();
			SS3Field lField = me.getValue();
			String fieldID = lField.getId();
			SS3Field dField = dbScheme.getField(fieldID);				
			if (dField == null) {
				dbScheme.addField(lField);
			} else {
				ArrayList<SS3FieldValue> defaultFieldValues = dField.getValuesAsList();
				ArrayList<SS3FieldValue> values = lField.getValuesAsList();
				ArrayList<SS3FieldValue> deletedFieldValues = new ArrayList<SS3FieldValue>();
				defaultFieldValues.stream()
					.filter(val -> lField.getFieldValue(val.getId()) == null)
					.forEach(val -> deletedFieldValues.add(val));
				
				for (SS3FieldValue value : values) {
					dField.addValue(value);
				}
				
				for (SS3FieldValue value: deletedFieldValues) {
					dField.remove(value);
				}

				dField.setName(lField.getName());
				dField.setLabel(lField.getLabel());

				dField.setGroup(lField.getGroup());
				dField.setType(lField.getType());

				dField.setPrefix(lField.getPrefix());
				dField.setAlignment(lField.getAlignment());
				
				dField.setColor(lField.getColor());
				dField.setOrder(lField.getOrder());
			}
		}
		
		dbScheme.getSortedFields();
	}

	public SS3CodingScheme getLocalSS3CodingScheme() {
		SS3CodingScheme localScheme = new SS3CodingScheme();
		SS3CodingScheme defaultScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		
		SS3CodingScheme currentDBSchema = this.getSS3CodingScheme();
		if (currentDBSchema.getFields().size() > 0) {
			for (SS3Field field: currentDBSchema.getFields().values()) {
				SS3Field defaultField = defaultScheme.getField(field.getId()); 
				if (defaultField == null) {
					localScheme.addField(field);
				} else if (!field.equals(defaultField)) {
					localScheme.addField(field);
				}
			}
		}
		
		return localScheme;
	}

	@Override
	public String toString() {
		return super.toString();
	}

	public String getShortDescription() {
		return "S3Database " + excerpt + " has [" + tracks.size() + "] fields.";
	}

	public String getExcerpt() {
		return excerpt;
	}

	public void setExcerpt(String string) {
		excerpt = string;
	}

	public File getReadFromXMLFile() {
		return readFromXMLFile;
	}

	public void setReadFromXMLFile(File file) {
		readFromXMLFile = file;
	}
	
	@Deprecated
	public String getParticipantName() {
		return participantName;
	}

	@Deprecated
	public void setParticipantName(String string) {
		participantName = string;
	}

	public String getNumber() {
		return number;
	}

	public void setNumber(String string) {
		number = string;
	}

	@Override
	public boolean equals(Object obj) {
		return (obj instanceof SS3Database) &&
				((SS3Database) obj).getDirectory().equals(directory) &&
				((SS3Database) obj).getFileName().equals(fileName);
	}

	public boolean isEmpty() {
		return mFiles.isEmpty() && selectedParticipants.isEmpty() && selectedAnnotators.isEmpty();
	}
}
