package edu.bu.signstream.grepresentation.fields.apexField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

/**
 * The nom-manual fields fall into two categories: anatomical and functional.
 * 
 * Anatomical fields refer to particular physical muscle movement.
 * Functional/interpretive fields provide a functional interpretation for a
 * particular movement or set of movements.
 * */

public class ApexField extends Field {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected ChainedApexEventsEntityCollection collection = new ChainedApexEventsEntityCollection(new ArrayList<ChainedApexEventsEntity>(), segmentPanel);

	public ApexField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		Track track = new Track();
		createTrackEntity(track);
	}

	public ApexField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public ApexField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		ChainedApexEventsEntity cEntity = (ChainedApexEventsEntity) entity;
		cEntity.setField(this);
		collection.addEntity(cEntity);
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		return collection.getEntity(entityID);
	}

	// return ChainedApexEventsEntities
	@Override
	public ArrayList getEntities() {
		return collection.getEntities();
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		return collection.getEntityWithRefID(refEntityId);
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		if (!highlight) {
			ArrayList<ChainedApexEventsEntity> entities = collection.getEntities();
			for (ChainedApexEventsEntity entity : entities) {
				entity.highlight(false);
			}
		} else {
			EventsEntity entity = collection.getEntityWithRefID(refID);
			if (entity != null) {
                entity.highlight(true);
            }
		}
	}

	@Override
	public int getFieldSize() {
		return collection.size();
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
		collection.deleteEntity((ChainedApexEventsEntity)entity);
	}

	public Track getTrack() {
		return track;
	}

	@Override
	public EventsEntity getEntityAt(double x) {
		return collection.getChainedApexEventsEntityAt(x);
	}

	@Override
	public Event getEventAt(double x) {
		return collection.getEventAt(x);
	}

	@Override
	public EventsEntity getSelectedEntity() {
		return collection.getSelectedEntity();
	}

	@Override
	public double getStartTime() {
		if (collection == null)
			return 0;
		return collection.getStartTime();
	}

	@Override
	public double getEndTime() {
		if (collection == null)
			return 0;
		return collection.getEndTime();
	}

	@Override
	public double getStartTimeCoordinate() {
		if (collection == null)
			return 0;
		return collection.getStartTimeCoordinate();
	}

	@Override
	public double getEndTimeCoordinate() {
		if (collection == null)
			return 0;
		return collection.getEndTimeCoordinate();
	}

	@Override
	public void unselect() {
		collection.unselect();
	}

	public void reset() {
		collection.reset();
	}

	public void setNewChainedApexEvent(NewChainedApexEvent event) {
		collection.setNewChainedApexEvent(event);
	}

	public boolean canDrawNewEvent(int x) {
		if (contains(x)) {
			ChainedApexEvent event = collection.getEventAt(x);
			return (event == null);
		}

		return false;
	}

	public void selectEntity(int x) {
		if (contains(x)) {
            collection.selectEntity(x);
        }
	}

	@Override
	public Event getSelectedEvent() {
		return collection.getSelectedEvent();
	}

	public NewChainedApexEvent getNewChainedApexEvent() {
		return collection.getNewChainedApexEvent();
	}

	@Override
	public String getFieldID() {
		if (track != null)
			return track.getFieldId();

		if (ss3Track != null)
			return ss3Track.getId();

		return null;
	}

	private void createTrackEntity(SS3Track track) {
		ArrayList chainedEvents = new ArrayList();
		ArrayList<SS3Entity> entities = track.getEntities();
        for (SS3Entity entity : entities) {
            ChainedApexEventsCollection collection = new ChainedApexEventsCollection();

            ArrayList<SS3Item> items = entity.getItemsAsList();

            for (SS3Item item : items) {
                String id = item.getId();

                if (id.equalsIgnoreCase("" + Constants.APEX_ONSET_CODE)) {
                    collection.setOnsetEvent(new ChainedApexEvent(item, segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.APEX_OFFSET_CODE)) {
                    collection.setOffsetEvent(new ChainedApexEvent(item, segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.APEX_INITIAL_HOLD_CODE)) {
                	collection.setInitialHoldEvent(new ChainedApexEvent(item, segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.APEX_FINAL_HOLD_CODE)) {
                	collection.setFinalHoldEvent(new ChainedApexEvent(item, segmentPanel));
                }
                else {
    				String fieldID = track.getId();
                	if (fieldID.equalsIgnoreCase(Util.DOM_GESTURE_APEX_FIELD_ID)
    						|| fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_APEX_FIELD_ID)) {
    					collection.setTextValueEvent(new ApexChainedApexEvent(item, segmentPanel));
    				} else {
    					collection.setValueEvent(new ChainedApexEvent(item, segmentPanel));
    				}
                }
            }
            
            ApexChainedApexEventsEntity cEntity = new ApexChainedApexEventsEntity(collection, segmentPanel, this);
            cEntity.setID(entity.getID());
            cEntity.setField(this);
            cEntity.setRefEntityId(entity.getRefEntityId());
            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
            	cEntity.updateParentEntity();
            } else {
            	cEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
            }

            boolean isLockedApexPhonAlignments = entity.isLockedApexPhoneAlignments();
            cEntity.setLockedApexPhonAlignments(isLockedApexPhonAlignments);
            chainedEvents.add(cEntity);
        }

		collection = new ChainedApexEventsEntityCollection(chainedEvents, segmentPanel);
	}

	@Deprecated	// legacy code involving Track, not SS3Track
	private void createTrackEntity(Track track) {
//		ArrayList<ChainedApexEventsEntity> chainedEvents = new ArrayList<ChainedApexEventsEntity>();
//		ArrayList trackValues = track.getTrackValues();
//
//		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
//		HashMap<String, TrackValue> onset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> onsetHold = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offsetHold = new HashMap<String, TrackValue>();
//
//        for (Object trackValue1 : trackValues) {
//            TrackValue trackValue = (TrackValue) trackValue1;
//            int valueId = trackValue.getValueId();
//
//            if (valueId == Constants.APEX_HOLD_CODE) {
//               double startFrame = trackValue.getStartTimeInfo().value();
//                double endFrame = trackValue.getEndTimeInfo().value();
//                onsetHold.put("" + endFrame, trackValue);
//                offsetHold.put("" + startFrame, trackValue);
//            } else if (valueId == Constants.APEX_ONSET_CODE) {
//                double endFrame = trackValue.getEndTimeInfo().value();
//                onset.put("" + endFrame, trackValue);
//            } else if (valueId == Constants.APEX_OFFSET_CODE) {
//                double startFrame = trackValue.getStartTimeInfo().value();
//                offset.put("" + startFrame, trackValue);
//            } else {
//                values.add(trackValue);
//            }
//        }
//
//        for (TrackValue trackValue : values) {
//            double startingFrame = trackValue.getStartTimeInfo().value();
//            double endingFrame = trackValue.getEndTimeInfo().value();
//
//            TrackValue onsetHoldValue = onsetHold.get("" + startingFrame);
//            TrackValue offsetHoldValue = offsetHold.get("" + endingFrame);
//            TrackValue onsetValue, offsetValue;
//
//            if (onsetHoldValue == null) {
//                onsetValue = onset.get("" + startingFrame);
//            } else {
//                onsetValue = onset.get(""
//                        + onsetHoldValue.getStartTimeInfo().value());
//            }
//
//            if (offsetHoldValue == null) {
//                offsetValue = onset.get("" + endingFrame);
//            } else {
//                offsetValue = onset.get(""
//                        + offsetHoldValue.getEndTimeInfo().value());
//            }
//
//            ChainedApexEventsCollection collection = new ChainedApexEventsCollection();
//            collection
//                    .setOnsetEvent(new ChainedApexEvent(onsetValue, segmentPanel));
//            collection.setInitialHoldEvent(new ChainedApexEvent(onsetHoldValue,
//                    segmentPanel));
//            collection
//                    .setValueEvent(new ChainedApexEvent(trackValue, segmentPanel));
//            collection.setFinalHoldEvent(new ChainedApexEvent(offsetHoldValue,
//                    segmentPanel));
//            collection.setOffsetEvent(new ChainedApexEvent(offsetValue,
//                    segmentPanel));
//
//            ChainedApexEventsEntity chainedEvent = new ChainedApexEventsEntity(collection, segmentPanel, this);
//            chainedEvent.determineParentEntity();
//
//            chainedEvent.setField(this);
//            chainedEvents.add(chainedEvent);
//        }
//
//		collection = new ChainedApexEventsEntityCollection(chainedEvents,
//				segmentPanel);
	}

	public void addChainedApexEventEntity(ChainedApexEventsEntity entity) {
		entity.setField(this);
		collection.addEntity(entity);
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);
		collection.paint(g2d, Y, font);
	}
	
	public ChainedApexEventsEntity getNextEntity(Time endTimeInfo) {
		return collection.getNextEntity(endTimeInfo);
	}
	
	public ChainedApexEventsEntity getPreviousEntity(Time startTimeInfo) {
		return collection.getPreviousEntity(startTimeInfo);
	}

	@Override
	public void deleteEmptyEntities() {
		collection.removeEmptyEntities();
		
		collection.deleteInvalidEntities(getFieldID());		
	}

	@Override
    public boolean isIntervalEmpty(Interval interval) {
        return collection.getEntitiesInInterval(interval).isEmpty();
	}
}
