package edu.bu.signstream.grepresentation.fields.dictionary;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.dictionary.SS3Dictionary;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.panels.table.sortable.CSFieldTableSorter;
import edu.bu.signstream.ui.panels.table.sortable.SortableTableModel;

public class DictionaryPanel extends JPanel {
	private JTable table;
	private Object[] columnNames = { " Literal Translation " };
	private SortableTableModel tableModel = null;

	public DictionaryPanel() {
		initUI();
	}

	private void initUI() {
		JScrollPane scrollPane = createValuesTable();

		JLabel title = new JLabel(" Dictionary ");
		title.setFont(UserInterfaceUtil.getDialogTitleFont());
		title.setForeground(UserInterfaceUtil.getDialogTitleColor());

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(title, gbc);

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 0, 10, 0);
		add(new JSeparator(), gbc);
		gbc.anchor = GridBagConstraints.WEST;

		add(scrollPane, gbc);
		table.setAutoCreateColumnsFromModel(false);
	}

	private JScrollPane createValuesTable() {
		tableModel =
			new SortableTableModel(columnNames, createTableModelData());

		CSFieldTableSorter sorter = new CSFieldTableSorter(tableModel);

		table = new JTable(sorter);
		sorter.setTableHeader(table.getTableHeader());

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		table
			.getSelectionModel()
			.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {

				if (e.getValueIsAdjusting()) {
					CSFieldTableSorter sorter1 =
						(CSFieldTableSorter) table.getModel();
					sorter1.update(
						table.getSelectedRow(),
						table.getSelectedColumn());
				}

				setPreferredSize();
			}
		});

		setPreferredSize();
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		return scrollPane;
	}

	private Object[][] createTableModelData() {
		Object[][] data = null;

		ArrayList valueList = new ArrayList();

		SS3Dictionary dictionary = SS3Singleton.getManager().getLoadedDictionary();
		ArrayList list = dictionary.getLiteralTranslationsList();

		for (int i = 0; i < list.size(); i++) {
			String translation = (String) list.get(i);

			Object[] values = { " " + translation };

			valueList.add(values);
		}

		data = new Object[valueList.size()][];

		for (int j = 0; j < valueList.size(); j++) {
			data[j] = (Object[]) valueList.get(j);
		}

		return data;
	}

	private void setPreferredSize() {
		TableColumn checkBox = table.getColumn(columnNames[0]);
		int checkBoxPrefSize = getPreferredWidthForColumn(checkBox);
		checkBox.setMinWidth(checkBoxPrefSize);
	}

	public ArrayList getSelectedFieldIDs() {
		ArrayList selectedFieldIDs = new ArrayList();
		ArrayList selectedFields = tableModel.getSelectedFields();

		for (int i = 0; i < selectedFields.size(); i++) {
			String fieldID = (String) selectedFields.get(i);
			selectedFieldIDs.add(fieldID);
		}
		return selectedFieldIDs;
	}

	public int getPreferredWidthForColumn(TableColumn col) {
			int hw = columnHeaderWidth(col), // hw = header width
	cw = widestCellInColumn(col); // cw = column width

		return hw > cw ? hw : cw;
	}

	private int columnHeaderWidth(TableColumn col) {
		TableCellRenderer renderer = col.getHeaderRenderer();
		return 75;
	}

	private int widestCellInColumn(TableColumn col) {
		int c = col.getModelIndex(), width = 0, maxw = 0;

		for (int r = 0; r < table.getRowCount(); ++r) {
			TableCellRenderer renderer = table.getCellRenderer(r, c);

			Component comp =
				renderer.getTableCellRendererComponent(
					table,
					table.getValueAt(r, c),
					false,
					false,
					r,
					c);

			width = comp.getPreferredSize().width;
			maxw = width > maxw ? width : maxw;
		}
		return maxw + 10;
	}
}
