package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.CollectionManager;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ConfirmationPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

/**
 * Manages collection of GestureChainedEventsEntity objects.
 */
public class GestureChainedEventsCollectionManager extends CollectionManager {

	/**
	 * collection of GestureChainedEventsEntity objects to manage
	 */
	private GestureChainedEventsCollection collection = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private CoordinateTimeConverter convertor = null;

	/**
	 * 
	 * @param collection
	 *            must not be null
	 */
	public GestureChainedEventsCollectionManager(GestureChainedEventsCollection collection,
			SignStreamSegmentPanel segmentPanel) {
		this.collection = collection;
		this.segmentPanel = segmentPanel;

		this.convertor = segmentPanel.getZoomer().getCoordinateTimeConvertor();
	}
	
	public void selectGestureChainedEvent(GestureChainedEvent event) {
		if (event != null)
			event.select();
	}

	public void unselectGestureChainedEvent(GestureChainedEvent event) {
		event.unselect();
	}

	@Override
	public void unselectEntity() {
		if (collection.getPreparationEvent() != null)
			collection.getPreparationEvent().unselect();
		if (collection.getPrestrokeHoldEvent() != null)
			collection.getPrestrokeHoldEvent().unselect();
		if (collection.getValueEvent() != null)
			collection.getValueEvent().unselect();
		if (collection.getTextValueEvent() != null)
			collection.getTextValueEvent().unselect();
		if (collection.getPoststrokeHoldEvent() != null)
			collection.getPoststrokeHoldEvent().unselect();
		if (collection.getRetractionEvent() != null)
			collection.getRetractionEvent().unselect();
		if (collection.getRelaxedEvent() != null)
			collection.getRelaxedEvent().unselect();
	}

	@Override
	public Event getSelectedEvent() {
		if (collection.getPreparationEvent() != null
				&& collection.getPreparationEvent().isSelected())
			return collection.getPreparationEvent();
		if (collection.getPrestrokeHoldEvent() != null
				&& collection.getPrestrokeHoldEvent().isSelected())
			return collection.getPrestrokeHoldEvent();
		if (collection.getValueEvent() != null
				&& collection.getValueEvent().isSelected())
			return collection.getValueEvent();
		if (collection.getTextValueEvent() != null
				&& collection.getTextValueEvent().isSelected())
			return collection.getTextValueEvent();
		if (collection.getPoststrokeHoldEvent() != null
				&& collection.getPoststrokeHoldEvent().isSelected())
			return collection.getPoststrokeHoldEvent();
		if (collection.getRetractionEvent() != null
				&& collection.getRetractionEvent().isSelected())
			return collection.getRetractionEvent();
		if (collection.getRelaxedEvent() != null
				&& collection.getRelaxedEvent().isSelected())
			return collection.getRelaxedEvent();
		return null; // throw Exception
	}

//	public void setStartTimeOrRemoveEvent(int time) {
//		Event firstEvent = getFirstEvent();
//		int event1StartTime = firstEvent.getStartTimeInfo().getMovieTime();
//
//		if (time < event1StartTime) {
//			firstEvent.getStartTimeInfo().setMovieTime(time);
//		} else {
//			if (collection.getOnsetEvent() != null) {
//				Event onset = collection.getOnsetEvent();ªª
//				if (onset.getEndTimeInfo().getMovieTime() < time) {
//					collection.setOnsetEvent(null);
//					setStartTimeOrRemoveEvent(time);
//				} else if (onset.getEndTimeInfo().getMovieTime() == time) {
//					collection.setOnsetEvent(null);
//				} else {
//					collection.getOnsetEvent().getStartTimeInfo()
//							.setMovieTime(time);
//				}
//			} else if (collection.getInitialHoldEvent() != null) {
//				Event hold = collection.getInitialHoldEvent();
//				if (hold.getEndTimeInfo().getMovieTime() < time) {
//					collection.setInitialHoldEvent(null);
//					setStartTimeOrRemoveEvent(time);
//				} else if (hold.getEndTimeInfo().getMovieTime() == time) {
//					collection.setInitialHoldEvent(null);
//				} else {
//					collection.getInitialHoldEvent().getStartTimeInfo()
//							.setMovieTime(time);
//				}
//			} else {
//				Event value = collection.getValueEvent();
//				if (value.getEndTimeInfo().getMovieTime() <= time) {
//					;
//				}
//			}
//		}
//	}

//	public void setEndTimeOrRemoveEvent(int time) {
//		Event lastEvent = getLastEvent();
//		int event1EndTime = lastEvent.getStartTimeInfo().getMovieTime();
//
//		if (time > event1EndTime) {
//			lastEvent.getEndTimeInfo().setMovieTime(time);
//		} else {
//			if (collection.getOffsetEvent() != null) {
//				Event offset = collection.getOffsetEvent();
//				if (offset.getStartTimeInfo().getMovieTime() > time) {
//					collection.setOffsetEvent(null);
//					setEndTimeOrRemoveEvent(time);
//				} else if (offset.getStartTimeInfo().getMovieTime() == time) {
//					collection.setOffsetEvent(null);
//				} else {
//					collection.getOffsetEvent().getEndTimeInfo()
//							.setMovieTime(time);
//				}
//			} else if (collection.getFinalHoldEvent() != null) {
//				Event hold = collection.getFinalHoldEvent();
//				if (hold.getStartTimeInfo().getMovieTime() > time) {
//					collection.setFinalHoldEvent(null);
//					setEndTimeOrRemoveEvent(time);
//				} else if (hold.getStartTimeInfo().getMovieTime() == time) {
//					collection.setFinalHoldEvent(null);
//				} else {
//					collection.getFinalHoldEvent().getEndTimeInfo()
//							.setMovieTime(time);
//				}
//			} else {
//				Event value = collection.getValueEvent();
//				if (value.getEndTimeInfo().getMovieTime() <= time) {
//					;
//				}
//			}
//		}
//	}

	// public Event getEventAt(int x) {
	@Override
	public Event getEventAt(double x) {
		deleteUnattachedEvents();

		GestureChainedEvent event = null;
		if ((event = collection.getPreparationEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getPrestrokeHoldEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getValueEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getTextValueEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getPoststrokeHoldEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getRetractionEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getRelaxedEvent()) != null && event.contains(x))
			return event;
		return null;
	}

	/**
	 * @return left existing GestureChainedEvent of the GestureChainedEventsEntity
	 */
	@Override
	public Event getFirstEvent() {

		deleteUnattachedEvents();

		GestureChainedEvent event = null;

		if ((event = collection.getPreparationEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getPrestrokeHoldEvent()) != null
				&& event.isExist())
			return event;

		if ((event = collection.getValueEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getTextValueEvent()) != null && event.isExist())
			return event;
		return null;
	}

	private void deleteUnattachedEvents() {
		GestureChainedEvent txtEvent = collection.getValueOrTextValueEvent();

		if (txtEvent == null)
			return;

		Time txtStartMovieTimeInfo = txtEvent.getStartTimeInfo();
		Time txtEndMovieTimeInfo = txtEvent.getEndTimeInfo();

		if (collection.getPrestrokeHoldEvent() != null) {
			Time ihEndMovieTimeInfo = collection.getPrestrokeHoldEvent()
					.getEndTimeInfo();
			if (! ihEndMovieTimeInfo.equals(txtStartMovieTimeInfo)) {
				collection.setPrestrokeHoldEvent(null);
				collection.setPreparationEvent(null);
			} else if (collection.getPreparationEvent() != null) {
				Time ihStartMovieTimeInfo = collection.getPrestrokeHoldEvent()
						.getStartTimeInfo();
				Time onsetEndMovieTimeInfo = collection.getPreparationEvent()
						.getEndTimeInfo();
				if (! ihStartMovieTimeInfo.equals(onsetEndMovieTimeInfo)) {
					collection.setPreparationEvent(null);
				}
			}
		} else if (collection.getPreparationEvent() != null) {
			Time onsetEndMovieTimeInfo = collection.getPreparationEvent().getEndTimeInfo();
			if (! onsetEndMovieTimeInfo.equals(txtStartMovieTimeInfo)) {
				collection.setPreparationEvent(null);
			}
		}

		if (collection.getPoststrokeHoldEvent() != null) {
			Time fhStartMovieTimeInfo = collection.getPoststrokeHoldEvent()
					.getStartTimeInfo();
			if (! txtEndMovieTimeInfo.equals(fhStartMovieTimeInfo)) {
				collection.setPoststrokeHoldEvent(null);
				collection.setRetractionEvent(null);
			} else {
				Time offsetStartMovieTimeInfo = collection.getPoststrokeHoldEvent()
						.getStartTimeInfo();
				if (! txtEndMovieTimeInfo.equals(offsetStartMovieTimeInfo)) {
					collection.setRetractionEvent(null);
				}
			}
		} else if (collection.getRetractionEvent() != null) {
			Time offsetStartMovieTimeInfo = collection.getRetractionEvent()
					.getStartTimeInfo();
			if (! txtEndMovieTimeInfo.equals(offsetStartMovieTimeInfo)) {
				collection.setRetractionEvent(null);
			}
		}
	}

	/**
	 * @return right existing GestureChainedEvent of the GestureChainedEventsEntity
	 */
	@Override
	public Event getLastEvent() {
		deleteUnattachedEvents();
		GestureChainedEvent event = null;

		// clean entity - sometimes it may have wrong HOLD transfered from
		// Legacy file
		
		if ((event = collection.getRelaxedEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getRetractionEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getPoststrokeHoldEvent()) != null && event.isExist()) {
			if (collection.getValueOrTextValueEvent().getEndTimeInfo()
					.doubleValue() != collection.getPoststrokeHoldEvent()
					.getStartTimeInfo().doubleValue()) {
				collection.setPoststrokeHoldEvent(null);
			} else {
				return event;
			}
		}

		if ((event = collection.getValueEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getTextValueEvent()) != null && event.isExist())
			return event;

		return null;
	}

	/**
	 * delete entire entity
	 */
	public boolean deleteEntity() {
		collection.setPreparationEvent(null);
		collection.setPrestrokeHoldEvent(null);
		collection.setValueEvent(null);
		collection.setTextValueEvent(null);
		collection.setPoststrokeHoldEvent(null);
		collection.setRetractionEvent(null);
		collection.setRelaxedEvent(null);
		return true;
	}

	/**
	 * @param event
	 *            GestureChainedEvent to be deleted
	 */
	public void deleteGestureChainedEvent(GestureChainedEvent event) {
		// if(event != null)
		// event.delete();
		event = null;
	}

	public GestureChainedEventsCollection getGestureChainedEventsCollection() {
		return collection;
	}

	@Override
	public void resetAll() {
		GestureChainedEvent event;
		if ((event = collection.getPreparationEvent()) != null)
			event.reset();
		if ((event = collection.getPrestrokeHoldEvent()) != null)
			event.reset();
		if ((event = collection.getValueEvent()) != null)
			event.reset();
		if ((event = collection.getTextValueEvent()) != null)
			event.reset();
		if ((event = collection.getPoststrokeHoldEvent()) != null)
			event.reset();
		if ((event = collection.getRetractionEvent()) != null)
			event.reset();
		if ((event = collection.getRelaxedEvent()) != null)
			event.reset();
	}

	public boolean isAnyEventSelected() {
		return getSelectedEvent() != null ? true : false;
	}

	@Override
	public boolean adjustEntity(double startTime, double endTime) {
		GestureChainedEvent onset = collection.getPreparationEvent();
		GestureChainedEvent offset = collection.getRetractionEvent();
		GestureChainedEvent finalHold = collection.getPoststrokeHoldEvent();
		GestureChainedEvent value = collection.getValueOrTextValueEvent();

        var startTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(startTime), TimeUnit.TIMESTAMP);
		var endTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(endTime), TimeUnit.TIMESTAMP);

		if (value == null)
			return false;

		if (startTime < value.getStartTimeInfo().doubleValue()
				&& endTime > value.getEndTimeInfo().doubleValue()) {
			deleteEntity();
			return true;
		} else if (startTime < value.getStartTimeInfo().doubleValue()
				&& endTime < value.getEndTimeInfo().doubleValue()) {
			deleteOnsetEvent();
			deletePrestrokeHoldEvent();
			collection.getValueEvent().setStartTimeInfo(endTimeInfo);
			return true;
		} else if (endTime < value.getEndTimeInfo().doubleValue()
				&& endTime > value.getStartTimeInfo().doubleValue()) {
			deletePoststrokeHoldEvent();
			deleteRetractionEvent();
            collection.getValueEvent().setEndTimeInfo(endTimeInfo);
			return true;
		} else if (offset != null
				&& startTime > offset.getStartTimeInfo().doubleValue()) {
			deleteRetractionEvent();
            collection.getPrestrokeHoldEvent().setStartTimeInfo(startTimeInfo);
			return true;
		} else if (onset != null
				&& startTime > onset.getStartTimeInfo().doubleValue()) {
            collection.getPreparationEvent().setStartTimeInfo(startTimeInfo);
			return true;
		} else if (finalHold != null
				&& endTime < finalHold.getEndTimeInfo().doubleValue()) {
			deletePoststrokeHoldEvent();
            collection.getPoststrokeHoldEvent().setStartTimeInfo(endTimeInfo);
			return true;
		} else if (offset != null
				&& endTime < onset.getEndTimeInfo().doubleValue()) {
            collection.getRetractionEvent().setStartTimeInfo(endTimeInfo);
			return true;
		}
		return false;
	}

	public boolean deleteOnsetEvent() {
		String message = "<HTML>This action will delete ONSET value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection.setPreparationEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deletePreparationEventNoWarning() {
		collection.setPreparationEvent(null);
		return true;
	}

	public boolean deleteRetractionEventNoWarning() {
		collection.setRetractionEvent(null);
		if (collection.getRelaxedEvent() != null) {
			Event prevEvent = collection.getPoststrokeHoldEvent();
			if (prevEvent == null) {
				prevEvent = collection.getValueOrTextValueEvent();
			}
			collection.getRelaxedEvent().setStartTimeInfo(
				prevEvent.getEndTimeInfo());
		}
		return true;
	}
	
	public boolean deleteRelaxedEventNoWarning() {
		collection.setRelaxedEvent(null);
		return true;
	}

	public boolean deletePoststrokeHoldEventNoWarning() {
		collection.setPoststrokeHoldEvent(null);

		Event value = collection.getValueOrTextValueEvent();
		if (collection.getRetractionEvent() != null) {			
			collection.getRetractionEvent().setStartTimeInfo(value.getEndTimeInfo());
		} else if (collection.getRelaxedEvent() != null) {
			collection.getRelaxedEvent().setStartTimeInfo(value.getEndTimeInfo());
		}
		return true;
	}

	public boolean deletePrestrokeHoldEventNoWarning() {
		collection.setPrestrokeHoldEvent(null);

		if (collection.getPreparationEvent() != null) {
			Event value = collection.getValueOrTextValueEvent();
			collection.getPreparationEvent().setEndTimeInfo(value.getStartTimeInfo());
		}
		return true;
	}

	public boolean deletePrestrokeHoldEvent() {
		String message = "<HTML>This action will delete INITIAL HOLD value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection
						.getValueOrTextValueEvent()
						.setStartTimeInfo(collection.getPrestrokeHoldEvent().getStartTimeInfo());
				deletePrestrokeHoldEventNoWarning();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteValueEvent() {
		String message = "<HTML>This action will delete selected event. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				deleteEntity();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteTextValueEvent() {
		String message = "<HTML>This action will delete selected event. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				deleteEntity();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deletePoststrokeHoldEvent() {
		String message = "<HTML>This action will delete FINAL HOLD value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (collection.getRetractionEvent() != null)
					collection
							.getRetractionEvent()
                            .setStartTimeInfo(collection.getPoststrokeHoldEvent().getStartTimeInfo());
				// collection.getFinalHoldEvent().delete();
				collection.setPoststrokeHoldEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteRetractionEvent() {
		String message = "<HTML>This action will delete OFFSET value. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection.setRetractionEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

//	public boolean setInitialHoldStartMovieTimeNoWarning(int movieTime) {
//		GestureChainedEvent initialHold = collection.getInitialHoldEvent();
//		GestureChainedEvent onset = collection.getOnsetEvent();
//
//		if (initialHold == null)
//			return false;
//
//		int maxHoldStartMovieTime = convertor
//				.getPreviousFrameMovieTime(initialHold.getEndTimeInfo()
//						.getMovieTime());
//
//		if (movieTime <= maxHoldStartMovieTime) {
//			if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//
//				if (movieTime >= onsetMinEndMovieTime) {
//					onset.getEndTimeInfo().setMovieTime(movieTime);
//					initialHold.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					onset.getEndTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					initialHold.getStartTimeInfo().setMovieTime(
//							onsetMinEndMovieTime);
//					// collection.setOnsetEvent(null);
//					// initialHold.setStartTimeCoordinate(coord);
//					return true;
//				}
//			} else {
//				initialHold.getStartTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			initialHold.getStartTimeInfo().setMovieTime(maxHoldStartMovieTime);
//			// collection.setInitialHoldEvent(null);
//			if (onset != null)
//				onset.getEndTimeInfo().setMovieTime(
//						collection.getValueOrTextValueEvent()
//								.getStartTimeInfo().getMovieTime());
//			return true;
//		}
//	}
//
//	public boolean setValueStartMovieTimeNoWarning(int movieTime) {
//		GestureChainedEvent onset = collection.getOnsetEvent();
//		GestureChainedEvent initialHold = collection.getInitialHoldEvent();
//		GestureChainedEvent value = collection.getValueOrTextValueEvent();
//
//		if (value == null)
//			return false;
//
//		int valueMaxStartMovieTime = convertor.getPreviousFrameMovieTime(value
//				.getEndTimeInfo().getMovieTime());
//
//		if (movieTime <= valueMaxStartMovieTime) {
//			if (initialHold != null) {
//				int holdMinEndMovieTime = convertor
//						.getNextFrameMovieTime(initialHold.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= holdMinEndMovieTime) {
//					initialHold.getEndTimeInfo().setMovieTime(movieTime);
//					value.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					// collection.setInitialHoldEvent(null);
//					// value.setStartTimeCoordinate(coord);
//					initialHold.getEndTimeInfo().setMovieTime(
//							holdMinEndMovieTime);
//					value.getStartTimeInfo().setMovieTime(holdMinEndMovieTime);
//					return true;
//				}
//			} else if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime > onsetMinEndMovieTime) {
//					onset.getEndTimeInfo().setMovieTime(movieTime);
//					value.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					onset.getEndTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					value.getStartTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					// collection.setOnsetEvent(null);
//					// value.setStartTimeCoordinate(coord);
//					return true;
//				}
//			} else {
//				value.getStartTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			return deleteEntity();
//		}
//	}
//
//	public boolean setOnsetStartMovieTimeNoWarning(int movieTime) {
//		GestureChainedEvent onset = collection.getOnsetEvent();
//
//		if (movieTime > onset.getStartTimeInfo().getMovieTime()) {
//
//			int maxStartMovieTime = convertor.getPreviousFrameMovieTime(onset
//					.getEndTimeInfo().getMovieTime());
//
//			if (movieTime <= maxStartMovieTime) {
//				onset.getStartTimeInfo().setMovieTime(movieTime);
//			} else {
//				onset.getStartTimeInfo().setMovieTime(maxStartMovieTime);
//			}
//		} else {
//			onset.getStartTimeInfo().setMovieTime(movieTime);
//		}
//		return true;
//	}
//
//	public Response checkOnsetStartMovieTimeUpdatePossibility(int movieTime) {
//		Response response = new Response();
//		GestureChainedEvent onset = collection.getOnsetEvent();
//
//		response.setUpdatePossible(true);
//		response.setMovieTime(movieTime);
//		response.setState(Response.OK);
//
//		if (movieTime > onset.getStartTimeInfo().getMovieTime()) {
//			int maxStartTime = convertor.getPreviousFrameMovieTime(onset
//					.getEndTimeInfo().getMovieTime());
//
//			if (movieTime > maxStartTime) {
//				response.setUpdatePossible(true);
//				response.setState(Response.HOLD_DELETE);
//				response.setErrorMessage("Onset event will be deleted.");
//				response.setMovieTime(maxStartTime);
//			}
//		}
//
//		return response;
//	}
//
//	public Response checkValueStartMovieTimeUpdatePossibility(int movieTime) {
//		GestureChainedEvent onset = collection.getOnsetEvent();
//		GestureChainedEvent initialHold = collection.getInitialHoldEvent();
//		GestureChainedEvent value = collection.getValueOrTextValueEvent();
//
//		Response response = new Response();
//		response.setState(Response.OK);
//		response.setUpdatePossible(true);
//		response.setMovieTime(movieTime);
//
//		int valueMaxStartMovieTime = convertor.getPreviousFrameMovieTime(value
//				.getEndTimeInfo().getMovieTime());
//
//		if (movieTime <= valueMaxStartMovieTime) {
//			if (initialHold != null) {
//				int holdMinEndMovieTime = convertor
//						.getNextFrameMovieTime(initialHold.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= holdMinEndMovieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Initial Hold will be deleted");
//					response.setMovieTime(holdMinEndMovieTime);
//				}
//			} else if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime > onsetMinEndMovieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Onset will be deleted");
//					response.setMovieTime(onsetMinEndMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setState(Response.ENTITY_DELETE);
//			response.setErrorMessage("Entity will be deleted");
//			response.setMovieTime(valueMaxStartMovieTime);
//		}
//		return response;
//	}
//
//	public Response checkValueEndMovieTimeUpdatePossibility(int movieTime) {
//		GestureChainedEvent offset = collection.getOffsetEvent();
//		GestureChainedEvent finalHold = collection.getFinalHoldEvent();
//		GestureChainedEvent value = collection.getValueOrTextValueEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int valueMinEndMovieTime = convertor.getNextFrameMovieTime(value
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= valueMinEndMovieTime) {
//			if (finalHold != null) {
//				int maxHoldStartMovieTime = convertor
//						.getPreviousFrameMovieTime(finalHold.endTimeInfo
//								.getMovieTime());
//
//				if (maxHoldStartMovieTime >= movieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Final Hold will be deleted");
//					response.setMovieTime(maxHoldStartMovieTime);
//				}
//
//			} else if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Offset will be deleted");
//					response.setMovieTime(maxOffsetStartMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setState(Response.ENTITY_DELETE);
//			response.setErrorMessage("Entity will be deleted");
//			response.setMovieTime(valueMinEndMovieTime);
//		}
//		return response;
//	}
//
//	public boolean setOffsetEndMovieTimeNoWarning(int movieTime) {
//		GestureChainedEvent offset = collection.getOffsetEvent();
//
//		int offsetMinEndMovieTime = convertor.getNextFrameMovieTime(offset
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= offsetMinEndMovieTime) {
//			offset.getEndTimeInfo().setMovieTime(movieTime);
//			return true;
//		} else {
//			offset.getEndTimeInfo().setMovieTime(offsetMinEndMovieTime);
//			// collection.setOffsetEvent(null);
//			return false;
//		}
//	}

//	public Response checkOffsetEndMovieTimeUpdatePossibility(int movieTime) {
//		GestureChainedEvent offset = collection.getOffsetEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int offsetMinEndMovieTime = convertor.getNextFrameMovieTime(offset
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime <= offsetMinEndMovieTime) {
//			response.setErrorMessage("Offset will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(offsetMinEndMovieTime);
//		}
//
//		return response;
//	}
//
//	public Response checkInitialHoldStartMovieTimeUpdatePossibility(
//			int movieTime) {
//		GestureChainedEvent onset = collection.getOnsetEvent();
//		GestureChainedEvent initialHold = collection.getInitialHoldEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int maxHoldStartMovieTime = convertor
//				.getPreviousFrameMovieTime(initialHold.getEndTimeInfo()
//						.getMovieTime());
//
//		if (movieTime < maxHoldStartMovieTime) {
//			if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= onsetMinEndMovieTime) {
//					;
//				} else {
//					response.setErrorMessage("Onset will be deleted");
//					response.setState(Response.HOLD_DELETE);
//					response.setMovieTime(onsetMinEndMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setErrorMessage("Initial Hold will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(maxHoldStartMovieTime);
//		}
//
//		return response;
//	}
//
//	public Response checkFinalHoldEndMovieTimeUpdatePossibility(int movieTime) {
//		GestureChainedEvent offset = collection.getOffsetEvent();
//		GestureChainedEvent finalHold = collection.getFinalHoldEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int holdMinEndMovieTime = convertor.getNextFrameMovieTime(finalHold
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime > holdMinEndMovieTime) {
//			if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					;
//				} else {
//					response.setErrorMessage("Offset will be deleted");
//					response.setState(Response.HOLD_DELETE);
//					// response.setMovieTime();
//				}
//			}
//		} else {
//			response.setErrorMessage("Final Hold will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(holdMinEndMovieTime);
//		}
//
//		return response;
//	}
//
//	public boolean setFinalHoldEndMovieTimeNoWarning(int movieTime) {
//		GestureChainedEvent finalHold = collection.getFinalHoldEvent();
//
//		if (finalHold == null)
//			return true;
//
//		GestureChainedEvent offset = collection.getOffsetEvent();
//
//		int holdMinEndMovieTime = convertor.getNextFrameMovieTime(finalHold
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= holdMinEndMovieTime) {
//			if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					finalHold.getEndTimeInfo().setMovieTime(movieTime);
//					offset.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					finalHold.getEndTimeInfo().setMovieTime(
//							maxOffsetStartMovieTime);
//					offset.getStartTimeInfo().setMovieTime(
//							maxOffsetStartMovieTime);
//					return true;
//				}
//			} else {
//				finalHold.getEndTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			finalHold.getEndTimeInfo().setTimeBase(holdMinEndMovieTime);
//			return true;
//		}
//	}
//
//	public boolean setValueEndMovieTimeNoWarning(int movieTime) {
//		GestureChainedEvent offset = collection.getOffsetEvent();
//		GestureChainedEvent finalHold = collection.getFinalHoldEvent();
//		GestureChainedEvent value = collection.getValueOrTextValueEvent();
//
//		if (value == null)
//			return false;
//
//		int valueMinEndMovieTime = convertor.getNextFrameMovieTime(value
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= valueMinEndMovieTime) {
//			if (finalHold != null) {
//				int maxHoldStartMovieTime = convertor
//						.getPreviousFrameMovieTime(finalHold.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxHoldStartMovieTime >= movieTime) {
//					value.getEndTimeInfo().setMovieTime(movieTime);
//					finalHold.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					value.getEndTimeInfo().setMovieTime(maxHoldStartMovieTime);
//					finalHold.getStartTimeInfo().setMovieTime(
//							maxHoldStartMovieTime);
//					return true;
//				}
//
//			} else if (offset != null) {
//
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					value.getEndTimeInfo().setMovieTime(movieTime);
//					offset.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					try {
//						finalHold.getStartTimeInfo().setMovieTime(
//								maxOffsetStartMovieTime);
//						value.getEndTimeInfo().setMovieTime(
//								maxOffsetStartMovieTime);
//					} catch (NullPointerException ex) {
//						ex.printStackTrace();
//					}
//					return true;
//				}
//			} else {
//				value.getEndTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			return deleteEntity();
//		}
//	}

	// add onset, one frame long approximately 1/30 second
	public void addPreparationEvent() {
		GestureChainedEvent event = collection.getPrestrokeHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GESTURE_PREPARATION_CODE;
			
			Time endTimeInfo = event.getStartTimeInfo();
			Time startTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(endTimeInfo);

			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setPreparationEvent(new GestureChainedEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! GestureChainedEventsCollectionManager");
		}
	}

	public void addRetractionEvent() {
		GestureChainedEvent relaxed = collection.getRelaxedEvent();
		GestureChainedEvent event = collection.getPoststrokeHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GESTURE_RETRACTION_CODE;
			
			Time startTimeInfo = event.getEndTimeInfo();
			Time endTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(startTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setRetractionEvent(new GestureChainedEvent(newValue, segmentPanel));
			if (relaxed != null) {
				Time relaxedEndTimeInfo = relaxed.getEndTimeInfo();
				Time relaxedStartTimeInfo = relaxed.getStartTimeInfo();
				Time newRelaxedStartTimeInfo
					= segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(relaxedStartTimeInfo);
				relaxed.setStartTimeInfo(newRelaxedStartTimeInfo);
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! GestureChainedEventsCollectionManager");
		}
	}
	
	public void addRelaxedEvent() {
		GestureChainedEvent event = collection.getRetractionEvent();
		if (event == null)
			event = collection.getPoststrokeHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GESTURE_RELAXED_CODE;
			Time eventStartTimeInfo = event.getEndTimeInfo();
			Time eventEndTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(eventStartTimeInfo);
			TrackValue newValue = new TrackValue(id, "", eventStartTimeInfo, eventEndTimeInfo);
			collection.setRelaxedEvent(new GestureChainedEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! GestureChainedEventsCollectionManager");
		}
	}

	public void addPrestrokeHoldEvent() {
		GestureChainedEvent onset = collection.getPreparationEvent();
		GestureChainedEvent event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GESTURE_PRESTROKE_HOLD_CODE;
			
			Time endTimeInfo = event.getStartTimeInfo();
			Time startTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(endTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setPrestrokeHoldEvent(new GestureChainedEvent(newValue, segmentPanel));
			
			if (onset != null) {
			    onset.setEndTimeInfo(startTimeInfo);
				// onset.getEndTimeInfo().setMovieTime(onset.getEndTimeInfo().getMovieTime()-minFrameDurationToAdd);
			}
		} catch (NullPointerException ex) {
			Logger logger = SS3SignStreamApplication.getLogger();
			logger.log(Level.WARNING, String.format("NullPointerException in GestureChainedEventsCollectionManager.java! %s", ex), ex);
		}
	}

	public void addPrestrokeHoldEvent(GestureChainedEvent hold) {
		collection.setPrestrokeHoldEvent(hold);
	}

	public void addPoststrokeHoldEvent(GestureChainedEvent hold) {
		collection.setPoststrokeHoldEvent(hold);
	}

	public void addPoststrokeHoldEvent() {
		GestureChainedEvent offset = collection.getRetractionEvent();
		GestureChainedEvent event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GESTURE_POSTSTROKE_HOLD_CODE;
			
			Time startTimeInfo = event.getEndTimeInfo();
			Time endTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(startTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setPoststrokeHoldEvent(new GestureChainedEvent(newValue, segmentPanel));

			if (offset != null) {
				offset.setStartTimeInfo(endTimeInfo);
			}
		} catch (NullPointerException ex) {
			Logger logger = SS3SignStreamApplication.getLogger();
			logger.log(Level.WARNING, "Not valid argument Exception will be thrown!!! GestureChainedEventsCollectionManager", ex);
		}
	}
}
