package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.logging.Level;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

/**
 * Non-manual fields fall into two categories: anatomical and functional.
 * 
 * Anatomical fields refer to particular physical muscle movement.
 * Functional/interpretive fields provide a functional interpretation for a
 * particular movement or set of movements.
 * */

public class GestureField extends NonManualField { // implements ActionListener {

	private static final long serialVersionUID = 1L;

	int selectedCompoundStartCoord = -1;
	int selectedCompoundEndCoord = -1;

	private NewGestureEvent newGestureEvent;
	private GestureEventsEntityCollection collection = null;

	public GestureField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
	}

	public GestureField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public GestureField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	public GestureEventsEntityCollection getGestureEventsEntityCollection() {
		return collection;
	}

	@Override
	public int getFieldSize() {
		return collection.size();
	}
	
	public double getPrevEntityOrTextStartCoord(double coord) {
		return collection.getPrevEntityOrTextStartCoord(coord);
	}
	
	public double getNextEntityOrTextStartCoord(double coord) {
		return collection.getNextEntityOrTextStartCoord(coord);
	}

    public ArrayList<GestureEventsEntity> getEntitiesInInterval(Interval interval) {
	    return collection.getEntitiesInInterval(interval);
    }

    public ArrayList<GestureEventsEntity> getEntitiesInInterval(Interval interval, Bound bound) {
	    return collection.getEntitiesInInterval(interval, bound);
    }
	
	public GestureEventsEntity getNextGestureEntity(Time endTimeInfo) {
		return collection.getNextEntity(endTimeInfo);
	}
	
	public GestureEventsEntity getPreviousGestureEntity(Time startTimeInfo) {
		return collection.getPreviousEntity(startTimeInfo);
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
		GestureEventsEntity gestEntity = (GestureEventsEntity)entity;
		collection.deleteEntity(gestEntity);
	}
	
	public void setGestureEventsEntityCollection(GestureEventsEntityCollection collection) {
		this.collection = collection;
	}
	
	public void setSelectCompoundStartCoord(int selectedCompoundStartCoord) {
		this.selectedCompoundStartCoord = selectedCompoundStartCoord;
	
		GestureEnteredText enteredText = null;

		try {
			enteredText = getGestureEventsEntityCollection().getEnteredText();
			enteredText.unselectAll();
		} catch (NullPointerException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		}
	}

	public void setSelectCompoundEndCoord(int selectedCompoundEndCoord) {
		this.selectedCompoundEndCoord = selectedCompoundEndCoord;
	}

	public int getSelectedCompoundStartCoord() {
		return selectedCompoundStartCoord;
	}

	public int getSelectedCompoundEndCoord() {
		return selectedCompoundEndCoord;
	}

	@Override
	public EventsEntity getSelectedEntity() {
		return collection.getSelectedEntity();
	}

	public NewGestureEvent getNewGestureEvent() {
		return newGestureEvent;
	}

	public void setNewGestureEvent(NewGestureEvent newGestureEvent) {
		this.newGestureEvent = newGestureEvent;
	}

	public ArrayList<GestureCompoundable> getEventsToCreateCompound() {
		
		if (selectedCompoundStartCoord == -1 || selectedCompoundEndCoord == -1)
			return null;

		// JACK TODO: This is always an empty list... leaving so we can find usages of the function.
		ArrayList<NotCompleteGestureEvent> nonComplEventsInInterval = collection
				.getNotCompleteGestureEventsInInterval(
						selectedCompoundStartCoord, selectedCompoundEndCoord);

        var converter = SS3Singleton.getVideoControlManager().getZoomer().getCoordinateTimeConvertor();
        var start = converter.convertCoordinateToTimeInfo(selectedCompoundStartCoord);
        var end = converter.convertCoordinateToTimeInfo(selectedCompoundEndCoord);
        var interval = new Interval(start, end);

        ArrayList<GestureEventsEntity> entitiesInInterval = collection.getEntitiesInInterval(interval, Bound.INCLUSIVE);
		collection.sort(entitiesInInterval);

		return new ArrayList<GestureCompoundable>(entitiesInInterval);

        // JACK TODO: None of this even runs since size() is always zero... commenting for now.
//		ArrayList<Compoundable_XYZ> compounds = new ArrayList<Compoundable_XYZ>();
//
//		collection.sort(entitiesInInterval);
//
//		for(GestureEventsEntity entity : entitiesInInterval) {
//			compounds.add(entity);
//		}
//
//		if (nonComplEventsInInterval.size() == 0) {
//			return compounds;
//		} else if (nonComplEventsInInterval.size() > 2) {
//			return null;
//
//		} else {
//			NotCompleteGestureEvent presNotComplEvent = null;
//			NotCompleteGestureEvent posNotComplEvent = null;
//
//			NotCompleteGestureEvent nCmlGlsEvent1 = nonComplEventsInInterval.get(0);
//
//			if (nCmlGlsEvent1.isEndMovieTimeFinal()) {
//				presNotComplEvent = nCmlGlsEvent1;
//			} else {
//				posNotComplEvent = nCmlGlsEvent1;
//			}
//
//			if (nonComplEventsInInterval.size() == 2) {
//				NotCompleteGestureEvent nCmlGlsEvent2 = nonComplEventsInInterval
//						.get(1);
//				if (presNotComplEvent != null
//						&& nCmlGlsEvent2.isStartMovieTimeFinal()
//						&& presNotComplEvent.getEndTimeInfo().doubleValue() <= nCmlGlsEvent2
//								.getStartTimeInfo().doubleValue()) {
//					posNotComplEvent = nCmlGlsEvent2;
//				} else if (posNotComplEvent != null
//						&& nCmlGlsEvent2.isEndMovieTimeFinal()
//						&& nCmlGlsEvent2.getEndTimeInfo().doubleValue() <= posNotComplEvent
//								.getStartTimeInfo().doubleValue()) {
//					presNotComplEvent = nCmlGlsEvent2;
//				}
//			}
//
//			if (compounds.size() == 0) {
//				if (presNotComplEvent != null)
//					compounds.add(presNotComplEvent);
//				if (posNotComplEvent != null)
//					compounds.add(posNotComplEvent);
//			} else {
//				Compoundable_XYZ strEntity = compounds.get(0);
//				Compoundable_XYZ endEntity = compounds.get(compounds.size() - 1);
//				if (presNotComplEvent != null
//						&& strEntity.getStartTimeInfo().doubleValue() >= presNotComplEvent
//								.getEndTimeInfo().doubleValue()) {
//					compounds.add(0, presNotComplEvent);
//				}
//				if (posNotComplEvent != null
//						&& endEntity.getEndTimeInfo().doubleValue() <= posNotComplEvent
//								.getStartTimeInfo().doubleValue()) {
//					compounds.add(compounds.size(), posNotComplEvent);
//				}
//			}
//			return compounds;
//		}
	}

	@Override
	public void deleteEmptyEntities() {
		ArrayList<GestureEventsEntity> entities = collection.getEntities();
		ArrayList<GestureEventsEntity> entitiesToDelete = new ArrayList<GestureEventsEntity>();
		for(GestureEventsEntity entity : entities) {
			if(entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null || entity.getValueOrTextValueEvent() == null) {
				entitiesToDelete.add(entity);
			}
		}
		
		for(GestureEventsEntity toDelete : entitiesToDelete) {
			collection.deleteEntity(toDelete);
		}
	//	collection.deleteEmptyEntities();
	//	collection.updateGestureEntities();
	}

	@Override
	public ArrayList getEntities() {
		return collection.getEntities();
	}
	
	
	public void addGestureEventsEntity(GestureEventsEntity entity) {
		collection.addEntity(entity);	
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		return collection.getEntity(entityID);
	}

	@Override
	public Event getEventAt(double x) {
		return collection.getEventAt(x);
	}
	
	@Override
	public EventsEntity getEntityAt(double x) {
		return collection.getGestureEventsEntityAt(x);
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		return collection.getEntityWithRefID(refEntityId);
	}

	@Override
	public Track getTrack() {
		return track;
	}

	@Override
	protected void createTrackEntity(SS3Track track) {
		ArrayList<GestureEventsEntity> chainedEvents = new ArrayList<>();
		ArrayList<SS3Entity> entities = track.getEntities();

		ArrayList<SS3Item> txtOnly = new ArrayList<SS3Item>();
	 
		for (SS3Entity entity : entities) {
			GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
			
			ArrayList<SS3Item> items = entity.getItemsAsList();
			for (SS3Item item : items) {
				String id = item.getId();

				if ((track.getId().equals(Util.DOM_GESTURE_FIELD_ID) ||
						track.getId().equals(Util.NDOM_GESTURE_FIELD_ID)) && item.isTextOnly()) {
					txtOnly.add(item);
				} else {
					if (id.equalsIgnoreCase("" + Constants.GESTURE_PREPARATION_CODE)) {
						collection.setPreparationEvent(new GestureChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GESTURE_RETRACTION_CODE)) {
						collection.setRetractionEvent(new GestureChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GESTURE_PRESTROKE_HOLD_CODE)) {
						collection.setPrestrokeHoldEvent(new GestureChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GESTURE_POSTSTROKE_HOLD_CODE)) {
						collection.setPoststrokeHoldEvent(new GestureChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GESTURE_RELAXED_CODE)) {
						collection.setRelaxedEvent(new GestureChainedEvent(item, segmentPanel));
					} else {
						collection.setTextValueEvent(new GestureEvent(item, segmentPanel));
					}
				}
			}
			
			// means its a chained entity and not a text-only
			if (collection.getTextValueEvent() != null) {
				GestureEventsEntity chainedEventEntity = new GestureEventsEntity(collection, segmentPanel, this);

				chainedEventEntity.setGestureEntityProperties(entity.getGestureEntityProperties());
				chainedEventEntity.setCorrespondingGestureEntityID(entity.getDependentGestureEntityID());
				
				String dependentGestureEntityID = entity.getDependentGestureEntityID();
				String dependentApexEntityID = entity.getDependentApexEntityID();
				String dependentHandEntityID = entity.getDependentHandEntityID();
				String dependentHandShapeEntityID = entity.getDependentHandShapeEntityID();
				String dependentLocationEntityID = entity.getDependentLocationEntityID();
				String dependentSignTypeEntityID = entity.getDependentSignTypeEntityID();
				boolean isLockedGesturePhonAlignments = entity.isLockedGesturePhoneAlignments();
				boolean isLockedDomNdomAlignments = entity.isLockedDomNdomAlignments();
				boolean isIncompleteGesture = entity.isIncompleteGesture();
				boolean showUniqueNumber = entity.getShowUniqueNumber();
				String uniqueNumber = entity.getUniqueNumber();

				int currentGestureCounter = this.getParentField().getSS3GlossWindowSegment().getCurrentGestureCounter();
				int currentTrackCounter = Integer.parseInt(uniqueNumber);

				this.getParentField().getSS3GlossWindowSegment().setCurrentGestureCounter(Math.max(currentGestureCounter, currentTrackCounter + 1));

				chainedEventEntity.setDependentGestureEntityID(dependentGestureEntityID);
				chainedEventEntity.setDependentApexEntityID(dependentApexEntityID);
				chainedEventEntity.setDependentHandEntityID(dependentHandEntityID);
				chainedEventEntity.setDependentHandShapeEntityID(dependentHandShapeEntityID);
				chainedEventEntity.setDependentLocationEntityID(dependentLocationEntityID);
				chainedEventEntity.setDependentPropertyEntityID(dependentSignTypeEntityID);
				
				chainedEventEntity.setLockedGesturePhonAlignments(isLockedGesturePhonAlignments);
				chainedEventEntity.setLockedDomNdomAlignments(isLockedDomNdomAlignments);
				chainedEventEntity.setIncompleteGesture(isIncompleteGesture);
				chainedEventEntity.setShowUniqueNumber(showUniqueNumber);
				chainedEventEntity.setUniqueNumber(uniqueNumber);

				chainedEventEntity.setID(entity.getID());
				chainedEventEntity.setRefEntityId(entity.getRefEntityId());
				//chainedEventEntity.setReference(entity.getReference());
	            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
	            	chainedEventEntity.updateParentEntity();
	            } else {
	            	chainedEventEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
	            }

//				if (entity.isValueStartTimeLocked() && entity.isValueEndTimeLocked()) {
//					chainedEventEntity.setLockedDomNdomAlignments(true);
//				}
				
//				|| entity.isInitialHoldStartTimeLocked())
//					chainedEventEntity.lockStart();
	//
//				if (entity.isValueEndTimeLocked() || entity.isFinalHoldEndTimeLocked())
//					chainedEventEntity.lockEnd();

				chainedEvents.add(chainedEventEntity);
			}
		}
// 
		// GestureEventsEntityCollection 
		collection = new GestureEventsEntityCollection(chainedEvents, segmentPanel);
//		ArrayList<GestureEventsEntity> entities, segmentPanel);		
// 		new GestureChainedEventsEntityCollection(chainedEvents, segmentPanel);
// 		collection.updateGestureEntities();

		// set compound mappings
		for (int c = 0; c < entities.size(); c++) {
			SS3Entity ent = entities.get(c);
			String sCmpEntID = ent.getStartCompoundEntityID();
			String eCmpEntID = ent.getEndCompoundEntityID();

			if (Util.isNotEmpty(sCmpEntID) || Util.isNotEmpty(eCmpEntID)) {
				GestureEventsEntity mEnt = collection.getEntity(ent.getID());
				if (mEnt != null) {
					if (Util.isNotEmpty(sCmpEntID)) {
						GestureEventsEntity sEnt = collection.getEntity(sCmpEntID);
						mEnt.setStartCompound(sEnt);
					}
					if (Util.isNotEmpty(eCmpEntID)) {
						GestureEventsEntity eEnt = collection.getEntity(eCmpEntID);
						mEnt.setEndCompound(eEnt);
					}
				}
			}
		}

		GestureEnteredText entText = new GestureEnteredText(segmentPanel, this);
		for(SS3Item txt : txtOnly) {
			GestureText text = new GestureText(txt.getText());
			text.setStartTime(txt.getStartTimeInfo().doubleValue());
			text.setEndTime(txt.getEndTimeInfo().doubleValue());
			entText.addText(text);
		}
		
		collection.setEnteredTextObject(entText);
	}

	@Deprecated	// legacy code involving Track, not SS3Track
	private void createTrackEntity(Track track) {
//		ArrayList<GestureEventsEntity> chainedEvents = new ArrayList<GestureEventsEntity>();
//		ArrayList trackValues = track.getTrackValues();
//
//		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
//		HashMap<String, TrackValue> onset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> onsetHold = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offsetHold = new HashMap<String, TrackValue>();
//
//		for (int i = 0; i < trackValues.size(); i++) {
//			TrackValue trackValue = (TrackValue) trackValues.get(i);
//			int valueId = trackValue.getValueId();
//
//			if (valueId == Constants.GESTURE_HOLD_CODE) {
//				double startFrame = trackValue.getStartTimeInfo().doubleValue();
//				double endFrame = trackValue.getEndTimeInfo().doubleValue();
//				onsetHold.put("" + endFrame, trackValue);
//				offsetHold.put("" + startFrame, trackValue);
//			} else if (valueId == Constants.GESTURE_PREPARATION_CODE) {
//				double endFrame = trackValue.getEndTimeInfo().doubleValue();
//				onset.put("" + endFrame, trackValue);
//			} else if (valueId == Constants.GESTURE_RETRACTION_CODE) {
//				double startFrame = trackValue.getStartTimeInfo().doubleValue();
//				offset.put("" + startFrame, trackValue);
//			} else {
//				values.add(trackValue);
//			}
//		}
//
//		for (TrackValue trackValue : values) { 
//			double startingFrame = trackValue.getStartTimeInfo().doubleValue();
//			double endingFrame = trackValue.getEndTimeInfo().doubleValue();
//
//			TrackValue onsetHoldValue = onsetHold.get(""
//					+ startingFrame);
//			TrackValue offsetHoldValue = offsetHold.get(""
//					+ endingFrame);
//			TrackValue onsetValue = null, offsetValue = null;
//
//			if (onsetHoldValue == null) {
//				onsetValue = onset.get("" + startingFrame);
//			} else {
//				onsetValue = onset.get(""
//						+ onsetHoldValue.getStartTimeInfo().doubleValue());
//			}
//
//			if (offsetHoldValue == null) {
//				offsetValue = onset.get("" + endingFrame);
//			} else {
//				offsetValue = onset.get(""
//						+ offsetHoldValue.getEndTimeInfo().doubleValue());
//			}
//
//			GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
//			collection.setPreparationEvent(new GestureChainedEvent(onsetValue, segmentPanel));
//			collection.setPrestrokeHoldEvent(new GestureChainedEvent(onsetHoldValue, segmentPanel));
//			collection.setTextValueEvent(new GestureEvent(trackValue, segmentPanel));
//			collection.setPoststrokeHoldEvent(new GestureChainedEvent(offsetHoldValue, segmentPanel));
//			collection.setRetractionEvent(new GestureChainedEvent(offsetValue, segmentPanel));
//
//			GestureEventsEntity chainedEvent = new GestureEventsEntity(collection, segmentPanel, this);
//			chainedEvent.determineParentEntity();
//
//			chainedEvents.add(chainedEvent);
//		}
//
//		collection = new GestureEventsEntityCollection(chainedEvents, segmentPanel); 
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		this.Y = Y;
		
		ArrayList<GestureEventsEntity> glossEntities = collection.getEntities();
		
		for(GestureEventsEntity entity : glossEntities) {
			entity.paint(g2d, Y, font);
		}

		if (selectedCompoundStartCoord > -1 && selectedCompoundEndCoord > -1) {
			g2d.setColor(Palette.SELECTED_BACKGROUND_COLOR);
			g2d.fillRect(selectedCompoundStartCoord, (int) (Y - 13),
					selectedCompoundEndCoord - selectedCompoundStartCoord, 19);
		}

		try {
			getGestureEventsEntityCollection().getEnteredText().paint(g2d, Y, font);
		} catch (NullPointerException ex) {
			;
		}
		
		collection.paintNewEvent(g2d, Y);
	}

    @Override
    public boolean isIntervalEmpty(Interval interval) {
	    return collection.getEntitiesInInterval(interval).isEmpty();
    }

	public void setNewGestureChainedEvent(NewGestureChainedEvent event) {
		collection.setNewGestureChainedEvent(event);
	}

	public NewGestureChainedEvent getNewGestureChainedEvent() {
		return collection.getNewGestureChainedEvent();
	}

	@Override
	public boolean canDrawNewEvent(double x) {
		if (contains(x)) {
			GestureChainedEvent event = collection.getEventAt(x);
			return (event == null);
		}

		return false;
	}
}
