package edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations;

import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import java.util.*;

/**
 * This class is used to by GlossEntityProperties to store the list of locations
 * for a gloss, as well as by LocationPanel to represent the list of locations that
 * the user has selected with the mouse
 */
public class GestureLocationInfo {

    private HashMap<GestureLocationType, GestureLocationObject> locations;
//    private HashMap<GestureLocationType, GestureRegion> locationsRegions;
//    private HashMap<GestureLocationType, Boolean> locationsIsContralateralByType;

    public GestureLocationInfo() {
//        locations = new HashMap<GestureLocationType, GestureRegion>();
//        locationsIsContralateralByType = new HashMap<GestureLocationType, Boolean>();
//        for (GestureLocationType locationType : SS3Singleton
//        		.getGestureLocationConfig()
//        		.getLocationTypes()) {
//            locations.put(locationType, GestureRegion.NONE);
//            locationsIsContralateralByType.put(locationType, false);
//        }
        locations = new HashMap<GestureLocationType, GestureLocationObject>();
        for (GestureLocationType locationType : SS3Singleton.getGestureLocationConfig().getLocationTypes()) {
            locations.put(locationType, new GestureLocationObject(GestureRegion.NONE, null));
        }
    }

    public GestureLocationInfo(HashMap<GestureLocationType, GestureLocationObject> locations) {
//        this.locationsRegions = new HashMap<GestureLocationType, GestureRegion>(regions);
//        this.locationsIsContralateralByType = new HashMap<GestureLocationType, Boolean>(perLocationIsContralateral);
        this.locations = locations;
    }

    public GestureLocationObject getLocationObj(GestureLocationType type) {
        return locations.get(type);
    }

    public void clear() {
        locations.clear();
    }

    public boolean isEmpty() {
        if (locations.isEmpty()) {
            return true;
        }
        Collection<GestureLocationObject> values = locations.values();
        for (GestureLocationObject value : values) {
            if (!value.getRegion().equals(GestureRegion.NONE)) {
                return false;
            }
        }
        return true;
    }

    public int size() {
        return locations.size();
    }

    public GestureLocationInfo cloneInfo() {
        return new GestureLocationInfo(locations);
    }

    @Override
    public String toString() {
        StringBuilder str = new StringBuilder("[");
        for (Iterator<Map.Entry<GestureLocationType, GestureLocationObject>> iterator = locations.entrySet().iterator(); iterator.hasNext(); ) {
            Map.Entry<GestureLocationType, GestureLocationObject> entry = iterator.next();
            str.append(entry.getKey().getName()).append(":").append(entry.getValue().getRegion().getName())
                    .append(",").append(entry.getValue().isContralateral());
            if (iterator.hasNext()) {
                str.append(" ->");
            }
        }
        str.append("]");
        return str.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof GestureLocationInfo)) {
            return false;
        }
        GestureLocationInfo other = (GestureLocationInfo) obj;
        for (GestureLocationType locationType : SS3Singleton.getGestureLocationConfig().getLocationTypes()) {
            if (!other.getLocationObj(locationType).equals(getLocationObj(locationType))) {
                return false;
            }
        }
        // GREG TODO: not sure if it is required to check for ipsi/contra value for equality
        return true;
    }
}

