package edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoPanel;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;

import java.util.ArrayList;

/**
 * This class displays a scaled down version of a region's image and its name
 */
public class RegionDisplayPanel extends JPanel  {

    private GestureInfoPanel gip;

    private JLabel regionNameLabel = new JLabel();

    private ImageDisplayPanel imageDisplayPanel;
    private GestureRegion subregion = GestureRegion.NONE;

    public RegionDisplayPanel(GestureInfoPanel gip2) {
        this.gip = gip2;
    }

    public void setRegion(GestureRegion subregion) {
        if (subregion == null || subregion.getParent() == null) {
            subregion = GestureRegion.NONE;
        }

        removeAll();

        this.subregion = subregion;
        regionNameLabel.setText(subregion.getName());
        GestureLocationImage image;
        GestureLocationConfig lc = SS3Singleton.getGestureLocationConfig();
        if (subregion.equals(GestureRegion.NONE)) {
            image = lc.getImage(lc.getRegion(GestureLocationConfig.REGION_FULL_BODY).getImageID());
        } else {
            image = lc.getImage(subregion.getParent().getImageID());
        }
        boolean flip = !gip.getDominantHand().equals(image.getDominance());
        imageDisplayPanel = new ImageDisplayPanel(subregion, image, flip);

        initUI();
        revalidate();
        repaint();
    }

    private void initUI() {
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;

        setLayout(new GridBagLayout());
        add(regionNameLabel, c);

        c.gridy++;
        add(imageDisplayPanel, c);
    }

    public GestureRegion getRegion() {
        return subregion;
    }
}

class ImageDisplayPanel extends JPanel {

    private static final int width = 160;
    private static final int height = 170;

    private double scale;

    private GestureRegion subregion;
    private GestureLocationImage parentImage;

    private boolean flip;

    ImageDisplayPanel(GestureRegion subregion, GestureLocationImage image, boolean flip) {
        this.subregion = subregion;
        this.parentImage = image;
        this.flip = flip;

        double wScale = ((double)width) / parentImage.getBufferedImage().getWidth();
        double hScale = ((double)height) / parentImage.getBufferedImage().getHeight();
        scale = Math.min(wScale, hScale);
    }

    @Override
    public void paint(Graphics _g) {
        Graphics2D g = (Graphics2D) _g;
        if (!subregion.equals(GestureRegion.NONE)) {
            int imageWidth = (int) (parentImage.getBufferedImage().getWidth() * scale);
            int imageHeight = (int) (parentImage.getBufferedImage().getHeight() * scale);

            int translateX = (width - imageWidth) / 2;
            int translateY = (height - imageHeight) / 2;

            AffineTransform currentTransform = g.getTransform();
            if (flip) {
                Util.flipHorizontal(imageWidth, g);
            }

            g.drawImage(parentImage.getBufferedImage(),
                        translateX, translateY,
                        translateX + imageWidth, translateY + imageHeight,
                        0, 0,
                        parentImage.getBufferedImage().getWidth(), parentImage.getBufferedImage().getHeight(),
                        this);

            ArrayList<Polygon> polygons = subregion.getParent().getPolygons(subregion);
            for (Polygon poly : polygons) {
                Polygon scaled = PolygonHelper.scaleBy(scale, poly);
                Polygon translated = PolygonHelper.translateBy(translateX, translateY, scaled);
                g.setColor(Color.GREEN);
                g.drawPolygon(translated);
                g.setColor(new Color(0, 255, 0, 128));
                g.fillPolygon(translated);
            }

            if (flip) {
                g.setTransform(currentTransform);
            }
        } else {
            g.setBackground(getBackground());
            g.clearRect(0, 0, parentImage.getBufferedImage().getWidth(), parentImage.getBufferedImage().getHeight());
        }
    }
    @Override
    public Dimension getPreferredSize() {
        return new Dimension(width, height);
    }


}
