package edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations;

import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * This class shows a JComboBox populated with a {@link GestureRegion}'s list of children and a panel
 * that allows the user to select one of the children by clicking on an image.
 */
public class RegionSelectPanel extends JPanel {

    private JComboBox<GestureRegion> regionComboBox;
    private JButton forwardBtn = new JButton("Continue");
    private JButton backBtn = new JButton("Back");
    private JButton submitBtn = new JButton("Submit");
    private ImagePanel imagePanel;

    private GestureRegion region;
    private GestureRegion defaultRegion;

    private RegionSelectDialog parentDialog;
    
    public RegionSelectPanel(GestureRegion rc, RegionSelectDialog dialog, boolean useBackBtn) {
        this.region = rc;
        this.parentDialog = dialog;
        Font size18 = new Font(null, Font.PLAIN, 18);

        GestureRegion[] regions = new GestureRegion[rc.getSubregions().size()];
        regions = rc.getSubregions().toArray(regions);

        regionComboBox = new JComboBox<GestureRegion>(regions);
        regionComboBox.setSelectedIndex(-1);
        regionComboBox.setFont(size18);
        regionComboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                GestureRegion value = ((GestureRegion) regionComboBox.getSelectedItem());
                if (!useBackBtn && value != null) {
                    if (value.isParent()) {
                        forwardBtn.setEnabled(true);
                    } else {
                        forwardBtn.setEnabled(false);
                    }
                }
                imagePanel.selectRegion(value);
                imagePanel.repaint();
            }
        });

        final ListCellRenderer<? super GestureRegion> renderer = regionComboBox.getRenderer();
        regionComboBox.setRenderer(new ListCellRenderer<GestureRegion>() {
            @Override
            public Component getListCellRendererComponent(JList<? extends GestureRegion> list, GestureRegion value, int index, boolean isSelected, boolean cellHasFocus) {
                if (isSelected) {
                    imagePanel.selectRegion(value);
                }
                return renderer.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            }
        });

        regionComboBox.addPopupMenuListener(new PopupMenuListener() {
            @Override public void popupMenuWillBecomeVisible(PopupMenuEvent e) {}
            @Override public void popupMenuCanceled(PopupMenuEvent e) {}

            @Override
            public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
                GestureRegion value = (GestureRegion) regionComboBox.getSelectedItem();
                if (value != null) {
                    defaultRegion = value;
                    imagePanel.setClickedRegion(defaultRegion);
                }
            }
        });

        submitBtn.setFont(size18);
        submitBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (regionComboBox.getSelectedItem() != null) {
                    parentDialog.selectedRegion((GestureRegion) regionComboBox.getSelectedItem(), false);
                }
            }
        });

        JPanel topPanel = new JPanel(new FlowLayout());

        backBtn.setFont(size18);
        if (rc.getParent() != null) {
            backBtn.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    parentDialog.goBack();
                }
            });
        } else {
            backBtn.setEnabled(false);
        }
        topPanel.add(backBtn);

        topPanel.add(regionComboBox);

        forwardBtn.setFont(size18);
        forwardBtn.setEnabled(false);
        forwardBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                forward();
            }
        });
        topPanel.add(forwardBtn);
        topPanel.add(submitBtn);


        imagePanel = new ImagePanel(rc, this);

        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(5, 10, 5, 10);
        c.gridx = 0;
        c.gridy = 0;
        add(topPanel, c);
        c.gridy++;
        add(imagePanel, c);
    }

    

	
	



	public void selectRegion(GestureRegion rc) {
        if (rc == null) {
            if (defaultRegion == null) {
                regionComboBox.setSelectedIndex(-1);
            } else {
                regionComboBox.setSelectedItem(defaultRegion);
            }
            return;
        }
        regionComboBox.setSelectedItem(rc);
    }

    public void forward() {
        if (regionComboBox.getSelectedItem() != null) {
            GestureRegion selected = (GestureRegion) regionComboBox.getSelectedItem();
            parentDialog.selectedRegion(selected, selected.isParent());
        }
    }

    GestureRegion getRegion() {
        return region;
    }

    public GestureRegion getDefaultRegion() {
        return defaultRegion;
    }

    public void setDefaultRegion(GestureRegion defaultRegion) {
        this.defaultRegion = defaultRegion;
    }
}