package edu.bu.signstream.grepresentation.fields.gestureField.locationField;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;

public class GestureLocationEntity extends EventsEntity {
    private GestureLocationEventsCollection events;

    private GestureEventsEntity refGestureEntity;

    public GestureLocationEntity(String id, GestureLocationEventsCollection events, SignStreamSegmentPanel panel, Field field) {
        this(events, panel, field);
        setID(id);
    }

    public GestureLocationEntity(GestureLocationEventsCollection events, SignStreamSegmentPanel segmentPanel, Field field) {
        super(segmentPanel, field);
        this.events = events;
    }

    public void setEvents(ArrayList<GestureLocationEvent> events) {
        this.events.deleteAll();
        for (GestureLocationEvent event : events) {
            this.events.addEvent(event);
        }
    }

    public boolean isEmptyGestureLocationEntity() {
        for (GestureLocationEvent event : events.getEvents()) {
            if (!event.isNoneLocation()) {
                return false;
            }
        }
        return true;
    }

    public GestureLocationEventsCollection getGestureLocationEventsCollection() {
        return events;
    }

    @Override
    public SS3Entity getSS3Entity() {
        SS3Entity ss3Entity = new SS3Entity();
        for (GestureLocationEvent event : events.getEvents()) {
            ss3Entity.addItem(new SS3Item(event));
        }
        ss3Entity.setID(getID());
        ss3Entity.setRefEntityId(refGestureEntity.getID());
		if (getParentEntities() != null)
			ss3Entity.setParentEntities(getParentEntities());
        return ss3Entity;
    }

    @Deprecated
    public void setStartTime(int startTime) {
        var time = SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP);
        getFirstEvent().setStartTimeInfo(time);
    }

    @Deprecated
    public void setEndTime(int endTime) {
        var time = SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP);
        getLastEvent().setEndTimeInfo(time);
   }

    @Override
    public double getStartTimeCoordinate() {
        return getFirstEvent().getStartTimeCoordinate();
    }

    // should return Start Time Coordinate!!!
    @Override
    public double getEndTimeCoordinate() {
        return getLastEvent().getStartTimeCoordinate();
    }

    @Override
    public Event getFirstEvent() {
        return events.getFirstEvent();
    }

    @Override
    public Event getLastEvent() {
        return events.getLastEvent();
    }

    @Deprecated
    public double getStartTime() {
        GestureLocationEvent event = events.getFirstEvent();
        if (event == null)
            return 0;
        return event.getStartTimeInfo().doubleValue();
    }

    @Deprecated
    public double getEndTime() {
        GestureLocationEvent event = events.getLastEvent();
        if (event == null)
            return 0;
        return event.getEndTimeInfo().doubleValue();
    }

    @Override
    public boolean selectEntityIfEventSelected() {
        return true;
    }

    public ArrayList<GestureLocationEvent> getEvents() {
        return events.getEvents();
    }

    @Override
    public boolean selectEventIfEntitySelected() {
        return true;
    }

    @Override
    public boolean select() {
        return true;
    }

    @Override
    public boolean unselect() {
        return true;
    }

    @Override
    public boolean adjustEntity(int x1, int x2) {
        CoordinateTimeConverter convertor =
                segmentPanel.getZoomer().getCoordinateTimeConvertor();
        Time startTimeInfo = convertor.convertCoordinateToTimeInfo(x1);
        Time endTimeInfo = convertor.convertCoordinateToTimeInfo(x2);
        return events.adjustEntity(startTimeInfo, endTimeInfo);
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public void reset() {
        selected = false;
        events.resetAll();
    }

    @Override
    public boolean delete() {
        return events.deleteAll();
    }

    @Override
    public Event getEventAt(double x) {
        return null;
    }

    @Override
    public ArrayList<String> getEventsToAdd() {
        return null;
    }

    @Override
    public String getToolTipText(int x) {
        StringBuilder str = new StringBuilder("[");
        for (int i = 0; i < events.size() - 1; i++) {
            str.append(events.getEvent(i).getToolTipText()).append(" -> ");
        }
        str.append(events.getEvent(events.size() - 1).getToolTipText()).append("]");
        return str.toString();
    }

    @Override
    public boolean selectEvent(double x) {
        return true;
    }

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        if (isEmptyGestureLocationEntity()) {
            return;
        }

        g2d.setFont(font);

        double X1 = getStartTimeCoordinate();
        double X2 = getEndTimeCoordinate();

        int y = (int) Y;

        ArrayList<GestureLocationEvent> eventsList = getGestureLocationEventsCollection().getEvents();

        g2d.setColor(Color.BLACK);

        for (int i = 0; i < eventsList.size(); i++) {
            GestureLocationEvent locationEvent = eventsList.get(i);
            if (!locationEvent.isNoneLocation()) {
            	String locationText = locationEvent.getText();
            	Boolean locationIsContralateral = locationEvent.getIsContralateral();
                if (locationIsContralateral != null) {
                    String locationIsContralateralStr = locationIsContralateral ? "C" : "I";        // "C" for contralateral, "I" for ipsilateral
                    locationText += "-" + locationIsContralateralStr;
                }
                double x;
                if (i == 0) {
                    x = X1;
                } else if (i == eventsList.size() - 1) {
                    x = X2 - g2d.getFontMetrics().stringWidth(locationText);
                } else {
                    x = locationEvent.getStartTimeCoordinate() - g2d.getFontMetrics().stringWidth(locationText) / 2;
                }
                g2d.drawString(locationText, (int)x, y);
            }
        }

        g2d.drawLine((int)X1, y + 1, (int)X2, y + 1);
        g2d.drawLine((int)X1, y + 2, (int)X2, y + 2);
    }

    @Override
    public String toString() {
        return "GestureLocationEntity: " + super.toString();
    }

    @Override
    public Time getStartTimeInfo() {
        GestureLocationEvent event = events.getFirstEvent();
        if (event == null)
            return null;
        return event.getStartTimeInfo();
    }

    @Override
    public Time getEndTimeInfo() {
        GestureLocationEvent event = events.getLastEvent();
        if (event == null)
            return null;
        return event.getEndTimeInfo();
    }

    @Override
    public void setStartTimeInfo(Time startTimeInfo) {
        GestureLocationEvent event = events.getFirstEvent();
        if (event != null)
            event.setStartTimeInfo(startTimeInfo);
    }

    @Override
    public void setEndTimeInfo(Time endTimeInfo) {
        GestureLocationEvent event = events.getLastEvent();
        if (event != null)
            event.setEndTimeInfo(endTimeInfo);
    }

    public void updateTimes() {
        if (refGestureEntity != null) {
            setStartTimeInfo(refGestureEntity.getStartTimeInfo());
            setEndTimeInfo(refGestureEntity.getEndTimeInfo());
        }
        
        Time start = getStartTimeInfo();
        Time end = getEndTimeInfo();
        int size = events.size();

		Duration dur = end.distance(start).divide(size - 1);
		
		for (int i = 0; i < size; i++) {
			Time eventTime = null;
            GestureLocationEvent locationEvent = events.getEvent(i);
            if (i == 0) {
            	eventTime = start;
            } else if (i == events.size() - 1) {
            	eventTime = end;
            } else {
            	eventTime = start.add(dur.multiply(i));
            }
            
            locationEvent.setStartTimeInfo(eventTime);
            locationEvent.setEndTimeInfo(eventTime);
		}
    }

    @Override
    public boolean contains(double x) {
        return super.contains(x);
    }

    public void setRefGestureEntity(GestureEventsEntity refDomGestureEntity) {
        this.refGestureEntity = refDomGestureEntity;
        this.setRefEntityId(refDomGestureEntity.getID());
    }

	@Override
	public ArrayList<Event> getEventsList() {
		return new ArrayList<Event>(events.getEvents());
	}
}
