package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.stream.Collectors;

import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.NewChainedEvent;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GlossChainedEventsEntityCollection {

	private ArrayList<GlossChainedEventsEntity> entities = new ArrayList<GlossChainedEventsEntity>();
	private NewChainedEvent newEvent = null;
	private EnteredText enteredGlossText = null;
	private SignStreamSegmentPanel segmentPanel;

	public GlossChainedEventsEntityCollection(
			ArrayList<GlossChainedEventsEntity> entities,
			SignStreamSegmentPanel segmentPanel) {
		this.entities = entities;
		this.segmentPanel = segmentPanel;

        sortEntities();
	}

    public void sortEntities() {
        int entitiesSize = 0;

        for(GlossChainedEventsEntity entity : entities) {
            // skip entered text values
            if(entity.getStartTimeInfo() != null) {
                entitiesSize++;
            }
        }

        if(entitiesSize == 1)
            return;

        double[] startTimes = new double[entitiesSize];
        HashMap<String, GlossChainedEventsEntity> map = new HashMap<String, GlossChainedEventsEntity>();

        int i = 0;
        for(GlossChainedEventsEntity entity : entities) {
            if(entity.getStartTimeInfo() != null) {
                double movieTime = entity.getStartTimeInfo().doubleValue();
                map.put(""+movieTime, entity);
                startTimes[i] = movieTime;
                i++;
            }
        }

        Sorter.sort(startTimes);

        i = 0;
        while(i < entitiesSize-1) {
            GlossChainedEventsEntity glsEntity = map.get(""+startTimes[i]);
            i++;
            GlossChainedEventsEntity nextGlsEntity = map.get(""+startTimes[i]);
            glsEntity.setNextEntity(nextGlsEntity);
            nextGlsEntity.setPreviousEntity(glsEntity);
        }
    }
	
	public ArrayList<GlossChainedEventsEntity> getGlossChainedEventsEntityWithText(String txt) {
		ArrayList<GlossChainedEventsEntity> glosses = new ArrayList<GlossChainedEventsEntity>();
		for(GlossChainedEventsEntity entity : entities) {
			String text = entity.getText();
			if(text.equalsIgnoreCase(txt)) {
				glosses.add(entity);
			}
		}
		return glosses;
	}
	
	public double getNextEntityOrTextStartCoord(double coord) {
		double smalestNextCoord = -1;
		for(GlossChainedEventsEntity entity : entities) {
			double entityCoord = entity.getStartTimeCoordinate(); 

			if(smalestNextCoord == -1) {
				if(entityCoord >= coord) {
					smalestNextCoord = entityCoord;
				}
			} else {
				if(entityCoord > coord && entityCoord < smalestNextCoord) {
					smalestNextCoord = entityCoord;
				}	
			}
		}
		
		if(enteredGlossText != null) {
			ArrayList<Text> entText = enteredGlossText.getEnteredText();
			for(Text text : entText) {
				double textStartCoord = text.getStartCoord();
				if(smalestNextCoord == -1) {
					if(textStartCoord > coord) {
						smalestNextCoord = textStartCoord;
					}
				} else {
					if(textStartCoord > coord && textStartCoord < smalestNextCoord) {
						smalestNextCoord = textStartCoord;
					}	
				}
			}
		}
		
		return smalestNextCoord;
	}
	
	public double getPrevEntityOrTextStartCoord(double coord) {
		double smalestPrevCoord = -1;
		
		for(GlossChainedEventsEntity entity : entities) {
			double entityCoord = entity.getEndTimeCoordinate(); 

			if(smalestPrevCoord == -1) {
				if(entityCoord <= coord) {
					smalestPrevCoord = entityCoord;
				}
			} else {
				if(entityCoord < coord && entityCoord > smalestPrevCoord) {
					smalestPrevCoord = entityCoord;
				}	
			}
		}
		
		if(enteredGlossText != null) {
			ArrayList<Text> entText = enteredGlossText.getEnteredText();
			for(Text text : entText) {
				double textStartCoord = text.getStartCoord();
				if(smalestPrevCoord == -1) {
					if(textStartCoord < coord) {
						smalestPrevCoord = textStartCoord;
					}
				} else {
					if(textStartCoord < coord && textStartCoord > smalestPrevCoord) {
						smalestPrevCoord = textStartCoord;
					}	
				}
			}
		}
		
		return smalestPrevCoord;
	}

	public GlossChainedEventsEntity getNextEntity(Time time) {
		GlossChainedEventsEntity nextEntity = null;

		for (GlossChainedEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();

				if (entStartTimeInfo.greaterThanOrEq(time)) {
					if (nextEntity == null) {
						nextEntity = ent;
					} else if (entStartTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())
							&& entEndTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())) {
						nextEntity = ent;
					}
				}
			}
		}

		return nextEntity;
	}
	
	public GlossChainedEventsEntity getPreviousEntity(Time time) {
		GlossChainedEventsEntity previousEntity = null;

		for (GlossChainedEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();

				if (entStartTimeInfo.lessThanOrEq(time)) {
					if (previousEntity == null) {
						previousEntity = ent;
					} else if (entStartTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo())
							&& (entEndTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo()))) {
						previousEntity = ent;
					}
				}
			}
		}
		
		return previousEntity;
	}

    // JACK TODO: ??????
	public ArrayList<NotCompleteGlossEvent> getNotCompleteGlossEventsInInterval(int startCoord, int endCoord) {
		return new ArrayList<NotCompleteGlossEvent>();
	}

    public ArrayList<GlossChainedEventsEntity> getEntitiesInInterval(Interval interval) {
        return entities.stream()
            .filter(e -> e.overlaps(interval))
            .collect(Collectors.toCollection(ArrayList::new));
    }

    public ArrayList<GlossChainedEventsEntity> getEntitiesInInterval(Interval interval, Bound bound) {
        return entities.stream()
            .filter(e -> e.overlaps(interval, bound))
            .collect(Collectors.toCollection(ArrayList::new));
    }

	public void removeEmptyEntities() {
		ArrayList<GlossChainedEventsEntity> entitiesToDelete = new ArrayList<GlossChainedEventsEntity>();

		for (GlossChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null
					|| entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (GlossChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}

	public void setEnteredTextObject(EnteredText enteredGlossText) {
		this.enteredGlossText = enteredGlossText;
	}

	public EnteredText getEnteredText() {
		return enteredGlossText;
	}

	public GlossChainedEventsEntity getEntityWithRefID(String refEntityId) {
		for (GlossChainedEventsEntity entity : entities) { 
			if (refEntityId != null
					&& refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	public GlossChainedEventsEntity getEntity(String entityID) {
		for (GlossChainedEventsEntity entity : entities) {
			String id = entity.getID();
			if (id.equalsIgnoreCase(entityID))
				return entity;
		}
		return null;
	}

	public ArrayList<GlossChainedEventsEntity> getEntities() {
		return entities;
	}

	public int size() {
		return entities.size();
	}

	public void deleteEntity(GlossChainedEventsEntity entity) {
		GlossChainedEventsEntity previousEntity = entity.getPreviousEntity();
		GlossChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		entities.remove(entity);
	}

	public Time getStartTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty()) 
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time earliestStartTime = entities.get(0).getStartTimeInfo();
				
		for (GlossChainedEventsEntity entity : entities) {
			earliestStartTime = (earliestStartTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
					? earliestStartTime
					: entity.getStartTimeInfo());
		}
		
		return earliestStartTime;
	}

	public Time getEndTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty())
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time latestEndTime = entities.get(0).getEndTimeInfo();
		
		for (GlossChainedEventsEntity entity : entities) {
			latestEndTime = (latestEndTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
						? latestEndTime
						: entity.getEndTimeInfo());
		}
		
		return latestEndTime;
	}

	public double getStartTimeCoordinate() {
		Time time = getStartTimeInfo();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(time);
	}

	public double getEndTimeCoordinate() {
		Time time = getEndTimeInfo();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(time);
	}

	public void unselect() {
		for (GlossChainedEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	public void reset() {
		for (GlossChainedEventsEntity entity : entities) {
			entity.reset();
		}
	}

	public void addEntity(GlossChainedEventsEntity entity) {

		Time startTimeInfo = entity.getStartTimeInfo();

		GlossChainedEventsEntity previousEntity = null;
		GlossChainedEventsEntity nextEntity = null;

		// GD: we need all this wacky logic because we're currently allowing one-frame glosses to exist
		// on the same frame as the start/end of existing glosses - or multiple even, at that
		for (GlossChainedEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();
				
				if (entStartTimeInfo.equals(startTimeInfo)) {
					nextEntity = ent;
				} else if (entEndTimeInfo.equals(startTimeInfo)) {
					previousEntity = ent;
				} else if (entStartTimeInfo.lessThanOrEq(startTimeInfo)) {
					if (previousEntity == null) {
						previousEntity = ent;
					} else if (entStartTimeInfo.greaterThanOrEq(previousEntity.getEndTimeInfo())
							&& (entEndTimeInfo.greaterThanOrEq(previousEntity.getEndTimeInfo()))) {
						previousEntity = ent;
					}
				} else {
					if (nextEntity == null) {
						nextEntity = ent;
					} else if (entStartTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())
							&& entEndTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())) {
						nextEntity = ent;
					}
				}
			}
		}

		if (previousEntity != null) {
			GlossChainedEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		this.entities.add(entity);
	}

	public void setNewChainedevent(NewChainedEvent newEvent) {
		this.newEvent = newEvent;
	}

	public void selectEntity(int x) {
		EventsEntity entity = getGlossChainedEventsEntityAt(x);
		if (entity != null)
			entity.select();
	}

	public GlossChainedEventsEntity getGlossChainedEventsEntityAt(double coordX) {
		for (GlossChainedEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public Event getSelectedEvent() {
		GlossChainedEventsEntity selectedEntity = getSelectedEntity();
		if (selectedEntity != null) {
			return selectedEntity.getSelectedEvent();
		}
		return null;
	}

	public GlossChainedEventsEntity getSelectedEntity() {
		for (GlossChainedEventsEntity entity : entities) {
			if (entity.getSelectedEvent() != null)
				return entity;
		}
		return null;
	}

	public ChainedEvent getEventAt(double x) {
		GlossChainedEventsEntity entity = getGlossChainedEventsEntityAt(x);
		if (entity == null)
			return null;

		return entity.getChainedEventsCollection().getEventAt(x);
	}

	public void setSelectedEvementValueID(String valueID) {
		Event event = getSelectedEvent();

		if (event instanceof ChainedEvent) {
			ChainedEvent chainedEvent = (ChainedEvent) event;
			chainedEvent.setID(valueID);
		}
	}

	public NewChainedEvent getNewChainedEvent() {
		return newEvent;
	}

	public ChainedEventsCollection createNewChainedEventCollection(String valueID) {
		if (valueID != null && newEvent != null) {
			TrackValue trackValue = new TrackValue();
			trackValue.setValueId(valueID);

			trackValue.setStartTimeInfo(newEvent.getStartTimeInfo());
			trackValue.setEndTimeInfo(newEvent.getEndTimeInfo());

			ChainedEvent event = new ChainedEvent(trackValue, segmentPanel);

			ChainedEventsCollection collection = new ChainedEventsCollection();
			collection.setValueEvent(event);
			newEvent = null;

			return collection;
		}
		return null;
	}

	public void sort(ArrayList<GlossChainedEventsEntity> entitiesToSort) {
		removeEmptyEntities();

		HashMap<String, GlossChainedEventsEntity> map = new HashMap<String, GlossChainedEventsEntity>();

		double[] startTimes = new double[entitiesToSort.size()];

		ArrayList<GlossChainedEventsEntity> toRemove = new ArrayList<GlossChainedEventsEntity>();

		for (int i = 0; i < entitiesToSort.size(); i++) {
			GlossChainedEventsEntity entity = entitiesToSort.get(i);
			if (entity != null && entity.getStartTimeInfo() != null) {
				double startTime = entity.getStartTimeInfo().doubleValue();
				map.put("" + startTime, entity);
				startTimes[i] = startTime;
			} else {
				toRemove.add(entity);
			}
		}

		for (GlossChainedEventsEntity cEnt : toRemove) {
			entitiesToSort.remove(cEnt);
		}

		Sorter.sort(startTimes);
		entitiesToSort.clear();

		for (int j = 0; j < startTimes.length; j++) {
			GlossChainedEventsEntity obj = map.get("" + startTimes[j]);
			entitiesToSort.add(obj);
		}
	}

	public void paintEntities(Graphics2D g2d, double Y, Font font) {
		for (GlossChainedEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}
	}

	public void paintNewEvent(Graphics2D g2d, double Y) {
		if (newEvent != null) {
			Painter painter = segmentPanel.getPainter(); // new Painter();
			painter.drawNewEvent(newEvent, g2d, Y);
		}
	}

	public void paintEnteredGlossText(Graphics2D g2d, double Y, Font font) {
		if (enteredGlossText != null)
			enteredGlossText.paint(g2d, Y, font);
	}
/*	
	//
	public Compoundable getPreviousCompoundable(int startTime) {
		Compoundable nCmp = null;
		
		ArrayList<Compoundable> cmps = new ArrayList<Compoundable>();
		cmps.addAll(entities);
		//cmps.addAll(enteredGlossText.getNotCompleteGlossEvents());
		//cmps.addAll(enteredGlossText.getTextCollection());
		
		Iterator<Compoundable> eIt = cmps.iterator();
		while (eIt.hasNext()) {
			Compoundable cmp = eIt.next();
			if (startTime >= cmp.getEndTimeInfo().getMovieTime()) {
				if (nCmp == null) {
					nCmp = cmp;
				} else if (nCmp.getEndTimeInfo().getMovieTime() < cmp
						   .getEndTimeInfo().getMovieTime()) {
					nCmp = cmp;
				}
			}
		}
		
		return nCmp;
	}
	
	public ArrayList<Compoundable> getBeforenAfterCompounds(Event event) {
		Compoundable beforeCmp = null;
		Compoundable afterCmp = null;
		
		for (ChainedEventsEntity entity : entities) {
			if (entity.getEndTimeInfo().getMovieTime() < event
				.getStartTimeInfo().getMovieTime()) {
				if (beforeCmp == null) {
					beforeCmp = entity;
				} else if (entity.getEndTimeInfo().getMovieTime() > beforeCmp
						   .getEndTimeInfo().getMovieTime()) {
					beforeCmp = entity;
				}
			} else if (entity.getStartTimeInfo().getMovieTime() > event
					   .getEndTimeInfo().getMovieTime()) {
				if (afterCmp == null) {
					afterCmp = entity;
				} else if (entity.getStartTimeInfo().getMovieTime() < afterCmp
						   .getStartTimeInfo().getMovieTime()) {
					afterCmp = entity;
				}
			}
		}
		
		ArrayList<Compoundable> cmp = new ArrayList<Compoundable>();
		cmp.add(0, beforeCmp);
		cmp.add(1, afterCmp);
		
		return cmp;
	}
	
	public Compoundable getNextCompoundable(int endTime) {
		Compoundable nCmp = null;
		
		ArrayList<Compoundable> cmps = new ArrayList<Compoundable>();
		cmps.addAll(entities);
		//cmps.addAll(enteredGlossText.getNotCompleteGlossEvents());
		//cmps.addAll(enteredGlossText.getTextCollection());
		
		Iterator<Compoundable> eIt = cmps.iterator();
		while (eIt.hasNext()) {
			Compoundable cmp = eIt.next();
			if (endTime < cmp.getStartTimeInfo().getMovieTime()) {
				if (nCmp == null) {
					nCmp = cmp;
				} else if (cmp.getStartTimeInfo().getMovieTime() < nCmp
						   .getStartTimeInfo().getMovieTime()) {
					nCmp = cmp;
				}
			}
		}
		return nCmp;
	}
	//
	 */
}
