package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.domNdomField.DomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.DependentEntitiesManager;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntityTypes;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;

public class HandShapePreprocessor {

	private FieldWrappersCollection fwc;
	private boolean rightHandDominant = true;

	public HandShapePreprocessor(FieldWrappersCollection fwc) {
		this.fwc = fwc;
	}

	public void populateFormBean(HandShapeFormBean formBean) {
		char type = getSelectedHandShapeType();

		boolean rightHandFieldsUpdatable = isRightHandFieldUpdatable();
		boolean leftHandFieldsUpdatable = isLeftHandFieldUpdatable();
		boolean fingerspelledSelected = isFingerspelledSelected(type);
		boolean oneHandedSelected = isOneHandedSelected(type);

		boolean oneHandedRightSelected = isOneHandedRightSelected(leftHandFieldsUpdatable, rightHandFieldsUpdatable);

		SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication()
				.getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();

		formBean.setSS3CodingScheme(dbScheme);

		formBean.setType(type);

		formBean.setFingerspelledR(oneHandedRightSelected);
		formBean.setFingerspelledSelected(fingerspelledSelected);

		formBean.setOneHandedR(oneHandedRightSelected);
		formBean.setOneHandedSelected(oneHandedSelected);
	}

	private boolean isOneHandedRightSelected(boolean leftHandFieldsUpdatable,
			boolean rightHandFieldsUpdatable) {
		if (rightHandFieldsUpdatable) {
			if (rightHandDominant) {
				return true;
			}
			return !leftHandFieldsUpdatable;
		}
		return false;
	}

	private boolean isRightHandFieldUpdatable() {

		GlossChainedEventsEntity glossEntity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).selectedEntity();

		FieldWrapper rhWrapper = fwc
				.getFieldWrapper(Constants.TYPE_DOM_HAND);
		String rhID = rhWrapper.getField().getFieldID();

		ChainedEventsEntity depHandEntity = glossEntity
				.getDependentHandEntity();
		if (depHandEntity != null) {
			String id = depHandEntity.getField().getFieldID();
			if (rhID.equalsIgnoreCase(id)) {
				return true;
			}
		}

		GlossChainedEventsEntity depGlossEntity = glossEntity
				.getDependentGlossEntity();
		if (depGlossEntity != null) {
			ChainedEventsEntity depHandEntity1 = depGlossEntity
					.getDependentHandEntity();
			if (depHandEntity1 != null) {
				String id1 = depHandEntity1.getField().getFieldID();
				if (rhID.equalsIgnoreCase(id1)) {
					return true;
				}
			}
		}

		DomNdomHandFieldWrapper rlhWrp = (DomNdomHandFieldWrapper) rhWrapper;

		return rlhWrp.getField().isIntervalEmpty(glossEntity.interval());
	}

	private boolean isLeftHandFieldUpdatable() {
		GlossChainedEventsEntity glossEntity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).selectedEntity();

		FieldWrapper lhWrapper = fwc
				.getFieldWrapper(Constants.TYPE_NDOM_HAND);
		String lhID = lhWrapper.getField().getFieldID();

		ChainedEventsEntity depHandEntity = glossEntity
				.getDependentHandEntity();
		if (depHandEntity != null) {
			String id = depHandEntity.getField().getFieldID();
			if (lhID.equalsIgnoreCase(id)) {
				return true;
			}
		}

		GlossChainedEventsEntity depGlossEntity = glossEntity
				.getDependentGlossEntity();
		if (depGlossEntity != null) {
			ChainedEventsEntity depHandEntity1 = depGlossEntity
					.getDependentHandEntity();
			if (depHandEntity1 != null) {
				String id1 = depHandEntity1.getField().getFieldID();
				if (lhID.equalsIgnoreCase(id1)) {
					return true;
				}
			}
		}

		DomNdomHandFieldWrapper lhWrp = (DomNdomHandFieldWrapper) lhWrapper;

		return lhWrp.getField().isIntervalEmpty(glossEntity.interval());
	}

	private boolean isFingerspelledSelected(char type) {
		return type == HandShapeEntityTypes.FINGERSPELLED;
	}

	private boolean isOneHandedSelected(char type) {
		return type == HandShapeEntityTypes.ONE_HANDED;
	}

	private char getSelectedHandShapeType() {

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) SS3Singleton
				.getMediaToolBar(null).selectedEntity();

		ArrayList<EventsEntity> dEntities = entity.getDependentEntities();

		DependentEntitiesManager depMngr = new DependentEntitiesManager(
				dEntities);

		HandShapeFieldWrapper rHshWrp = (HandShapeFieldWrapper) fwc.getFieldWrapper(
						Constants.TYPE_DOM_HAND_SHAPE);

		if (rHshWrp != null) {
			EventsEntity rEnt = depMngr.getEventsEntityWithFieldID(rHshWrp
					.getField().getFieldID());

			if (rEnt != null) {
				HandShapeEntity rHshEnt = (HandShapeEntity) rEnt;
				return rHshEnt.getHandShapeType();
			}
		}

		HandShapeFieldWrapper lHshWrp = (HandShapeFieldWrapper) fwc.getFieldWrapper(
						Constants.TYPE_DOM_HAND_SHAPE);
		if (lHshWrp != null) {
			EventsEntity lEnt = depMngr.getEventsEntityWithFieldID(lHshWrp
					.getField().getFieldID());

			if (lEnt != null) {
				HandShapeEntity lHshEnt = (HandShapeEntity) lEnt;
				return lHshEnt.getHandShapeType();
			}
		}
		return HandShapeEntityTypes.ONE_HANDED;
	}

	public void setRightHandDominant(boolean rightHandDominant) {
		this.rightHandDominant = rightHandDominant;
	}
}
