package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

public class IPAKeyboardPanel extends JPanel {

	private final MouseAdapter mouseListener;
	
	public IPAKeyboardPanel() {
		mouseListener = new MouseAdapter() {
			final Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();

			@Override
			public void mousePressed(MouseEvent e) {
				JLabel label = (JLabel) e.getSource();
				label.setBackground(Color.GRAY);

				String symbol = label.getText().substring(label.getText().length() - 1);

				try {
				    var old = cb.getContents(null);
					cb.setContents(new StringSelection(symbol), null);

                    var focused = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                    SwingUtilities.invokeLater(() -> {
                        focused.dispatchEvent(new KeyEvent(
                            focused,
                            KeyEvent.KEY_PRESSED,
                            System.currentTimeMillis(),
                            KeyEvent.META_DOWN_MASK,
                            KeyEvent.VK_V,
                            'v'));
                        focused.dispatchEvent(new KeyEvent(
                            focused,
                            KeyEvent.KEY_RELEASED,
                            System.currentTimeMillis(),
                            KeyEvent.META_DOWN_MASK,
                            KeyEvent.VK_V,
                            'v'));

                        cb.setContents(old, null);
                    });
                } catch (Exception e1) {
					e1.printStackTrace();
				}
			}
			
			@Override
			public void mouseReleased(MouseEvent e) {
				JLabel label = (JLabel) e.getSource();
				label.setBackground(Color.WHITE);
			}
		};
		initUI();
	}
	
	private void initUI() {
		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		setLayout(layout);
		JLabel lettersLabel = new JLabel("Symbols");
		lettersLabel.setAlignmentX(CENTER_ALIGNMENT);
		lettersLabel.setFont(getFontOfSize(18));
		add(lettersLabel);

		JPanel lettersPanel = getLettersPanel(IPAKeyboard.SYMBOLS);
		lettersPanel.setAlignmentX(CENTER_ALIGNMENT);
		add(lettersPanel);
		JLabel diaLabel = new JLabel("Diacritics");
		diaLabel.setAlignmentX(CENTER_ALIGNMENT);
		diaLabel.setFont(getFontOfSize(18));
		add(diaLabel);
		JLabel diaSubtitle = new JLabel("Type the character you want to apply");
		JLabel diaSubtitle2 = new JLabel("the symbol to, then choose one of the below:");
		diaSubtitle.setAlignmentX(CENTER_ALIGNMENT);
		add(diaSubtitle);
		diaSubtitle2.setAlignmentX(CENTER_ALIGNMENT);
		add(diaSubtitle2);
		JPanel diaPanel = getLettersPanel(IPAKeyboard.DIACRITICS);
		diaPanel.setAlignmentX(CENTER_ALIGNMENT);
		add(diaPanel);
		setBackground(Color.WHITE);
	}
	
	private JPanel getLettersPanel(String[] charset) {
		JPanel lettersPanel = new JPanel();
		GridLayout layout = new GridLayout((int) Math.ceil(charset.length / 10.), 10);
		layout.setHgap(5);
		layout.setVgap(5);
		lettersPanel.setLayout(layout);
		lettersPanel.setBackground(Color.WHITE);
		for (int i = 0; i < charset.length; i++) {
			String c = charset[i];
			JLabel charLabel = new JLabel(c);
			charLabel.setBackground(Color.WHITE);
			charLabel.setOpaque(true);
			charLabel.setFont(getFontOfSize(18));
			charLabel.setHorizontalAlignment(SwingConstants.CENTER);
			charLabel.setVerticalAlignment(SwingConstants.CENTER);
			charLabel.setBorder(new EmptyBorder(0, 0, 5, 0));
			charLabel.addMouseListener(mouseListener);
			lettersPanel.add(charLabel);
		}
		lettersPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		return lettersPanel;
	}
	
	private static Font getFontOfSize(int size) {
		return new Font(Font.SANS_SERIF, Font.PLAIN, 20);
	}
}
