package edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;

public class DblHandshapePalette extends JPanel implements ActionListener {
	private HandShapesPaletteFormBean rhFormBean = null;
	private HandShapesPaletteFormBean lhFormBean = null;
	private HandShapeFormBean bean = null;
	private GeneralHandshapePalette rPalette = null;
	private GeneralHandshapePalette lPalette = null;
	private JDialog dialog = null;
    private SS3CodingScheme scheme;
    private HandShapePanel hsPanel;

	//private JComboBox templatesCb = new JComboBox();
   private JPanel languagePanel = null;

	private JButton enter = new JButton("Enter");
	private JButton cancel = new JButton("Cancel");

	private final int ENTER = 0;
	private final int CANCEL = 1;

	public DblHandshapePalette(
			HandShapesPaletteFormBean rhFormBean,
			HandShapesPaletteFormBean lhFormBean,
			HandShapeFormBean bean, SS3CodingScheme scheme,
			JDialog dialog, HandShapePanel hsPanel) {

		this.rhFormBean = rhFormBean;
		this.lhFormBean = lhFormBean;
		this.bean = bean;
		this.dialog = dialog;
		this.scheme = scheme;
		this.hsPanel = hsPanel;
		initUI(scheme);

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		enter.addActionListener(this);
		cancel.addActionListener(this);

		enter.setSelected(true);
	}
   
   private JPanel createLanguageSelectionPanel() {
      
      if(languagePanel != null)
         return languagePanel;
      
		String[] lstr = new String[1];
		lstr[0] = "American Sign Language";
		JComboBox l = new JComboBox(lstr);		

		String[] tstr1 = new String[4];
		tstr1[0] = "ASL - All";
		tstr1[1] = "ASL - Letters";
		tstr1[2] = "ASL - Numbers";
		tstr1[3] = "ASL - Unmarked";
		final JComboBox t1 = new JComboBox(tstr1);
		
		t1.setSelectedIndex(rhFormBean.getPaletteID());
		//rPalette.setPalett(1);	
		
		t1.addItemListener(new ItemListener() {
		      @Override
			public void itemStateChanged(ItemEvent itemEvent) {
			        int selectedIndex = t1.getSelectedIndex();
			        rPalette.setPalette(selectedIndex);
		      }
		});

		String[] tstr2 = new String[4];
		tstr2[0] = "ASL - All";
		tstr2[1] = "ASL - Letters";
		tstr2[2] = "ASL - Numbers";
		tstr2[3] = "ASL - Unmarked";
		final JComboBox t2 = new JComboBox(tstr2);
		
		t2.setSelectedIndex(lhFormBean.getPaletteID());
		
		t2.addItemListener(new ItemListener() {
		      @Override
			public void itemStateChanged(ItemEvent itemEvent) {
			        int selectedIndex = t2.getSelectedIndex();
			        lPalette.setPalette(selectedIndex);
		      }
		});

      ButtonPanel bp = new ButtonPanel();
      bp.add(new JLabel("Palettes: "));
      bp.add(l);
      bp.add(new JLabel(" Right Hand: "));
      bp.add(t1);
      bp.add(new JLabel(" Left Hand: "));
      bp.add(t2);

      return bp;
   }

	private void initUI(SS3CodingScheme scheme) {
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gbl);
      
      JPanel languagePanel = createLanguageSelectionPanel();
            gbc.insets = new Insets(15, 0, 0, 0);
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            add(languagePanel, gbc);       
      
       boolean rhSelected = rhFormBean.isRightHandSelected();

	  int all = 0;
      if(!rhSelected)
		all = 3;
       
       rPalette =
			new GeneralHandshapePalette(
				rhFormBean.getValues(),
				rhFormBean.getSelectedHshImages(), scheme, all, hsPanel);

		rPalette.setBorder(new TitledBorder(rhFormBean.getTitle()));
		rPalette.setPalette(rhFormBean.getPaletteID());

		int unm = 3;
		if(!rhSelected)
			unm = 0;
		
		//
		//System.err.println("rhSelected = " + rhSelected + ", all = " + all + ", unm = " + unm);
		//
		
		lPalette =
			new GeneralHandshapePalette(
				lhFormBean.getValues(),
				lhFormBean.getSelectedHshImages(), scheme, unm, hsPanel);

		lPalette.setBorder(new TitledBorder(lhFormBean.getTitle()));
		lPalette.setPalette(lhFormBean.getPaletteID());
		
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		add(rPalette, gbc);

		//gbc.anchor = GridBagConstraints.NORTHWEST;
		add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(lPalette, gbc);

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(cancel);
		buttonPanel.add(enter);

		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.insets = new Insets(15, 0, 0, 0);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = 7;
		add(buttonPanel, gbc);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
			case ENTER :
				bean.setUpdated(true);

	            ArrayList<HandShapeImage> rhImages = new ArrayList<HandShapeImage>(getRightHandSelectedHandshapes());
	            ArrayList<HandShapeImage> lhImages =  new ArrayList<HandShapeImage>(getLeftHandSelectedHandshapes());
	            
				// first check if this is a one-frame gloss, and if it is then disallow anything more than one handshape in either hand
				if (hsPanel.isOneFrameEntity()) {
					if (rhImages.size() > 1 || lhImages.size() > 1) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_EXACTLY_ONE);
						break;
					}
				} else if ( (hsPanel.getSelectedSameDifStartEndHandshapesDom() == StartEndHandshapesType.DIFFERENT && rhImages.size() == 1)
							|| (hsPanel.getSelectedSameDifStartEndHandshapesNdom() == StartEndHandshapesType.DIFFERENT && lhImages.size() == 1) ) {
					// conversely, if this is not a one-frame gloss and you have dif start/end selected,
					// then disallow a selection of just one handshape in either hand
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_MULTIPLE);
					break;
				}

				if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
					hsPanel.updateParentInfoSelectedHandshapes(rhImages, lhImages);
				} else {
					hsPanel.updateParentInfoSelectedHandshapes(lhImages, rhImages);
				}

				dialog.dispose();
				break;
			case CANCEL:
				dialog.dispose();
				break;
		}
	}

	public ArrayList<HandShapeImage> getRightHandSelectedHandshapes() {
		return rPalette.getSelectedHandshapes();
	}

	public ArrayList<HandShapeImage> getLeftHandSelectedHandshapes() {
		return lPalette.getSelectedHandshapes();
	}
}