package edu.bu.signstream.grepresentation.fields.movementField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class MovementField extends Field {
	private MovementEntityCollection entities = null;

	public MovementField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public MovementField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	@Override
	public void deleteEmptyEntities() {
	}

    @Override
    public boolean isIntervalEmpty(Interval interval) {
        return true;
    }

    @Override
	public void addEventsEntity(EventsEntity entity) {
		MovementEntity mEntity = (MovementEntity) entity;
		entities.addEntity(mEntity);
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		return entities.getEntity(entityID);
	}

	@Override
	public ArrayList getEntities() {
		return entities.getEntities();
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		return entities.getEntityWithRefID(refEntityId);
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		if (!highlight) {
			ArrayList ents = entities.getEntities();
			for (int i = 0; i < ents.size(); i++) {
				EventsEntity entity = (EventsEntity) ents.get(i);
				entity.highlight(false);
			}
		} else {
			EventsEntity ent = entities
					.getEntityWithRefID(refID);
			if (ent != null)
				ent.highlight(true);
		}
	}

	public void createTrackEntity(Track track) {
		ArrayList entity = new ArrayList();
		entities = new MovementEntityCollection(entity);
	}

	private void createTrackEntity(SS3Track track) {
		ArrayList lEntities = new ArrayList();
		ArrayList ent = track.getEntities();
		for (int j = 0; j < ent.size(); j++) {
			SS3Entity entity = (SS3Entity) ent.get(j);
			MovementEventsCollection collection = new MovementEventsCollection(segmentPanel);

			ArrayList items = entity.getItemsAsList();
			for (int i = 0; i < items.size(); i++) {
				SS3Item item = (SS3Item) items.get(i);
				ChainedEvent event = new ChainedEvent(item, segmentPanel);
				// event.setSS3Item(item);
				collection.addEvent(event);
			}

			MovementEntity lEntity = new MovementEntity(collection, segmentPanel, this);

			lEntity.setID(entity.getID());
			lEntity.setRefEntityId(entity.getRefEntityId());
            lEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));

			lEntities.add(lEntity);
		}
		entities = new MovementEntityCollection(lEntities);
	}

	@Override
	public Event getEventAt(double x) {
		return entities.getEventAt(x);
	}

	@Override
	public EventsEntity getEntityAt(double entityAt) {
		Time timeInfo = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertCoordinateToTimeInfo(entityAt);
		ArrayList list = entities.getEntities();
		for (int i = 0; i < list.size(); i++) {
			MovementEntity entity = (MovementEntity) list.get(i);
			Time startTimeInfo = entity.getFirstEvent().getStartTimeInfo();
			Time endTimeInfo = entity.getLastEvent().getStartTimeInfo();

			if (startTimeInfo.lessThanOrEq(timeInfo) && endTimeInfo.greaterThanOrEq(timeInfo))
				return entity;
		}
		return null;
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
	}

	@Override
	public int getFieldSize() {
		return 0;
	}

	@Override
	public Event selectEvent(double x) {
		Event event = getEventAt(x);
		if (event != null)
			event.select();
		return event;
	}

	@Override
	public boolean contains(double y) {
		return y > Y - 14 && y < Y + 5;
	}

	@Override
	public double getStartTime() {
		return ss3Track.getStartTime();
	}

	@Override
	public double getEndTime() {
		return ss3Track.getEndTime();
	}

	@Override
	public double getStartTimeCoordinate() {
		return 0;
	}

	@Override
	public double getEndTimeCoordinate() {
		return 0;
	}

	@Override
	public void unselect() {
		entities.unselect();
	}

	@Override
	public Event getSelectedEvent() {
		return entities.getSelectedEvent();
	}

	@Override
	public EventsEntity getSelectedEntity() {
		return null;
	}

	public boolean setSelectedEventStartTime(int time) {
		return false;
	}

	public boolean setSelectedEventEndTime(int time) {
		return false;
	}

	public boolean setSelectedEventStartCoordinate(double coord) {
		MovementEntity entity = entities.getSelectedEntity();

		boolean overlap = entities.isOverlappingNotSelectedEvent(coord, entity
				.getEndTimeCoordinate());

		if (entity != null && !overlap)
			return entity.setSelectedEventStartCoordinate(coord);
		return false;
	}

	public boolean setSelectedEventEndCoordinate(double coord) {
		MovementEntity entity = entities.getSelectedEntity();

		boolean overlap = entities.isOverlappingNotSelectedEvent(entity
				.getStartTimeCoordinate(), coord);

		if (entity != null && !overlap)
			return entity.setSelectedEventEndCoordinate(coord);
		return false;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);
		entities.paintEntities(g2d, Y, font);
	}
}
