package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;

public class ChainedEventsCollection {
   protected ChainedEvent onset = null;
   protected ChainedEvent initialHold = null;
   protected ChainedEvent value = null;
   protected GlossChainedEvent textValue = null;
   protected ChainedEvent finalHold = null;
   protected ChainedEvent offset = null;

   public ChainedEvent getOnsetEvent() {
      return onset;
   }

   public ChainedEventsCollection cloneChainedEventsCollection() {
      ChainedEventsCollection collection = new ChainedEventsCollection();
      if (onset != null)
         collection.setOnsetEvent(onset.cloneChainedEvent());
      if (initialHold != null)
         collection.setInitialHoldEvent(initialHold.cloneChainedEvent());
      if (value != null)
         collection.setValueEvent(value.cloneChainedEvent());
      if (textValue != null)
         collection.setTextValueEvent(textValue.cloneGlossChainedEvent());
      if (finalHold != null)
         collection.setFinalHoldEvent(finalHold.cloneChainedEvent());
      if (offset != null)
         collection.setOffsetEvent(offset.cloneChainedEvent());
      return collection;
   }
   
   public Event getNextEvent(Event event) {
      ArrayList<ChainedEvent> collection = (ArrayList<ChainedEvent>) getCollection();
      for(int i = 0; i < collection.size()-1; i++) {
         Event cEvent = collection.get(i);
         if(event.equals(cEvent)) {
            return collection.get(i+1);
         }
      }
      return null;
   }
   
   public Event getPreviousEvent(Event event) {
      ArrayList<ChainedEvent> collection = (ArrayList<ChainedEvent>) getCollection();
      for(int i = 1; i < collection.size(); i++) {
         Event cEvent = collection.get(i);
         if(event.equals(cEvent)) {
            return collection.get(i-1);
         }
      }
      return null;
   }

//   public ArrayList<NoteItem> getNoteItems(String entityID) {
//      ArrayList<NoteItem> noteItems = new ArrayList<NoteItem>();
//      ArrayList<ChainedEvent> col = getCollection();
//
//      for (ChainedEvent event : col) {
//         if (event != null) {
//            String noteID = event.getNoteID();
//            if (noteID != null && noteID.length() > 0) {
//               String valueID = event.getID();
//               String text = null;
//               if (event instanceof GlossChainedEvent)
//                  text = event.getText();
//               NoteItem item = new NoteItem();
//               item.setNoteID(noteID);
//               item.setEntityID(entityID);
//               item.setValueID(valueID);
//               item.setText(text);
//               noteItems.add(item);
//            }
//         }
//      }
//      return noteItems;
//   }

   public ArrayList<? extends ChainedEvent> getCollection() {
      ArrayList<ChainedEvent> list = new ArrayList<ChainedEvent>();
      addEvent(list, onset);
      addEvent(list, initialHold);
      addEvent(list, value);
      addEvent(list, textValue);
      addEvent(list, finalHold);
      addEvent(list, offset);
      return list;
   }
   
   public ChainedEvent getEventAt(double x) {
      ArrayList<ChainedEvent> list = (ArrayList<ChainedEvent>) getCollection();
      for(ChainedEvent event : list) {
    	  double startTimeCoord = event.getStartTimeCoordinate();
    	  double endTimeCoord = event.getEndTimeCoordinate();
    	  
    	  if(event.getStartTimeInfo().doubleValue() == event.getEndTimeInfo().doubleValue()) {
    		  startTimeCoord = startTimeCoord - 5;
    		  endTimeCoord = endTimeCoord + 5; 
    	  }
    	  
    	  if(startTimeCoord <= x && x <= endTimeCoord) {
    		  return event;
    	  }
      }
      return null;
   }

   private void addEvent(ArrayList<ChainedEvent> list, ChainedEvent event) {
      if (event != null)
         list.add(event);
   }

   public void setOnsetEvent(ChainedEvent onset) {
      this.onset = onset;
      if (onset != null)
         onset.setText(Constants.ONSET);
   }

   public ChainedEvent getInitialHoldEvent() {
      return initialHold;
   }

   public void setInitialHoldEvent(ChainedEvent initialHold) {
      this.initialHold = initialHold;
      if (initialHold != null)
         initialHold.setText(Constants.INITIAL_HOLD);
   }

   public ChainedEvent getValueEvent() {
      return value;
   }

   public ChainedEvent getValueOrTextValueEvent() {
      return value == null ? textValue : value;
   }

   public void setValueEvent(ChainedEvent value) {
      this.value = value;
   }

   public GlossChainedEvent getTextValueEvent() {
      return textValue;
   }

   public void setTextValueEvent(GlossChainedEvent textValue) {
      this.textValue = textValue;
      if (textValue != null)
         textValue.setText(textValue.getText());
   }

   public ChainedEvent getFinalHoldEvent() {
      return finalHold;
   }

   public void setFinalHoldEvent(ChainedEvent finalHold) {
      this.finalHold = finalHold;
      if (finalHold != null)
         finalHold.setText(Constants.FINAL_HOLD);
   }

   public ChainedEvent getOffsetEvent() {
      return offset;
   }

   public void setOffsetEvent(ChainedEvent offset) {
      this.offset = offset;
      if (offset != null)
         offset.setText(Constants.OFFSET);
   }

   @Override
public String toString() {
      StringBuffer buffer = new StringBuffer("ChainedEventsCollection: ");
      buffer.append("\n ONSET: ");
      buffer.append(onset);
      buffer.append("\n INITIAL HOLD: ");
      buffer.append(initialHold);
      buffer.append("\n VALUE: ");
      buffer.append(value);
      buffer.append("\n TEXT VALUE: ");
      buffer.append(textValue);
      buffer.append("\n FINAL HOLD: ");
      buffer.append(finalHold);
      buffer.append("\n OFFSET: ");
      buffer.append(offset);

      return buffer.toString();
   }
}
