package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

/**
 * The nom-manual fields fall into two categories: anatomical and functional.
 * 
 * Anatomical fields refer to particular physical muscle movement.
 * Functional/interpretive fields provide a functional interpretation for a
 * particular movement or set of movements.
 * */

public class NonManualField extends Field {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected ChainedEventsEntityCollection collection = new ChainedEventsEntityCollection(new ArrayList<ChainedEventsEntity>(), segmentPanel);

	public NonManualField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		Track track = new Track();
		createTrackEntity(track);
	}

	public NonManualField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public NonManualField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		ChainedEventsEntity cEntity = (ChainedEventsEntity) entity;
		cEntity.setField(this);
		collection.addEntity(cEntity);
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		return collection.getEntity(entityID);
	}

	// return ChainedEventsEntities
	@Override
	public ArrayList getEntities() {
		return collection.getEntities();
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		return collection.getEntityWithRefID(refEntityId);
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		if (!highlight) {
			ArrayList<ChainedEventsEntity> entities = collection.getEntities();
			for (ChainedEventsEntity entity : entities) {
				entity.highlight(false);
			}
		} else {
			EventsEntity entity = collection.getEntityWithRefID(refID);
			if (entity != null) {
                entity.highlight(true);
            }
		}
	}

	@Override
	public int getFieldSize() {
		return collection.size();
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
		collection.deleteEntity((ChainedEventsEntity)entity);
	}

	public Track getTrack() {
		return track;
	}

	@Override
	public EventsEntity getEntityAt(double x) {
		return collection.getChainedEventsEntityAt(x);
	}

	@Override
	public Event getEventAt(double x) {
		return collection.getEventAt(x);
	}

	@Override
	public EventsEntity getSelectedEntity() {
		return collection.getSelectedEntity();
	}

	@Override
	public double getStartTime() {
		if (collection == null)
			return 0;
		return collection.getStartTime();
	}

	@Override
	public double getEndTime() {
		if (collection == null)
			return 0;
		return collection.getEndTime();
	}

	@Override
	public double getStartTimeCoordinate() {
		if (collection == null)
			return 0;
		return collection.getStartTimeCoordinate();
	}

	@Override
	public double getEndTimeCoordinate() {
		if (collection == null)
			return 0;
		return collection.getEndTimeCoordinate();
	}

	@Override
	public void unselect() {
		collection.unselect();
	}

	public void reset() {
		collection.reset();
	}

	public void setNewChainedEvent(NewChainedEvent event) {
		collection.setNewChainedEvent(event);
	}

	public boolean canDrawNewEvent(double x) {
		if (contains(x)) {
			ChainedEvent event = collection.getEventAt(x);
			return (event == null);
		}

		return false;
	}

	public void selectEntity(int x) {
		if (contains(x)) {
            collection.selectEntity(x);
        }
	}

	@Override
	public Event getSelectedEvent() {
		return collection.getSelectedEvent();
	}

	public NewChainedEvent getNewChainedEvent() {
		return collection.getNewChainedEvent();
	}

	@Override
	public String getFieldID() {
		if (track != null)
			return track.getFieldId();

		if (ss3Track != null)
			return ss3Track.getId();

		return null;
	}

	protected void createTrackEntity(SS3Track track) {
		ArrayList chainedEvents = new ArrayList();
		ArrayList<SS3Entity> entities = track.getEntities();
        for (SS3Entity entity : entities) {
            ChainedEventsCollection collection = new ChainedEventsCollection();

            ArrayList<SS3Item> items = entity.getItemsAsList();

            for (SS3Item item : items) {
                String id = item.getId();

                if (id.equalsIgnoreCase("" + Constants.NONMANUAL_ONSET_CODE)) {
                    collection.setOnsetEvent(new ChainedEvent(item, segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.NONMANUAL_OFFSET_CODE)) {
                    collection.setOffsetEvent(new ChainedEvent(item, segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.NONMANUAL_INITIAL_HOLD_CODE)) {
                	collection.setInitialHoldEvent(new ChainedEvent(item, segmentPanel));
                } else if (id.equalsIgnoreCase("" + Constants.NONMANUAL_FINAL_HOLD_CODE)) {
                	collection.setFinalHoldEvent(new ChainedEvent(item, segmentPanel));
                }
                else {
                	String fieldID = track.getId();

                    if (fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID) || fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
                        collection.setTextValueEvent(new GlossChainedEvent(item, segmentPanel));
                    } else {
                        collection.setValueEvent(new ChainedEvent(item, segmentPanel));
                    }
                }
            }

            ChainedEventsEntity cEntity = new ChainedEventsEntity(collection, segmentPanel, this);
            cEntity.setID(entity.getID());
            cEntity.setField(this);
            cEntity.setRefEntityId(entity.getRefEntityId());
            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
            	cEntity.updateParentEntity();
            } else {
            	cEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
            }

            boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
            cEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);
            chainedEvents.add(cEntity);
        }

		collection = new ChainedEventsEntityCollection(chainedEvents, segmentPanel);
	}

	@Deprecated	// legacy code involving Track, not SS3Track
	private void createTrackEntity(Track track) {
//		ArrayList<ChainedEventsEntity> chainedEvents = new ArrayList<ChainedEventsEntity>();
//		ArrayList trackValues = track.getTrackValues();
//
//		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
//		HashMap<String, TrackValue> onset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> onsetHold = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offsetHold = new HashMap<String, TrackValue>();
//
//        for (Object trackValue1 : trackValues) {
//            TrackValue trackValue = (TrackValue) trackValue1;
//            int valueId = trackValue.getValueId();
//
//            if (valueId == Constants.NONMANUAL_HOLD_CODE) {
//               double startFrame = trackValue.getStartTimeInfo().doubleValue();
//                double endFrame = trackValue.getEndTimeInfo().doubleValue();
//                onsetHold.put("" + endFrame, trackValue);
//                offsetHold.put("" + startFrame, trackValue);
//            } else if (valueId == Constants.NONMANUAL_ONSET_CODE) {
//                double endFrame = trackValue.getEndTimeInfo().doubleValue();
//                onset.put("" + endFrame, trackValue);
//            } else if (valueId == Constants.NONMANUAL_OFFSET_CODE) {
//                double startFrame = trackValue.getStartTimeInfo().doubleValue();
//                offset.put("" + startFrame, trackValue);
//            } else {
//                values.add(trackValue);
//            }
//        }
//
//        for (TrackValue trackValue : values) {
//            double startingFrame = trackValue.getStartTimeInfo().doubleValue();
//            double endingFrame = trackValue.getEndTimeInfo().doubleValue();
//
//            TrackValue onsetHoldValue = onsetHold.get("" + startingFrame);
//            TrackValue offsetHoldValue = offsetHold.get("" + endingFrame);
//            TrackValue onsetValue, offsetValue;
//
//            if (onsetHoldValue == null) {
//                onsetValue = onset.get("" + startingFrame);
//            } else {
//                onsetValue = onset.get(""
//                        + onsetHoldValue.getStartTimeInfo().doubleValue());
//            }
//
//            if (offsetHoldValue == null) {
//                offsetValue = onset.get("" + endingFrame);
//            } else {
//                offsetValue = onset.get(""
//                        + offsetHoldValue.getEndTimeInfo().doubleValue());
//            }
//
//            ChainedEventsCollection collection = new ChainedEventsCollection();
//            collection
//                    .setOnsetEvent(new ChainedEvent(onsetValue, segmentPanel));
//            collection.setInitialHoldEvent(new ChainedEvent(onsetHoldValue,
//                    segmentPanel));
//            collection
//                    .setValueEvent(new ChainedEvent(trackValue, segmentPanel));
//            collection.setFinalHoldEvent(new ChainedEvent(offsetHoldValue,
//                    segmentPanel));
//            collection.setOffsetEvent(new ChainedEvent(offsetValue,
//                    segmentPanel));
//
//            ChainedEventsEntity chainedEvent = new ChainedEventsEntity(collection, segmentPanel, this);
//            chainedEvent.determineParentEntity();
//
//            chainedEvent.setField(this);
//            chainedEvents.add(chainedEvent);
//        }
//
//		collection = new ChainedEventsEntityCollection(chainedEvents,
//				segmentPanel);
	}

	public void addChainedEventEntity(ChainedEventsEntity entity) {
		entity.setField(this);
		collection.addEntity(entity);
        SS3Singleton.getMediaToolBar().select(entity.getFirstEvent(), entity);
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);
		collection.paint(g2d, Y, font);
	}
	
	public ChainedEventsEntity getNextEntity(Time endTimeInfo) {
		return collection.getNextEntity(endTimeInfo);
	}
	
	public ChainedEventsEntity getPreviousEntity(Time startTimeInfo) {
		return collection.getPreviousEntity(startTimeInfo);
	}

	@Override
	public void deleteEmptyEntities() {
		collection.removeEmptyEntities();
		
		collection.deleteInvalidEntities(getFieldID());		
	}

    @Override
    public boolean isIntervalEmpty(Interval interval) {
	    return collection.getEntitiesInInterval(interval).isEmpty();
    }
}
