package edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

public class LockUnlockDomNdomHandAlignmentsListener implements ActionListener {

	@Override
	public void actionPerformed(ActionEvent e) {
		
		GlossChainedEventsEntity selectedEntity  = (GlossChainedEventsEntity)SS3Singleton.getMediaToolBar(null).selectedEntity();
		
		boolean locked = !selectedEntity.isLockedDomNdomAlignments();
		GlossChainedEventsEntity ndEntity = selectedEntity
				.getDependentGlossEntity();

		// if we are at this point, both entities are guaranteed to exist, so enforce that they both are aware of the other hand
		selectedEntity.getGlossEntityProperties().setTwoHanded(true);
		ndEntity.getGlossEntityProperties().setTwoHanded(true);

		// move to a locked state
		if (locked) {
			// check if possible to perform the action
			GlossField ndField = (GlossField) ndEntity.getField();
			ArrayList<GlossChainedEventsEntity> ndEntities = ndField.getEntitiesInInterval(selectedEntity.interval(), Bound.EXCLUSIVE);
			if (!ndEntities.isEmpty()) {
				boolean cannotLock = false;
				for (GlossChainedEventsEntity entity: ndEntities) {
					if (entity != null && !entity.equals(ndEntity)) {
						cannotLock = true;
					}
				}
				
				// warning!!!
				if (cannotLock) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CANT_LOCK_GESTURES);
					return;
				}
			}
			
			// all good to proceed, do the update by setting the times
			update(selectedEntity, ndEntity);
		}

		ndEntity.setLockedDomNdomAlignments(locked);
		selectedEntity.setLockedDomNdomAlignments(locked);
		SS3Singleton.getSignStreamApplication()
				.getSignStreamSegmentPanel().repaint();
	}

	private void update(GlossChainedEventsEntity selectedEntity,
			GlossChainedEventsEntity ndEntity) {

		ChainedEvent value = selectedEntity.getTextValue();
		ChainedEvent onset = selectedEntity.getOnset();
		ChainedEvent initialHold = selectedEntity.getInitialHold();
		ChainedEvent offset = selectedEntity.getOffset();
		ChainedEvent finalHold = selectedEntity.getFinalHold();

		ChainedEvent ndValue = ndEntity.getTextValue();
		ChainedEvent ndOnset = ndEntity.getOnset();
		ChainedEvent ndInitialHold = ndEntity.getInitialHold();
		ChainedEvent ndOffset = ndEntity.getOffset();
		ChainedEvent ndFinalHold = ndEntity.getFinalHold();

		ndValue.setStartTimeInfo(value.getStartTimeInfo());
		ndValue.setEndTimeInfo(value.getEndTimeInfo());

		// order matters for the below
		if (initialHold != null) {
			if (ndInitialHold == null) {
				ndEntity.addInitialHoldEvent();
				ndInitialHold = ndEntity.getInitialHold();
			}
			ndInitialHold.setStartTimeInfo(initialHold.getStartTimeInfo());
			ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo());
		} else {
			ndEntity.deleteInitialHold();
		}

		if (onset != null) {
			if (ndOnset == null) {
				ndEntity.addOnsetEvent();
				ndOnset = ndEntity.getOnset();
			}
			ndOnset.setStartTimeInfo(onset.getStartTimeInfo());
			ndOnset.setEndTimeInfo(onset.getEndTimeInfo());
		} else {
			ndEntity.deleteOnset();
		}

		if (finalHold != null) {
			if (ndFinalHold == null) {
				ndEntity.addFinalHoldEvent();
				ndFinalHold = ndEntity.getFinalHold();
			}
			ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo());
			ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo());
		} else {
			ndEntity.deleteFinalHold();
		}

		if (offset != null) {
			if (ndOffset == null) {
				ndEntity.addOffsetEvent();
				ndOffset = ndEntity.getOffset();
			}
			ndOffset.setStartTimeInfo(offset.getStartTimeInfo());
			ndOffset.setEndTimeInfo(offset.getEndTimeInfo());
		} else {
			ndEntity.deleteOffset();
		}

		if (ndEntity.isLockedGlossPhonAlignments()) {

			ChainedEventsEntity depHandEntity = ndEntity
					.getDependentHandEntity();
			HandShapeEntity depHandShapeEntity = ndEntity
					.getDependentHandShapeEntity();

			Event valueOrTextEvent = ndEntity.getValueOrTextValueEvent();
			Event startTimeEvent = ndEntity.getInitialHold();
			startTimeEvent = startTimeEvent != null ? startTimeEvent : valueOrTextEvent;
			Event endTimeEvent = ndEntity.getFinalHold();
			endTimeEvent = endTimeEvent != null ? endTimeEvent : valueOrTextEvent;

			Time startTimeInfo = startTimeEvent.getStartTimeInfo();
			Time endTimeInfo = endTimeEvent.getEndTimeInfo();

			// NOTE: we are not setting start/end times according to onsets/offsets as the handshapes do not track to those times

			if (depHandEntity != null) {
				depHandEntity.getValueOrTextValueEvent().setStartTimeInfo(startTimeInfo);
				depHandEntity.getValueOrTextValueEvent().setEndTimeInfo(endTimeInfo);

				if(depHandShapeEntity != null) {
					MediaToolbarCommons.setHandshapeStartTime(depHandShapeEntity, startTimeInfo);
					MediaToolbarCommons.setHandshapeEndTime(depHandShapeEntity, endTimeInfo);
				}
			}
		}
	}
}
