package edu.bu.signstream.grepresentation.fields.palettes;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.logging.Level;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GenericStartEndHandShapeTypes;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.PaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.PaletteUIFactory;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GesturePaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GlossPaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class GeneralPaletteDialog<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends JDialog implements ActionListener, ItemListener {

    private static final long serialVersionUID = 8045326414761820183L;
    private final SS3CodingScheme scheme;
    private final PaletteInformationHelper paletteInformationHelper;

    private final JPanel topPanel;

    private final HandShapesPaletteFormBean formBean;
    private final HandShapeFormBean bean;
    private final JDialog parentDialog;
    private final ParentPaletteInformationHelper<K> parentHandShapePanel;
    private final T selectionPanel;
    private JScrollPane selectedHandshapesScrollPane;

    private JButton clearButton;
    private JPanel clearButtonPanel;
    private JPanel bottomPanel;
    private JButton enterButton;
    private JButton cancelButton;

    private final String paletteGroupID;
    private final String paletteID;
    private final String defaultMenuID;

    private final String CANCEL = "CANCELBUTTON";
    private final String ENTER = "ENTERBUTTON";


    public GeneralPaletteDialog(HandShapesPaletteFormBean formBean, HandShapeFormBean bean, SS3CodingScheme scheme,
                                JDialog parentDialog, ParentPaletteInformationHelper<K> parentHandShapePanel, T selectionPanel, String defaultMenuID, String paletteGroupID, String paletteID) {
        super(parentDialog, "Palette", true); // True does not allow the previous dialog (the info panel) to be clicked on while this dialog is open.
        super.setLocationRelativeTo(parentDialog);
        super.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        super.setResizable(true);
        super.setLayout(new BorderLayout());

        this.scheme = scheme;
        this.formBean = formBean;
        this.bean = bean;
        this.parentHandShapePanel = parentHandShapePanel;
        this.selectionPanel = selectionPanel;
        this.parentDialog = parentDialog;
        this.defaultMenuID = defaultMenuID;
        this.paletteGroupID = paletteGroupID;
        this.paletteID = paletteID;
        this.paletteInformationHelper = new PaletteInformationHelper(this.scheme, this.defaultMenuID);

        this.topPanel = this.initializeTopPanel();

        if (this.topPanel == null) {
            SS3SignStreamApplication.logger.log(Level.SEVERE, "XML Palettes have some inconsistencies. Please check the XML templates and palettes related structure IDs.");
            return;
        }

        this.bottomPanel = this.initializeBottomPanel();

        this.add(this.topPanel, BorderLayout.NORTH);
        this.add(this.bottomPanel, BorderLayout.SOUTH);

        this.pack();
    }

    private JPanel initializeTopPanel() {
        JPanel topPanel = new JPanel(); // This top panel will contain the palette menu and palette group comboboxes.
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
        topPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        JLabel palettes = new JLabel("Palettes: ");
        topPanel.add(palettes);
        JComboBox<String> templateStandardComboBox = PaletteUIFactory.createComboBox(this.paletteInformationHelper.getTemplateData());
        topPanel.add(templateStandardComboBox);

        if (this.paletteInformationHelper.getInconsistentXML()) {
            return null;
        }

        JComboBox<String> paletteItemGroupOptionComboBox;
        if (this.paletteInformationHelper.getCurrentTemplateID() != null) {
            // This line determines the menu group for handshapes, depending on whether the parent is a gloss or gesture.
            paletteItemGroupOptionComboBox = PaletteUIFactory.createComboBox(
                    this.paletteInformationHelper.getPaletteItemGroupOptions(this.paletteInformationHelper.getCurrentTemplateID(),
                            this.paletteGroupID, this.parentHandShapePanel.isDomHandSelected(), this.parentHandShapePanel.isNondomHandSelected())
            ); // populateTemplateStandardComboBox will set the default template id.

            String selectedPaletteName = this.paletteInformationHelper.getPaletteNameFromID(this.paletteID);

            if (selectedPaletteName != null) {
                if (this.parentHandShapePanel.isDomHandSelected()) {
                    this.paletteInformationHelper.setCurrentDomPaletteID(this.paletteID);
                } else if (this.parentHandShapePanel.isNondomHandSelected()) {
                    this.paletteInformationHelper.setCurrentNDomPaletteID(this.paletteID);
                }
                paletteItemGroupOptionComboBox.setSelectedItem(this.paletteInformationHelper.getPaletteNameFromID(this.paletteID));
            }
        } else {
            throw new NullPointerException("Default template ID is null.");
        }

        paletteItemGroupOptionComboBox.addItemListener(this);

        topPanel.add(paletteItemGroupOptionComboBox);

        return topPanel;
    }

    private JPanel createSelectionPanel() {
        JPanel selectionClearPanel = new JPanel(new GridBagLayout());

        // Set the preferred size for the selectionPanel
        Dimension d = new Dimension(super.getPreferredSize().width, this.selectionPanel.getPreferredSize().height);
        this.selectionPanel.setPreferredSize(d);
        this.selectionPanel.setMinimumSize(d);

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets = new Insets(10, 0, 0, 0);
        constraints.fill = GridBagConstraints.BOTH;
        constraints.weightx = 1.0;
        constraints.gridx = 0;
        constraints.gridy = 0;

        this.selectedHandshapesScrollPane = new JScrollPane(this.selectionPanel);
        this.selectedHandshapesScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        this.selectedHandshapesScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        this.selectedHandshapesScrollPane.setMinimumSize(d);

        // Add JScrollPane containing selectionPanel to the main panel
        selectionClearPanel.add(this.selectedHandshapesScrollPane, constraints);

        // Create the clear button and its panel
        this.clearButton = new JButton("Clear");
        this.clearButton.addActionListener(e -> {
            this.selectionPanel.removeAll();
            this.selectionPanel.revalidate(); // You may need to add this
            this.selectionPanel.repaint();    // and this to ensure UI updates correctly after removing all components
        });

        this.clearButtonPanel = new JPanel();
        this.clearButtonPanel.add(this.clearButton);

        // Add the clearButtonPanel to the main panel below the JScrollPane
        constraints.weightx = 0;
        constraints.weighty = 0; // Try setting this to a non-zero value, if the button needs more vertical space.
        constraints.gridy = 1; // Place the clear button below the JScrollPane
        selectionClearPanel.add(this.clearButtonPanel, constraints);

        return selectionClearPanel;
    }


    private JPanel initializeBottomPanel() {
        JPanel bottomContainerPanel = new JPanel();
        bottomContainerPanel.setLayout(new BoxLayout(bottomContainerPanel, BoxLayout.Y_AXIS));

        JPanel selectionClearPanel = this.createSelectionPanel();
        bottomContainerPanel.add(selectionClearPanel);

        JPanel bottomButtonPanel = new JPanel();
        this.enterButton = new JButton("Enter");
        this.enterButton.setActionCommand(this.ENTER);
        this.enterButton.addActionListener(this);

        this.cancelButton = new JButton("Cancel");
        this.cancelButton.setActionCommand(this.CANCEL);
        this.cancelButton.addActionListener(this);

        bottomButtonPanel.add(this.cancelButton);
        bottomButtonPanel.add(this.enterButton);

        bottomContainerPanel.add(bottomButtonPanel);

        return bottomContainerPanel;
    }


    /**
     * Invoked when an action occurs.
     *
     * @param e the event to be processed
     */
    @Override
    public void actionPerformed(ActionEvent e) {}

    public SS3CodingScheme getScheme() {
        return this.scheme;
    }

    public PaletteInformationHelper getPaletteInformationHelper() {
        return this.paletteInformationHelper;
    }

    public HandShapesPaletteFormBean getFormBean() {
        return this.formBean;
    }

    public HandShapeFormBean getBean() {
        return this.bean;
    }

    public JDialog getParentDialog() {
        return this.parentDialog;
    }

    public ParentPaletteInformationHelper<K> getParentHandShapePanel() {
        return this.parentHandShapePanel;
    }

    public T getSelectionPanel() {
        return this.selectionPanel;
    }

    public String getPaletteID() {
        return this.paletteID;
    }

    public String getDefaultMenuID() {
        return this.defaultMenuID;
    }

    public String getCANCEL() {
        return this.CANCEL;
    }

    public String getENTER() {
        return this.ENTER;
    }

    public JButton getEnterButton() {
    	return this.enterButton;
    }

    public JButton getCancelButton() {
    	return this.cancelButton;
    }

    public JPanel getBottomPanel() {
    	return this.bottomPanel;
    }

    public JPanel getTopPanel() {
    	return this.topPanel;
    }

    /**
     * Invoked when an item has been selected or deselected by the user.
     * The code written for this method performs the operations
     * that need to occur when an item is selected (or deselected).
     *
     * @param e the event to be processed
     */
    @Override
    public void itemStateChanged(ItemEvent e) {}
}
