package edu.bu.signstream.grepresentation.fields.palettes.doublePalettes;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GenericStartEndHandShapeTypes;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.SelectedGestureHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.GeneralHandShapePalettePanel;
import edu.bu.signstream.grepresentation.fields.palettes.LettersPalette;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.PaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GlossPaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.util.ArrayList;

public class GlossDoublePaletteDialog<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends GeneralDoublePaletteDialog<T, K> {
    private GeneralHandShapePalettePanel<T, K> rightHandPanel;
    private GeneralHandShapePalettePanel<T, K> leftHandPanel;
    private final T domHandSelectionPanel;
    private final T nonDomHandSelectionPanel;
    private JPanel centerPanel;

    public GlossDoublePaletteDialog(HandShapesPaletteFormBean domHandFormBean, HandShapesPaletteFormBean nonDomHandFormBean, HandShapeFormBean bean, ParentPaletteInformationHelper<K> parentInformationPanel, T domHandSelectionPanel, T nonDomHandSelectionPanel, SS3CodingScheme scheme, JDialog parentDialog, String defaultPaletteMenuID, String selectedPaletteID) {
        super(domHandFormBean, nonDomHandFormBean, bean, parentInformationPanel, domHandSelectionPanel, nonDomHandSelectionPanel, scheme, parentDialog, defaultPaletteMenuID, selectedPaletteID);

        this.domHandSelectionPanel = domHandSelectionPanel;
        this.nonDomHandSelectionPanel = nonDomHandSelectionPanel;

        this.centerPanel = this.initializeCenterPanel();

        this.centerPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            @Override
			public void componentResized(java.awt.event.ComponentEvent evt) {
                GlossDoublePaletteDialog.this.setSelectionPanelSizeBounds();
            }
        });

        this.addComboBoxListeners();
        this.addBottomPanelListeners();

        super.add(this.centerPanel, BorderLayout.CENTER);

        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();
    }

    protected void addBottomPanelListeners() {
        JButton cancelButton = super.getCancelButton();
        JButton enterButton = super.getEnterButton();

        cancelButton.addActionListener(e -> {
            this.dispose();
        });

        enterButton.addActionListener(e -> {
            super.getBean().setUpdated(true);

            ArrayList<HandShapeImage> rightHandImages = new ArrayList<>(super.getDomHandSelectionPanel().getHandshapes());
            ArrayList<HandShapeImage> leftHandImages = new ArrayList<>(super.getNonDomHandSelectionPanel().getHandshapes());
            GlossPaletteInformationHelper parentPaletteInformationPanel = (GlossPaletteInformationHelper) super.getParentPaletteInformationPanel();

            if (parentPaletteInformationPanel.isOneFrameEntity()) {
                if (rightHandImages.size() > 1 || leftHandImages.size() > 1) {
                    SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_EXACTLY_ONE);
                    return;
                }
            } else if ((parentPaletteInformationPanel.getSelectedSameDifStartEndHandshapesDom() == StartEndHandshapesType.DIFFERENT && rightHandImages.size() == 1)
                    || (parentPaletteInformationPanel.getSelectedSameDifStartEndHandshapesNdom() == StartEndHandshapesType.DIFFERENT && leftHandImages.size() == 1) ) {
                // conversely, if this is not a one-frame gloss and you have dif start/end selected,
                // then disallow a selection of just one handshape in either hand
                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_MULTIPLE);
                return;
            }

            parentPaletteInformationPanel.updateParentInfoSelectedHandshapes(rightHandImages, leftHandImages);

            this.dispose();
        });
    }

    private void setSelectionPanelSizeBounds() {
        Dimension rightHandDimension = new Dimension(this.rightHandPanel.getPreferredSize().width, this.domHandSelectionPanel.getPreferredSize().height);
        Dimension leftHandDimension = new Dimension(this.leftHandPanel.getPreferredSize().width, this.nonDomHandSelectionPanel.getPreferredSize().height);

        this.domHandSelectionPanel.setPreferredSize(rightHandDimension);
        this.domHandSelectionPanel.setMinimumSize(new Dimension(0, rightHandDimension.height));
        this.domHandSelectionPanel.setSize(rightHandDimension);

        this.nonDomHandSelectionPanel.setPreferredSize(leftHandDimension);
        this.nonDomHandSelectionPanel.setMinimumSize(new Dimension(0, leftHandDimension.height));
        this.nonDomHandSelectionPanel.setSize(leftHandDimension);

        this.domHandSelectionPanel.revalidate();
        this.domHandSelectionPanel.repaint();

        this.nonDomHandSelectionPanel.revalidate();
        this.nonDomHandSelectionPanel.repaint();

        super.getBottomPanel().revalidate();
        super.getBottomPanel().repaint();
    }

    private JPanel initializeCenterPanel() {
        JPanel handShapePanel = new JPanel();
        handShapePanel.setLayout(new BoxLayout(handShapePanel, BoxLayout.X_AXIS));
        handShapePanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        String domHandPaletteID = super.getRightHandFormBean().isRightHandSelected() ? super.getPaletteInformationHelper().getCurrentDomPaletteID() : super.getPaletteInformationHelper().getCurrentNDomPaletteID();
        String nDomHandPaletteID = super.getRightHandFormBean().isRightHandSelected() ? super.getPaletteInformationHelper().getCurrentNDomPaletteID() : super.getPaletteInformationHelper().getCurrentDomPaletteID();

        HandShapesPaletteFormBean rightHandFormBean = super.getRightHandFormBean();
        HandShapesPaletteFormBean leftHandFormBean = super.getLeftHandFormBean();
        SS3CodingScheme scheme = super.getScheme();
        PaletteInformationHelper paletteInformationHelper = super.getPaletteInformationHelper();
        ParentPaletteInformationHelper parentPaletteInformationPanel = super.getParentPaletteInformationPanel();

        if (super.getPaletteInformationHelper().getCurrentDomPaletteID().equals(Util.LETTERS_PALETTE_ID)) {
            this.rightHandPanel = new LettersPalette<>(rightHandFormBean, scheme, parentPaletteInformationPanel, this.domHandSelectionPanel, paletteInformationHelper.getCurrentTemplateID(), Util.LETTERS_PALETTE_ID);
            var selectionPanel = this.domHandSelectionPanel;
            if (selectionPanel instanceof SelectedHandshapesPanel) {
            	SelectedHandshapesPanel shp = (SelectedHandshapesPanel) selectionPanel;
            	shp.setLettersPalette((LettersPalette)this.rightHandPanel);
            } else if (selectionPanel instanceof SelectedGestureHandshapesPanel) {
            	SelectedGestureHandshapesPanel sghp = (SelectedGestureHandshapesPanel) selectionPanel;
            	sghp.setLettersPalette((LettersPalette)this.rightHandPanel);
            }
        } else {
            this.rightHandPanel = new GeneralHandShapePalettePanel<>(rightHandFormBean, scheme, parentPaletteInformationPanel, this.domHandSelectionPanel, paletteInformationHelper.getCurrentTemplateID(), domHandPaletteID);
        }

        this.leftHandPanel = new GeneralHandShapePalettePanel<>(leftHandFormBean, scheme, parentPaletteInformationPanel, this.nonDomHandSelectionPanel, paletteInformationHelper.getCurrentTemplateID(), nDomHandPaletteID);

        handShapePanel.add(this.rightHandPanel);
        handShapePanel.add(this.leftHandPanel);

        this.setSelectionPanelSizeBounds();

        return handShapePanel;
    }

    protected void addComboBoxListeners() {
        super.getRightHandComboBox().addItemListener(e -> {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                String currentItem = (String) e.getItem();
                super.getPaletteInformationHelper().setCurrentDomPaletteID(super.getPaletteID(currentItem));
                int rightHandPanelIndex = this.centerPanel.getComponentZOrder(this.rightHandPanel);
                this.centerPanel.remove(rightHandPanelIndex);
                this.rightHandPanel = this.getNewPanel(true);
                this.updateCenterPanel(this.rightHandPanel, rightHandPanelIndex);
            }
        });

        super.getLeftHandComboBox().addItemListener(e -> {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                String currentItem = (String) e.getItem();
                super.getPaletteInformationHelper().setCurrentNDomPaletteID(super.getPaletteID(currentItem));
                int leftHandPanelIndex = this.centerPanel.getComponentZOrder(this.leftHandPanel);
                this.centerPanel.remove(leftHandPanelIndex);
                this.leftHandPanel = this.getNewPanel(false);
                this.updateCenterPanel(this.leftHandPanel, leftHandPanelIndex);
            }
        });
    }

    protected GeneralHandShapePalettePanel<T, K> getNewPanel(boolean domHand) {
        GeneralHandShapePalettePanel<T, K> newPanel;

        SS3CodingScheme scheme = super.getScheme();
        HandShapesPaletteFormBean rightHandFormBean = super.getRightHandFormBean();
        HandShapesPaletteFormBean leftHandFormBean = super.getLeftHandFormBean();
        PaletteInformationHelper paletteInformationHelper = super.getPaletteInformationHelper();
        ParentPaletteInformationHelper<K> parentPaletteInformationHelper = super.getParentPaletteInformationPanel();


        HandShapesPaletteFormBean handShapesPaletteFormBean = domHand ? rightHandFormBean : leftHandFormBean;
        T selectionPanel = domHand ? this.domHandSelectionPanel : this.nonDomHandSelectionPanel;
        String paletteID = domHand ? paletteInformationHelper.getCurrentDomPaletteID() : paletteInformationHelper.getCurrentNDomPaletteID();
        String paletteInformationHelperPaletteID = domHand ? paletteInformationHelper.getCurrentDomPaletteID() : paletteInformationHelper.getCurrentNDomPaletteID();

        if (paletteInformationHelperPaletteID.equals(Util.LETTERS_PALETTE_ID)) {
            newPanel = new LettersPalette<>(handShapesPaletteFormBean, scheme, parentPaletteInformationHelper, selectionPanel, paletteInformationHelper.getCurrentTemplateID(), paletteID);
            if (selectionPanel instanceof SelectedHandshapesPanel) {
            	SelectedHandshapesPanel shp = (SelectedHandshapesPanel) selectionPanel;
            	shp.setLettersPalette((LettersPalette) newPanel);
            } else if (selectionPanel instanceof SelectedGestureHandshapesPanel) {
            	SelectedGestureHandshapesPanel sghp = (SelectedGestureHandshapesPanel) selectionPanel;
            	sghp.setLettersPalette((LettersPalette) newPanel);
            }
        } else {
            newPanel = new GeneralHandShapePalettePanel<>(handShapesPaletteFormBean, scheme, parentPaletteInformationHelper, selectionPanel, paletteInformationHelper.getCurrentTemplateID(), paletteID);
        }

        return newPanel;
    }

    protected void updateCenterPanel(GeneralHandShapePalettePanel<T, K> panel, int indexToAdd) {
        panel.revalidate();
        panel.repaint();
        panel.setSize(panel.getPreferredSize());

        this.centerPanel.add(panel, indexToAdd);
        this.centerPanel.revalidate();
        this.centerPanel.repaint();
        this.centerPanel.setSize(this.centerPanel.getPreferredSize());

        super.revalidate();
        super.repaint();
        super.setSize(super.getPreferredSize());

        System.gc();
    }

    public GeneralHandShapePalettePanel<T, K> getRightHandPanel() {
        return this.rightHandPanel;
    }

    public GeneralHandShapePalettePanel<T, K> getLeftHandPanel() {
        return this.leftHandPanel;
    }

    @Override
	public T getDomHandSelectionPanel() {
        return this.domHandSelectionPanel;
    }

    @Override
	public T getNonDomHandSelectionPanel() {
        return this.nonDomHandSelectionPanel;
    }

    public JPanel getCenterPanel() {
        return this.centerPanel;
    }

    public void setRightHandPanel(GeneralHandShapePalettePanel<T, K> rightHandPanel) {
        this.rightHandPanel = rightHandPanel;
    }

    public void setLeftHandPanel(GeneralHandShapePalettePanel<T, K> leftHandPanel) {
        this.leftHandPanel = leftHandPanel;
    }

    protected void setCenterPanel(JPanel centerPanel) {
        super.remove(centerPanel);
        this.centerPanel = centerPanel;
        super.add(this.centerPanel, BorderLayout.CENTER);
        this.centerPanel.revalidate();
        this.centerPanel.setSize(this.centerPanel.getPreferredSize());
        this.centerPanel.repaint();
        super.revalidate();
        super.repaint();
    }
}
