package edu.bu.signstream.grepresentation.fields.presentation;

import java.awt.Cursor;
import java.awt.Point;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.*;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.Manager;
import edu.bu.signstream.grepresentation.requestsHandler.RequestHandler;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SegmentFieldWrapper extends FieldWrapper {
	private PresentationField prField = null;
	private String partitionLabel = null;
	private String segmentLabel = null;

	public SegmentFieldWrapper(PresentationField field,
			SignStreamSegmentPanel segmentPanel, String partitionLabel,
			String segmentLabel, TimeRange timeRange) {
		super(field, segmentPanel, timeRange);
		// super.setType(Constants.SegmentFieldWrapper); <------ We might need to add this back in case something breaks with setTime of gestures...
		this.prField = field;
		this.partitionLabel = partitionLabel;
		this.segmentLabel = segmentLabel;
	}

	public void removeAllEntities() {
		prField.removeAllEntities();
	}

	public void setPartitionLabel(String partitionLabel) {
		this.partitionLabel = partitionLabel;
	}

	public String getPartitionLabel() {
		return partitionLabel;
	}

	public void setSegmentLabel(String segmentLabel) {
		this.segmentLabel = segmentLabel;
	}

	public String getSegmentLabel() {
		return segmentLabel;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	public boolean canMoveEvent(Event event, int x) {
		return false;
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		prField.addEventsEntity(entity);
	}

	// MouseListener
	@Override
	public void mouseClicked(Point point, JPanel panel) {
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()))
            .clamp(SS3Singleton.getMediaPlayer().interval());

        Event sEvent = field.getEventAt(point.x);

        if (!isEntitySelected() && sEvent == null && time.lessThan(SS3Singleton.getMediaPlayer().end())) {
            NewEvent event = new NewEvent(time, segmentPanel);
            if (prField.getNextEntity(event.getStartTimeInfo()) != null) {
                // If this entity has an event in front of it, set a reference to it
                event.setNextEntity(prField.getNextEntity(event.getStartTimeInfo()));
            } else {
                // If this entity has no event in front of it, create a temporary "boundary" event at
                // the end of the movie duration to prevent this new event from going out of bounds.
                PresentationEvent endBound = new PresentationEvent(segmentPanel);
                endBound.setStartTimeInfo(
                    SS3Singleton.getVideoTimeRange().time(Math.round(segmentPanel.getMultipleMovieController().getMovieDuration()), TimeUnit.TIMESTAMP));
                event.setNextEntity(new PresentationEventsEntity(endBound, segmentPanel, field));
            }

            if (prField.getPreviousEntity(event.getStartTimeInfo()) != null) {
                // If this entity has an event behind it, set a reference to it
                event.setPreviousEntity(prField.getPreviousEntity(event.getStartTimeInfo()));
            } else {
                // If this entity has no event behind it, create a temporary "boundary" event at the
                // beginning of the movie to prevent this new event from going out of bounds.
                PresentationEvent startBound = new PresentationEvent(segmentPanel);
                startBound.setEndTimeInfo(SS3Singleton.getMediaPlayer().start());
                event.setPreviousEntity(new PresentationEventsEntity(startBound, segmentPanel, field));
            }
            prField.setNewEvent(event);
        }
	}

	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}

	public void mouseDblClicked(Point point) {
	}
	
	public void mouseCntrAltDblLeftClicked(Point point) {
	}

	/*
	 * Show selected utterance/segment, etc
	 */
	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {
		SS3SignStreamApplication.logger.log(Level.INFO, "Updating loaded database");
		
//		segmentPanel.updateLoadedDatabase();

		// Update saved DB view state
//		segmentPanel.databaseScroll(segmentPanel.getDatabaseView().getVisiblePoint().x);
//		segmentPanel.zoomLock(segmentPanel.getDatabaseView().getUpperLeft().locked());

//		double segmentMovieTime = segmentPanel.getMultipleMovieController().getCurrentTime();
//		int segmentOnScreenLocation = segmentPanel.getDatabaseView().getVisiblePoint().x;
		
//		SS3Database ss3 = segmentPanel.getSS3Database();

//		SS3SignStreamApplication.logger.log(Level.INFO, "Updated Loaded database");

//		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation();
//		rep.getFieldWrappersCollection().unselectAll();

		ArrayList<PresentationEventsEntity> eventsEntitiesToLoad = field.getEntities();
		
		Event event = field.getEventAt(point.x);
		var entity = field.getEntityAt(point.x);

//        SS3Singleton.getMediaToolBar().select(event, entity);
//		SS3SignStreamApplication.logger.log(Level.INFO, "Selected event " + event);
		
		if (event == null)
			return;

		PresentationEvent pEvent;

		if (event instanceof PresentationEventsEntity) {
			PresentationEventsEntity pEntity = (PresentationEventsEntity) event;
			pEvent = pEntity.getEvent();
//			SS3Singleton.getMediaToolBar().deactivate();
//			SS3Singleton.getMediaToolBar().deselect();
//			SS3Singleton.getMediaToolBar().enterUtterance(pEntity);
		} else if (event instanceof PresentationEvent) {
			pEvent = (PresentationEvent) event;
		} else {
			return;
		}
		
		SS3SignStreamApplication.logger.log(Level.INFO, "Casted event " + pEvent);
		
        var presentationEntity = (PresentationEventsEntity) entity;
//        SS3Singleton.getMediaToolBar().enterUtterance(presentationEntity);

//		ArrayList<SS3GlossWindowItem> surroundingItems = new ArrayList<SS3GlossWindowItem>();
//
//		int size = eventsEntitiesToLoad.size();
//
//		int startIndex = 0;
//		int endIndex = size;	// top bound exclusive
//
//		for (PresentationEventsEntity eventsEntity: eventsEntitiesToLoad.subList(startIndex, endIndex)) {
//			surroundingItems.add(eventsEntity.getSS3GlossWindowItem());
//		}
//
//		SS3SignStreamApplication.logger.log(Level.INFO, "Selected segment tier represents " + surroundingItems.size() + " of WindowItem objects");
//		SS3SignStreamApplication.logger.log(Level.INFO, "Loading window items");

//		SegmentGraphReprUtil.utteranceStart = pEvent.getStartTimeInfo();
//        SegmentGraphReprUtil.utteranceEnd = pEvent.getEndTimeInfo();
//        SegmentGraphReprUtil.visibleStart = pEvent.getStartTimeInfo();
//        SegmentGraphReprUtil.visibleEnd = pEvent.getEndTimeInfo();

//		segmentPanel.getSegments().clear();
//		segmentPanel.loadUtterance(prField);
//		handler.convertMultipleItemsAndAddToSegmentPanel(prField.getSS3GlossWindowSegment(), (PresentationField) field);
//		handler.loadPredefinedPanelSelection();
//		boolean displayWarningDialog = handler.displayWarningDialog();

//        var zoomer = segmentPanel.getZoomer();
//        double width = segmentPanel.getDatabaseView().getVisibleWidth2();
//        zoomer.getCoordinateTimeConvertor().resize(Math.round(width));
//
//        segmentPanel.utteranceZoom()
//            .flatMap(z -> segmentPanel.zoomLocked() ? Optional.of(z) : Optional.empty())
//            .ifPresentOrElse(
//                zoom -> segmentPanel.zoom(zoom),
//                () -> segmentPanel.fitZoom(pEvent.interval().duration()));

        segmentPanel.loadUtteranceView(prField, (PresentationEventsEntity) entity);

        Manager mngr = SS3Singleton.getManager();
        RequestHandler handler = mngr.getRequestHandler();
        handler.init(segmentPanel);
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {

	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {

		if (panel != null) {
			panel.setCursor(new Cursor(getCursorType(point)));

			Event event = prField.getEventAt(point.x);
			if (event != null) {
				panel.setToolTipText(event.getToolTipText());
			} else {
				panel.setToolTipText("");
			}
		}
	}

	@Override
	public void mouseDragged(Point point, JPanel panel) {
		var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.NEAREST)
            .clamp(SS3Singleton.getMediaPlayer().interval());

		var tb = SS3Singleton.getMediaToolBar();
		var player = SS3Singleton.getMediaPlayer();

        // Returns null if user drags normally, and not null if user is creating a new event
        var newEvent = prField.getNewEvent();
        if (newEvent != null) {
            // Returns a temporary entity representing the end of the movie if there is no event
            // after this one.
            PresentationEventsEntity nextPresEntity =
                (PresentationEventsEntity) newEvent.getNextEntity();
            // Returns a temporary entity representing the start of the movie if there is no event
            // before this one.
            PresentationEventsEntity prevPresEntity =
                (PresentationEventsEntity) newEvent.getPreviousEntity();

            // If there is an event after this one, limit this event's end time to that event's
            // start time. Otherwise, limit it to the end of the movie.
            Time max = nextPresEntity.getStartTimeInfo()
                .in(Util.getCurrentRange(getField()), Rounding.FLOOR);
            // If there is an event before this one, limit this event's start time to that event's
            // end time. Otherwise, limit it to zero to prevent a negative event start time
            Time min = prevPresEntity.getEndTimeInfo()
                .in(Util.getCurrentRange(getField()), Rounding.CEIL);

            if (time.greaterThan(max)) {
                newEvent.setEndTimeInfo(max);
                player.seek(max);
            } else if (time.lessThan(min)) {
                newEvent.setEndTimeInfo(min);
                player.seek(min);
            } else {
                newEvent.setEndTimeInfo(time);
                player.seek(time);
            }
        } else {
            var mid = (tb.selectedEvent().getStartTimeCoordinate() + tb.selectedEvent().getEndTimeCoordinate()) / 2.0;
            var parent = (PresentationEventsEntity)field.getParentField().getEntityAt(mid);

            if (MOVE_DIRECTION == MOVE_START_POINT) {
                MediaToolbarCommons.setPresentationEventStartTime(time, parent.getEvent(), parent);
                SS3Singleton.getMediaPlayer().seek(tb.selectedEvent().getStartTimeInfo());
            }
            if (MOVE_DIRECTION == MOVE_END_POINT) {
                MediaToolbarCommons.setPresentationEventEndTime(time, parent.getEvent(), parent);
                SS3Singleton.getMediaPlayer().seek(tb.selectedEvent().getEndTimeInfo());
            }
        }
	}

	private int counter = 1;

	@Override
	public void mouseReleased(Point point, JPanel panel) {
		NewEvent newEvent = prField.getNewEvent();

        if (newEvent != null) {

            SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

            var parentField = (PresentationField)prField.getParentField();
            var parentWrapper = (PresentationFieldWrapper)parentField.getWrapper();

            ArrayList<?> entities = parentField.getEntities();

            var start = newEvent.getStartTimeInfo();
            var end = newEvent.getEndTimeInfo();

            // If event was created by dragging backwards, switch the start and end coordinates
            if (start.greaterThan(end)) {
                var temp = start;
                start = end;
                end = temp;
            }

            for (int i = 0; i < entities.size(); i++) {
                PresentationEventsEntity ent = (PresentationEventsEntity) entities
                    .get(i);
                ent.unhighlight();
                ent.unselect();
                ent.getEvent().unhighlight();
                ent.getEvent().unselect();
            }

            SS3GlossWindowItem item = new SS3GlossWindowItem("", "");
            item.setId(Util.getLongUniqueNumber());

            PresentationEvent event = new PresentationEvent(item, segmentPanel);

            event.setStartTimeInfo(start);
            event.setEndTimeInfo(end);

            item.setEvent(event);

            event.select();

            // Deals with utterances overlapping or being too small
            PresentationEventsEntity nextPresEntity =
                (PresentationEventsEntity) newEvent.getNextEntity();
            PresentationEventsEntity prevPresEntity =
                (PresentationEventsEntity) newEvent.getPreviousEntity();

            if (event.getStartTimeInfo().lessThan(prevPresEntity.getEndTimeInfo())) {
                event.setStartTimeInfo(prevPresEntity.getEndTimeInfo());
            }

            if(event.getEndTimeInfo().greaterThan(nextPresEntity.getStartTimeInfo())){
                event.setEndTimeInfo(nextPresEntity.getStartTimeInfo());
//				event.getEndTimeInfo().setUnitValue(nextPresEntity.getStartTimeInfo().getUnitValueAs(convertUnit));
            }


            boolean isUnique = false;
            counter = parentField.getFieldSize() + 1;

            String label = "";
            while (!isUnique) {
                label = "U" + counter;
                isUnique = parentField.isEventLabelUnique(label);
                counter++;
            }

            event.setText(label);
            event.setStartTimeInfo(event.getStartTimeInfo());
            event.setEndTimeInfo(event.getEndTimeInfo());

            PresentationEventsEntity entity = new PresentationEventsEntity(
                event, segmentPanel, parentField);

            parentField.addEventsEntity(entity);

            item.setStartTimeInfo(event.getStartTimeInfo());
            item.setEndTimeInfo(event.getEndTimeInfo());
            item.setLabel(event.getText());
            item.setId(event.getID());

            parentField.getSS3GlossWindowSegment().addSS3GlossWindowItem(item);

            PresentationEventsEntity self = null;

            for (var wrapper : parentWrapper.getSegmentsFieldWrappers()) {

                PresentationEventsEntity clonedEntity = entity.clone();
                entity.addDependentEntity(clonedEntity);
                wrapper.addEventsEntity(clonedEntity);

                if (wrapper == this)
                    self = clonedEntity;

                SS3GlossWindowItem pItem = new SS3GlossWindowItem(wrapper.getPartitionLabel(), wrapper.getSegmentLabel());
                pItem.setId(Util.getLongUniqueNumber());
                pItem.setStartTimeInfo(item.getStartTimeInfo());
                pItem.setEndTimeInfo(item.getEndTimeInfo());
                pItem.setLabel(event.getText());
                //
                PresentationEvent pEvent = clonedEntity.getEvent();
                pEvent.setSS3GlossWindowItem(pItem);
                pItem.setEvent(pEvent);
                item.addSS3GlossWindowItem(pItem);

                PresentationField dependentPresentationField = (PresentationField) wrapper
                    .getField();
                dependentPresentationField.getSS3GlossWindowSegment()
                    .addSS3GlossWindowItem(pItem);
            }

            MediaToolBar tb = segmentPanel.getToolBarPanels().getMediaToolBar();

            self.select();
            tb.select(self.getEvent(), self);

            prField.setNewEvent(null);
        } else {
			Event event = field.getEventAt(point.x);
			if (event instanceof PresentationEvent) {
				EventsEntity entity = field.getEntityAt(point.x);
				SS3Singleton.getMediaToolBar(null).selectEvent(event, entity,
						prField);
			}
		}
	}

	private int getCursorType(Point point) {
		int cursorType = Cursor.DEFAULT_CURSOR;
		Event event = prField.getSelectedEvent();

		if (event != null) {
			double startCoord = event.getStartTimeCoordinate();
			double endCoord = event.getEndTimeCoordinate();

			if (startCoord - 3 < point.x && point.x < startCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_START_POINT;
			}

			if (endCoord - 3 < point.x && point.x < endCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_END_POINT;
			}
		}
		return cursorType;
	}

	@Override
	public void unselectAll() {
		field.unselect();
	}

	@Override
	public double getStartTime() {
		return field.getStartTime();
	}

	@Override
	public double getEndTime() {
		return field.getEndTime();
	}

	@Override
	public int getViewLineNumber() {
		if (viewLineNumber >= 0)
			return viewLineNumber;
		return Integer.parseInt(field.getFieldID());
	}

	@Override
	public Event getSelectedEvent() {
		return field.getSelectedEvent();
	}

	@Override
	public void mouseHoldReleased(Point point, JPanel panel) {
	}
}
