package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.*;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class CustomFieldsLabelView extends JPanel {

	private static final long serialVersionUID = 1L;

	private LabelViewCustomFields labelView = null;
	private SegmentGraphicRepresentation rep = null;
	private NonManualFieldsButtonPanel fieldsButtonPanel = null;
	private SegmentControlPanel controlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;

	private ArrayList<LabelObject> labels = null;

	private Segment segment;

	protected JScrollPane pictureScrollPane;
	
	private FieldWrappersCollection fwc;


	public CustomFieldsLabelView(Segment segment, SignStreamSegmentPanel panel, SegmentControlPanel controlPanel,
			CustomFieldsView customView, FieldWrappersCollection fwc) {
		this.segmentPanel = panel;
		this.segment = segment;
		this.controlPanel = controlPanel;
//		this.glossNRelatedView = glossNRelatedView;
		this.fwc = fwc;

		labels = segment.getRootLabelObject().getCustomLabelObject().getVisibleObjects();

		labels = filterLabelObjs(labels);

		for (int i = 0; i < labels.size(); i++) {
			LabelObject object = labels.get(i);
			fwc.addFieldWrapper(object.getWrapper());
		}

		rep = new SegmentGraphicRepresentation(panel, segment, fwc, SegmentGraphReprUtil.customHeightOffset);
		
		labelView = new LabelViewCustomFields(labels, rep, controlPanel);

		pictureScrollPane = new JScrollPane(labelView, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.getVerticalScrollBar().addAdjustmentListener(e -> {
			customView.setVerticalScrollValue(e.getValue());
		});
		pictureScrollPane.getVerticalScrollBar().setPreferredSize(new Dimension(0, 0));
		

		removeAll();
		setLayout(new BorderLayout(0, 1));
		add(pictureScrollPane, BorderLayout.CENTER);
	}

	public void deselect() {
	    if (!labelView.updating)
            labelView.list.clearSelection();
    }

	public void update(Segment segment) {
		this.segment = segment;

		rep.update();

		labels = segment.getRootLabelObject().getCustomLabelObject().getVisibleObjects();
		
//		ArrayList<LabelObject> labelObjs = segment.getRootLabelObject().getGraphLabelObject().getVisibleObjects();
//		labelObjs = filterLabelObjs(labelObjs);
		
//		labelObjs.addAll(labels);

//		labelView = new LabelViewNonManualFields(labelObjs, rep, controlPanel);
		labelView = new LabelViewCustomFields(labels, rep, controlPanel);

		pictureScrollPane.setViewportView(labelView);

		add(pictureScrollPane, BorderLayout.CENTER);
	}

	public void updateLabelOrder() {
		labelView.update();
	}

	public ArrayList<LabelObject> getLabels() {
		return labels;
	}

	public int getLabelsHeight() {
		int totalHeight = 0;

		for (LabelObject obj : labels) {
			int currentWrapperHeight;
			FieldWrapper wrapper = obj.getWrapper();
			if (wrapper instanceof GraphFieldWrapper) {
				currentWrapperHeight =
						SegmentGraphReprUtil.getCorrectCellHeight(((GraphFieldWrapper) wrapper).getGraphField());
			} else {
				currentWrapperHeight = SegmentGraphReprUtil.cellHeight;
			}
			totalHeight += currentWrapperHeight;
		}

		totalHeight += SegmentGraphReprUtil.axesTopHeight; // just to buffer the space at the top (should be enough)

		return totalHeight;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRepresentation() {
		return rep;
	}

	public void setPreferredHeight(int ph) {
		int size = getPreferredSize().width;
		setPreferredSize(new Dimension(size, ph));
	}

	public void setPreferredWidth(int pw) {
		int size = getPreferredSize().height;
		setPreferredSize(new Dimension(pw, size));
	}

	public void loadWrappers() {
		ArrayList<LabelObject> visibleCustomObjects = segment.getRootLabelObject().getCustomLabelObject()
				.getVisibleObjects();
//		ArrayList<LabelObject> visibleObjects = segment.getRootLabelObject().getGraphLabelObject().getVisibleObjects();
//		visibleObjects = filterLabelObjs(visibleObjects);
		
//		visibleObjects.addAll(visibleNonManualObjects);
		FieldWrappersCollection collection = new FieldWrappersCollection();

//		for (LabelObject obj : visibleObjects) {
		for (LabelObject obj : visibleCustomObjects) {
			FieldWrapper fieldWrapper = obj.getWrapper();
			fieldWrapper.setVisible(true);
			collection.addFieldWrapper(fieldWrapper);
		}

		rep.setFieldWrappersCollection(collection);
//		labelView = new LabelViewNonManualFields(visibleObjects, rep, controlPanel);
		labelView = new LabelViewCustomFields(visibleCustomObjects, rep, controlPanel);

		pictureScrollPane.setViewportView(labelView);

		removeAll();
		setLayout(new BorderLayout(0, 1));
		add(pictureScrollPane, BorderLayout.CENTER);

	}

	public LabelObject getSelectedField() {
		LabelObject selectedFieldLabel = (LabelObject) labelView.getLabelViewIndependentItems().getList()
				.getSelectedValue();
		return selectedFieldLabel;
	}

	public SegmentControlPanel getControlPanel() {
		return controlPanel;
	}

	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}

	public void setVerticalScrollModel(BoundedRangeModel model) {
		pictureScrollPane.getVerticalScrollBar().setModel(model);
	}
	
	public void setVerticalScrollValue(int value) {
		pictureScrollPane.getVerticalScrollBar().setValue(value);
	}

	private ArrayList<LabelObject> filterLabelObjs(ArrayList<LabelObject> labelObjects) {
		ArrayList<LabelObject> filteredLabels = new ArrayList<LabelObject>();

		for (LabelObject object : labelObjects) {
			FieldWrapper wrapper = object.getWrapper();
			if (wrapper instanceof GraphFieldWrapper) {
				GraphFieldWrapper gphWrapper = ((GraphFieldWrapper) wrapper);
				SS3Participant participant = segment.getParentField().getSS3GlossWindowSegment().getParticipant();
				if (gphWrapper.getGraphFileID() != null && participant != null &&
						gphWrapper.getGraphFileID().equals(participant.getGraphFileID())) {
					filteredLabels.add(object);
				}
			} else {
				filteredLabels.add(object);
			}
		}

		return filteredLabels;
	}

}
