package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.listener.SegmentMouseListener;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class DatabaseView extends JPanel {

	private static final long serialVersionUID = 1L;

	protected LinenView linenView;
	protected LabelViewTemporalPartitions labelView;
	private UpperLeftCorner upperLeft = null;
	protected JScrollPane pictureScrollPane;
	protected JSplitPane splitPane;
	
	private SegmentGraphicRepresentation rep = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private Segment segment;
	
	private int length = 350;
	public static final int screenBufferAmount = 50;
	
	/** Controls how much the database view scrolls when dragging a new utterance outside the view */
	private int rectScrollGranularity = 50;

	public DatabaseView(Segment segment, SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;

		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

		rep = new SegmentGraphicRepresentation(segmentPanel, segment);

//		try {
//			MultipleMovieController contr = SS3Singleton
//					.getVideoControlManager().getMultipleMovieController();
//			double movieDuration = contr.getMovieDuration();
//			double l = segmentPanel.getZoomer().getCoordinateTimeConvertor()
//					.convertTimeToCoordinate(movieDuration);
//
//			if (l > length)
//				length = (int) l + 20;
//
//		} catch (NullPointerException ex) {
//			ex.printStackTrace();
//			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
//		}

		rep.setPreferredSize(new Dimension(length, (int) rep
				.getPreferredDimension().getHeight()));

		SegmentMouseListener listener = new SegmentMouseListener(segmentPanel);
		rep.addMouseListener(listener);
		rep.addMouseMotionListener(listener);

		linenView = new LinenView(LinenView.HORIZONTAL, true, length,
				segmentPanel);
		labelView = new LabelViewTemporalPartitions(segment, this);

		linenView.setPreferredWidth(length);
		labelView.setPreferredSize(new Dimension(125, rep
				.getPreferredDimension().height));

		pictureScrollPane = new JScrollPane(rep,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.setPreferredSize(new Dimension(300, 250));
		pictureScrollPane.setViewportBorder(BorderFactory
				.createLineBorder(Color.black));

		pictureScrollPane.setColumnHeaderView(linenView);
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER,
				new UpperRightCorner());

		splitPane = new JSplitPane();
		
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		upperLeft = new UpperLeftCorner(segmentPanel);
		upperLeft.setMinimumSize(new Dimension(100,linenView.getPreferredSize().height));
		upperLeft.setMaximumSize(new Dimension(Integer.MAX_VALUE,linenView.getPreferredSize().height));
		left.add(upperLeft);
		left.add(labelView);
		
		splitPane.setLeftComponent(left);
		splitPane.setRightComponent(pictureScrollPane);
		splitPane.setPreferredSize(new Dimension(300, 250));
		
		add(splitPane);
	}

	public Segment getSegment() { return segment; }

	public void scrollRectToVisible(Point point) {
		JViewport vp = pictureScrollPane.getViewport();
		vp.setViewPosition(point);
	}
	
	/**
	 * Scrolls the database view forward an incremental amount. Meant to provide more granularity
	 * when scrolling than the scrollRectToVisible(Point) method.
	 */
	public void scrollRectForward(){
		JViewport vp = pictureScrollPane.getViewport();
		Point temp = new Point(vp.getViewPosition().x + vp.getExtentSize().width/rectScrollGranularity, 0);
		vp.setViewPosition(temp);
	}
	
	/**
	 * Scrolls the database view backward an incremental amount. Meant to provide more granularity
	 * when scrolling than the scrollRectToVisible(Point) method.
	 */
	public void scrollRectBackward(){
		JViewport vp = pictureScrollPane.getViewport();
		Point temp = new Point(vp.getViewPosition().x - vp.getExtentSize().width/rectScrollGranularity, 0);
		if (temp.x < 0)
			temp.x = 0;
		vp.setViewPosition(temp);
	}
	
	@Override
	public int getWidth() {
		return splitPane.getWidth();
	}
	
	public int getLength() {
		return length;
	}

	public int getVisibleWidth2() {
		if (pictureScrollPane == null) {
			return -1;
		} else {
			JViewport vp = pictureScrollPane.getViewport();
			Dimension d = vp.getExtentSize();
			return d.width;
		}
	}

	public LabelViewTemporalPartitions getLabelView() {
		return labelView;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRepresentation() {
		return rep;
	}

	public Point getVisiblePoint() {
		JViewport vp = pictureScrollPane.getViewport();
		return vp.getViewPosition();
	}

	/**
	 * Keeps the new utterance visible on the screen while it is being created.
	 * @param coord - The coordinate of the mouse while it is being dragged. 
	 */
	public void keepUtteranceVisible(int coord) {
		
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int rectLeftCorner = viewPosition.x;
			int rectRightCorner = rectLeftCorner + visibleWidth;

			if (rectLeftCorner < coord && coord < rectRightCorner) {
				return;
			} else {
				if (coord >= rectRightCorner)
					scrollRectForward();
				else
					scrollRectBackward();
			}

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}
	

	public void setVisible(double startCoord, double endCoord) {
        var viewport = pictureScrollPane.getViewport();
        var visibleRect = viewport.getVisibleRect();
        var viewPosition = viewport.getViewPosition();

        int visibleWidth = visibleRect.width;
        int X1 = viewPosition.x;
        int X2 = X1 + visibleWidth;

        if (visibleWidth == 0)
            throw new RuntimeException("DatabaseView.pictureScrollPane visibleWidth=0");

        if (X1 <= startCoord && X2 >= endCoord)
            return;

        Point scrollTo;
        if (Util.dragging() && startCoord >= X2)
            scrollTo = new Point(X1 + (int)(startCoord - X2), 0);
        else
            scrollTo = new Point( (int) (startCoord - 1 < 0 ? startCoord : startCoord - 1), 0);

        scrollRectToVisible(scrollTo);
	}


	/**
	 * updatePanels() function updates the local LabelViewTemporalPartitions
	 * and SegmentGraphicRepresentation panels.
	 * Both require that the Segment object be updated prior to calling this function.
	 */
	public void updatePanels() {
		labelView.updateTree();
		rep.update();
		repaint();
	}

	public synchronized void zoom(double zoomValue) {
		Point scrollTo = null;
		getUpperLeft().setZoom(zoomValue);

		double sliderVisibleCoord = getDistanceToCoord(segmentPanel.getSlider()
				.getSliderPosition());
		Event selectedEvent = segmentPanel.getSelectedEvent();

        // JACK TODO: Maybe put this back, but the logic is broken for now.
		if (false && selectedEvent != null) {
			double eventCoord = selectedEvent.getStartTimeCoordinate();
			double eventVisibleCoord = getDistanceToCoord(eventCoord);

            // Recalculate the slider position based on the new zoom level
            segmentPanel.getSlider().setSliderPosition(SS3Singleton.getMediaPlayer().time());

			double X2 = eventCoord - eventVisibleCoord;

			if (X2 < 0)
				X2 = 0;

			scrollTo = new Point((int) X2, 0);
		} else if (sliderVisibleCoord > -1) {

			segmentPanel.getZoomer().zoom(zoomValue);
			segmentPanel.repaint();

            // Recalculate the slider position based on the new zoom level
            segmentPanel.getSlider().setSliderPosition(SS3Singleton.getMediaPlayer().time());

			double newSliderPosition = segmentPanel.getSlider()
					.getSliderPosition();
			double X = newSliderPosition - sliderVisibleCoord;

			if (X < 0)
				X = 0;

			scrollTo = new Point((int) X, 0);
		} else {
			EventsEntity selectedEntity = SS3Singleton.getMediaToolBar(null).selectedEntity();
			var parentEntityIDs = selectedEntity.getParentEntities();
			if (parentEntityIDs != null && !parentEntityIDs.isEmpty()) {
				int size = parentEntityIDs.size();
				Event firstParentEvent = parentEntityIDs.get(0);
				Event lastParentEvent = parentEntityIDs.get(size - 1);
				
				if (firstParentEvent != null && lastParentEvent != null) {
					setVisible(firstParentEvent.getStartTimeCoordinate(),
							lastParentEvent.getEndTimeCoordinate());
				} else {
					double midPointTime = getVisibleMiddlePointTime();
					setMidPointTimeVisible(midPointTime);
				}
			}
		}
		
		updateDrawingsAreaSize();

		scrollRectToVisible(scrollTo);

		segmentPanel.repaint();
		segmentPanel.updateUI();
	}

	public void updateDrawingsAreaSize() {
        var sw = SS3Singleton.getMediaPlayer().duration()
            .as(TimeUnit.COORDINATE);

        double screenWidth = sw + DatabaseView.screenBufferAmount;

		Dimension d = rep.getPreferredSize();
		Dimension dd = new Dimension((int) screenWidth, d.height);
		rep.setPreferredSize(dd);
		rep.setSize(dd);
		rep.revalidate();

		if (linenView != null) {
			linenView.setPreferredWidth((int) screenWidth);
			linenView.revalidate();
		}
	}

	public void setMidPointTimeVisible(double time) {

		double midCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(time);
		Rectangle visibleRect = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();

			double halfVisibleWidth = visibleRect.width / 2;
			double X1 = midCoord - halfVisibleWidth;

			if (X1 < 0)
				X1 = 0;

			Point scrollTo = new Point((int) X1, 0);
			scrollRectToVisible(scrollTo);

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	public double getVisibleMiddlePointTime() {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X = X1 + visibleWidth / 2;

			double time = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertCoordinateToTime(X);

			return time;

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}

		return -1;
	}

	double distance = 0;

	private double getDistanceToCoord(double coord) {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X2 = X1 + visibleWidth;

			if (X1 < coord && X2 > coord) {
				distance = coord - X1;
			}
		} catch (Exception ex) {
			return distance;
		}
		return distance;
	}

	public void unselectAll() {
		FieldWrappersCollection masterWrappersCollection = rep
				.getFieldWrappersCollection();
		masterWrappersCollection.unselectAll();
	}

	public UpperLeftCorner getUpperLeft() {
		return upperLeft;
	}
}
