package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;
import java.util.*;
import java.util.stream.Collectors;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.gesturePropertyField.GesturePropertyEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class SegmentControlPanel extends JPanel implements ActionListener, MouseListener {

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private SegmentTemplate segmentTemplate;
	
	private JComboBox<String> partitionComboBox = new JComboBox<String>();
	private String currentPartitionString = null;
	private SS3TemporalPartition currentPartition = null;
	
	private JComboBox<String> segmentComboBox = new JComboBox<String>();
	private String currentSegmentString= null;
	private SS3GlossWindowSegment currentSegment = null;
	
	private JList<String> utteranceList = null;
	private JScrollPane utteranceListScroller = null;
	private final UtteranceListSelectionListener utteranceListSelectionListener;
    private final UtteranceListMouseListener utteranceListMouseListener;
	private int selectedUtteranceIndex = -1;
	
	private JButton prevUtteranceBtn = null;
	private JButton nextUtteranceBtn = null;
	
	private JLabel annotatorLabel = new JLabel("  Participant: N/A  ");
	private JButton notesBtn = new JButton(" Notes ");
	
	private JPanel top;
	private JRadioButton radioButton;
	
	private SS3Participant selectedParticipant = null;
	private SS3Participant selectedAnnotator = null;
	
	//public static final String selectedIndexKey = "selectedIndex";

	public static final int PREV_BTN = 1;
	public static final int NEXT_BTN = 2;
	public final int NOTES_BTN = 4;

	// objects to discard
	private JDialog notesDialog = null;
	private NotesPanel notesPanel = null;

	public SegmentControlPanel(UtteranceView view, SignStreamSegmentPanel segmentPanel,
							   Segment segment, SegmentTemplate template) {
		
		this.segmentTemplate = template;
		this.segmentPanel = segmentPanel;
		this.segment = segment;

		PresentationField parentField = segment.getParentField();
		setCurrentPartition(parentField.getTemporalPartition());
		setCurrentSegment(parentField.getSS3GlossWindowSegment());
		setSelectedParticipant(parentField.getSS3GlossWindowSegment().getParticipant());
//		setSelectedAnnotator(parentField.getSS3GlossWindowSegment().getAnnotator());

        var selectedSegments = view.getSegments().stream()
            .map(s -> s.getParentField()
                .getSS3GlossWindowSegment())
            .collect(Collectors.toSet());

        String currentValueForTemporalPartition = null;
        String currentValueForSegment = null;

		ArrayList<SS3TemporalPartition> temporalPartitions = segmentPanel.getSS3Database().getSS3GlossWindow().getTemporalPartitions();	
		String partitionLabel = null;
		for (SS3TemporalPartition temporalPartition: temporalPartitions) {
			var current = getCurrentPartition();
			var equals = (temporalPartition == current);
			
		    if (selectedSegments.containsAll(temporalPartition.getTemporalPartitionSegments()) && !equals)
		        continue;

			partitionLabel = temporalPartition.getLabel();
			partitionComboBox.addItem(partitionLabel);
			
			if (equals)
				currentValueForTemporalPartition = partitionLabel;
			
			if (partitionLabel.equals(getCurrentPartitionString())) {
				partitionComboBox.setSelectedItem(partitionLabel);
//				break;
			}
		}
		TemporalPartitionActionListener partitionActionListener = new TemporalPartitionActionListener(this);
		partitionActionListener.setCurrentValue(currentValueForTemporalPartition);
		partitionComboBox.addActionListener(partitionActionListener);

        ArrayList<SS3GlossWindowSegment> temporalPartitionSegments = getCurrentPartition().getTemporalPartitionSegments();
        for (SS3GlossWindowSegment temporalPartitionSegment: temporalPartitionSegments) {
        	var current = getCurrentSegment();
        	var equals = (temporalPartitionSegment == current);
        	
            if (selectedSegments.contains(temporalPartitionSegment) && !equals)
                continue;

			String segmentLabel = temporalPartitionSegment.getLabel();
			String participantLabel = temporalPartitionSegment.getParticipantLabel();
			String combinedLabel = participantLabel + Util.SEGMENT_TIER_SEPARATOR + segmentLabel;
			
			if (equals)
				currentValueForSegment = combinedLabel;
			
			if (combinedLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) == 0)
				combinedLabel = combinedLabel.substring(1);
			
			segmentComboBox.addItem(combinedLabel);
			
			if (combinedLabel.equals(getCurrentSegmentString())) {
				segmentComboBox.setSelectedItem(combinedLabel);
			}
		}
		SegmentActionListener segmentActionListener = new SegmentActionListener(this);
		segmentActionListener.setCurrentValue(currentValueForSegment);
		segmentComboBox.addActionListener(segmentActionListener);

		utteranceList = new JList();
		utteranceList.setCellRenderer(new DefaultListCellRenderer() {
			@Override
		     public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		         Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		         if (isSelected) {
		             c.setBackground(Color.LIGHT_GRAY);
		         }
		         if (index == selectedUtteranceIndex) {
		        	 	c.setBackground(Color.BLUE);
		         }
		         return c;
		     }
		});
		utteranceListScroller = new JScrollPane(utteranceList);
		utteranceListScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		
		utteranceListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		utteranceList.setDragEnabled(false);
		utteranceList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		utteranceListSelectionListener = new UtteranceListSelectionListener(this);
		utteranceList.getSelectionModel().addListSelectionListener(utteranceListSelectionListener);
		utteranceListMouseListener = new UtteranceListMouseListener(this);
		utteranceList.addMouseListener(utteranceListMouseListener);

		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0), "focusZoomAndRight");
		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0), "focusZoomAndLeft");
		utteranceList.getActionMap().put("focusZoomAndRight", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut()
						.requestFocus();
                SS3Singleton.getMediaToolBar(null);
                var player = SS3Singleton.getMediaPlayer();
                var step = Util.getStep();
                var range = Util.getStepRange();
                player.pause();
                player.seek(player.time().add(step).in(range));
            }
		});
		utteranceList.getActionMap().put("focusZoomAndLeft", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut()
						.requestFocus();
                SS3Singleton.getMediaToolBar(null);
                var player = SS3Singleton.getMediaPlayer();
                var step = Util.getStep();
                var range = Util.getStepRange();
                player.pause();
                player.seek(player.time().subtract(step).in(range));
            }
		});
		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), "moveListUp");
		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), "moveListDown");
		utteranceList.getActionMap().put("moveListUp", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				offsetSelectedUtterance(-1);
			}
		});
		utteranceList.getActionMap().put("moveListDown", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				offsetSelectedUtterance(1);
			}
		});
		
		
		updateUtteranceList();
		
		String currentParticipant = currentSegment.getParticipantLabel();
		annotatorLabel.setText(currentParticipant.isEmpty() ? "  Participant: N/A  " : "Participant: " + currentParticipant);
		
		prevUtteranceBtn = new JButton("< Prev Utterance" );
		nextUtteranceBtn = new JButton(" Next Utterance >");
		
		prevUtteranceBtn.setActionCommand("" + PREV_BTN);
		nextUtteranceBtn.setActionCommand("" + NEXT_BTN);
		
		prevUtteranceBtn.addActionListener(new PrevNextUtteranceListener(this));
		nextUtteranceBtn.addActionListener(new PrevNextUtteranceListener(this));
		
		notesBtn.setActionCommand("" + NOTES_BTN);
		notesBtn.addActionListener(this);
		
		radioButton = new JRadioButton();
		radioButton.addActionListener(e -> {
            segmentTemplate.selectTemplate();
            SS3Singleton.getMediaToolBar()
                .enterUtterance(getSelectedUtterance());
        });

		UserInterfaceUtil.setComboBoxLNF(partitionComboBox);
		partitionComboBox.setPreferredSize(null);
		partitionComboBox.setMaximumSize(null);
		UserInterfaceUtil.setComboBoxLNF(segmentComboBox);
		segmentComboBox.setPreferredSize(null);
		segmentComboBox.setMaximumSize(null);
		UserInterfaceUtil.setListLNF(utteranceList);
		UserInterfaceUtil.setLabelLNF(annotatorLabel);
		UserInterfaceUtil.setButtonsLNF(notesBtn);
		UserInterfaceUtil.setButtonsLNF(prevUtteranceBtn);
		UserInterfaceUtil.setButtonsLNF(nextUtteranceBtn);

		annotatorLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		notesBtn.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		top = new JPanel();
		
		addMouseListener(this);
		top.addMouseListener(this);
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.weightx = 1.0;
		c.gridwidth = 2;
		
		top.setLayout(new GridBagLayout());
		c.gridy = 0;
		top.add(partitionComboBox, c);
		
		c.gridy = 1;
		top.add(segmentComboBox, c);
		
		c.gridy = 0;
		c.gridx = 2;
		top.add(radioButton, c);
		
		c.gridy = 0;
		c.gridx = 0;
		add(top,c);
		
		c.gridy = 2;
		c.weighty = 1.0;
		add(utteranceListScroller, c);
		
		c.weighty = 0;
		c.gridwidth = 1;
		
		c.gridy = 3;
		add(prevUtteranceBtn, c);
		
		c.gridx = 1;
		add(nextUtteranceBtn, c);
		
		c.gridx = 0;
		c.gridy = 4;
		add(annotatorLabel, c);
		
		c.gridx = 1;
		add(notesBtn, c);

	}

	public void update(Segment segment) {
		this.segment = segment;
	}
	
	public void updateUtteranceList(String id) {
		SS3GlossWindowSegment currentSegment = getCurrentSegment();
		currentSegment.setIndexFieldID(id);
		updateUtteranceList();
	}
	
	public void updateUtteranceList() {
        SS3GlossWindowSegment currentSegment = getCurrentSegment();
		String indexFieldID = currentSegment.getIndexFieldID();

		var id = indexFieldID != null && !indexFieldID.isEmpty()
            ? indexFieldID
            : Util.DOM_GLOSS_FIELD_ID;

        var field = segment.getRootLabelObject().getAllLabelObjects().stream()
            .filter(l -> l.getFieldID().equals(id))
            .map(l -> l.getWrapper().getField())
            .findFirst().orElse(null);

        // Use the existing labels if the index field label object doesn't exist
        if (field == null) {
            var labels = currentSegment.getSS3GlossWindowItems().stream()
                .map(SS3GlossWindowItem::getLabel)
                .collect(Collectors.toList());
            utteranceList.setListData(Arrays.copyOf(labels.toArray(), labels.size(), String[].class));

            return;
        };

        // Otherwise, append the entity text to the existing labels
		var labels = new ArrayList<String>();
		var utterances = currentSegment.getSS3GlossWindowItems();
		for (var utterance : utterances) {
		    var entities = field.getAllEntities().stream()
	                .filter(e -> e.getParentEntities() != null 	&& e.getParentEntities().stream().anyMatch(p -> p.getSS3GlossWindowItem().getId().equals(utterance.getId())))
	                .sorted(Comparator.comparing(Event::getStartTimeInfo))
	                .collect(Collectors.toList());

		    StringBuilder label = new StringBuilder(utterance.getLabel());
		    for (var e : entities) {
		        if (e instanceof ChainedEventsEntity) {
		            var cee = (ChainedEventsEntity)e;
		            if (cee.getTextValue() != null && !cee.getTextValue().getText().isBlank())
		                label.append(" ").append(cee.getTextValue().getText());
		            else if (cee.getValue() != null && !cee.getValue().getText().isBlank())
		                label.append(" ").append(cee.getValue().getText());
                }

		        if (e instanceof GestureChainedEventsEntity) {
		            var gcee = (GestureChainedEventsEntity)e;
                    if (gcee.getTextValue() != null && !gcee.getTextValue().getText().isBlank())
                        label.append(" ").append(gcee.getTextValue().getText());
                    else if (gcee.getValue() != null && !gcee.getValue().getText().isBlank())
                        label.append(" ").append(gcee.getValue().getText());
                }

		        if (e instanceof ApexChainedApexEventsEntity) {
                    var aee = (ApexChainedApexEventsEntity)e;
                    if (aee.getTextValue() != null && !aee.getTextValue().getText().isBlank())
                        label.append(" ").append(aee.getTextValue().getText());
                    else if (aee.getValue() != null && !aee.getValue().getText().isBlank())
                        label.append(" ").append(aee.getValue().getText());
                }

		        if (e instanceof SignTypeEntity) {
		            var se = ((SignTypeEntity)e).getEvent();
		            if (se.getSignType().getLabel() != null)
                        label.append(" ").append(se.getSignType().getLabel());
                }

                if (e instanceof LocationEntity)
                    for (var le : ((LocationEntity)e).getEvents())
                        label.append(" ").append(le.getLocation().getName());

                if (e instanceof HandShapeEntity)
                    label.append(" ").append(((HandShapeEntity)e).getHandShapeType());

                if (e instanceof GestureHandShapeEntity)
                    label.append(" ").append(((GestureHandShapeEntity)e).getHandShapeType());

                if (e instanceof GestureOrientationEntity)
                    label.append(" ").append(((GestureOrientationEntity)e).getOrientationType());

                if (e instanceof GesturePropertyEntity)
                    for (var ty : ((GesturePropertyEntity)e).getSelectedProperties())
                        label.append(" ").append(ty.getName());
            }
		    labels.add(label.toString());
        }

        utteranceList.setListData(Arrays.copyOf(labels.toArray(), labels.size(), String[].class));

//		ArrayList<SS3GlossWindowItem> items = currentSegment.getSS3GlossWindowItems();
////		ArrayList<String> utterances = new ArrayList<String>();
//		// need to use LinkedHashMap - or at least, it's the easiest way to keep the order of the utterances
//		LinkedHashMap<String, ArrayList<String>> utteranceIDMap = new LinkedHashMap<String, ArrayList<String>>();
//		for (SS3GlossWindowItem item: items) {
//			String itemID = item.getId();
//			utteranceIDMap.put(itemID, new ArrayList<String>());
//		}
//
//		// 2021-03-22 -- NOTE: the below works badly for gesture properties - it needs a special case, not sure how best to handle it right now
//
//		//need specific ss3Entitites to access location/sign type labels
//		switch (indexFieldID) {
//		case Util.DOM_LOCATION_FIELD_ID:
//			if (currentSegment.getSS3StatementField(Util.DOM_GLOSS_FIELD_ID) != null) {
//				ArrayList<SS3Entity> ss3Entities = currentSegment.getSS3StatementField(Util.DOM_GLOSS_FIELD_ID).getSS3Entities();
//				for (SS3Entity ss3Entity: ss3Entities) {
//					if (ss3Entity.getParentEntityID() == null || ss3Entity.getParentEntityID().isEmpty()) continue;
//					if (ss3Entity.getGlossEntityProperties().getLocationInfo() == null) continue;
//
//					String text = ss3Entity.getGlossEntityProperties().getLocationInfo().toString();
//					utteranceIDMap.get(ss3Entity.getParentEntityID()).add(text);
//				}
//			}
//			break;
//		case Util.NDOM_LOCATION_FIELD_ID:
//			if (currentSegment.getSS3StatementField(Util.NDOM_GLOSS_FIELD_ID) != null) {
//				ArrayList<SS3Entity> ss3Entities = currentSegment.getSS3StatementField(Util.NDOM_GLOSS_FIELD_ID).getSS3Entities();
//				for (SS3Entity ss3Entity: ss3Entities) {
//					if (ss3Entity.getParentEntityID() == null || ss3Entity.getParentEntityID().isEmpty()) continue;
//					if (ss3Entity.getGlossEntityProperties().getLocationInfo() == null) continue;
//
//					String text = ss3Entity.getGlossEntityProperties().getLocationInfo().toString();
//					utteranceIDMap.get(ss3Entity.getParentEntityID()).add(text);
//				}
//			}
//			break;
//		case Util.DOM_SIGN_TYPE_FIELD_ID:
//			if (currentSegment.getSS3StatementField(Util.DOM_GLOSS_FIELD_ID) != null) {
//				ArrayList<SS3Entity> ss3Entities = currentSegment.getSS3StatementField(Util.DOM_GLOSS_FIELD_ID).getSS3Entities();
//				for (SS3Entity ss3Entity: ss3Entities) {
//					if (ss3Entity.getParentEntityID() == null || ss3Entity.getParentEntityID().isEmpty()) continue;
//					if (ss3Entity.getGlossEntityProperties().getClassifier() == null) continue;
//
//					String text = ss3Entity.getGlossEntityProperties().getClassifier().getLabel();
//					utteranceIDMap.get(ss3Entity.getParentEntityID()).add(text);
//				}
//			}
//			break;
//		case Util.NDOM_SIGN_TYPE_FIELD_ID:
//			if (currentSegment.getSS3StatementField(Util.NDOM_GLOSS_FIELD_ID) != null) {
//				ArrayList<SS3Entity> ss3Entities = currentSegment.getSS3StatementField(Util.NDOM_GLOSS_FIELD_ID).getSS3Entities();
//				for (SS3Entity ss3Entity: ss3Entities) {
//					if (ss3Entity.getParentEntityID() == null || ss3Entity.getParentEntityID().isEmpty()) continue;
//					if (ss3Entity.getGlossEntityProperties().getClassifier() == null) continue;
//
//					String text = ss3Entity.getGlossEntityProperties().getClassifier().getLabel();
//					utteranceIDMap.get(ss3Entity.getParentEntityID()).add(text);
//				}
//			}
//			break;
//		default:
//			//for all other nonmanuals/custom/gloss/gestures fields
//			if (currentSegment.getSS3StatementField(indexFieldID) != null) {
//				// entities are in order by startTime
//				ArrayList<SS3Entity> ss3Entities = currentSegment.getSS3StatementField(indexFieldID).getSS3Entities();
//
//				for (SS3Entity ss3Entity: ss3Entities) {
//					if (ss3Entity.getParentEntityID() == null || ss3Entity.getParentEntityID().isEmpty()) continue;
//
//					ArrayList<SS3Item> entItems = ss3Entity.getItems();
//					for (SS3Item entItem: entItems) {
//						// if entItem.isTextOnly() gave back the correct/up-to-date result, then it would be used here instead
//						String text = entItem.getText();
//						// TODO: there's a better way to do this:
//						if (text.equals("Onset") || text.equals("Offset")
//								|| text.equals("Initial Hold") || text.equals("Final Hold")
//								|| text.equals("Preparation") || text.equalsIgnoreCase("Pre-stroke Hold")
//								|| text.equalsIgnoreCase("Post-stroke Hold") || text.equals("Retraction")
//								|| text.equals("Relaxed")
//								)
//							continue;
//
//						utteranceIDMap.get(ss3Entity.getParentEntityID()).add(entItem.getText());
//					}
//				}
//			}
//			break;
//		}
//
//		for (Entry<String, ArrayList<String>> entry: utteranceIDMap.entrySet()) {
//			String itemID = entry.getKey();
//
//			String utteranceLabel = "";
//			for (SS3GlossWindowItem item: items) {
//				if (item.getId().equalsIgnoreCase(itemID)) {
//					utteranceLabel = item.getLabel();
//					break;
//				}
//			}
//			String combinedTextLabels = String.join(" ", entry.getValue());
////			utterances.add(utteranceLabel + " " + combinedTextLabels);
//		}
//
//		utteranceList.setListData(Arrays.copyOf(utterances.toArray(), utterances.size(), String[].class));
	}
	
	public void updateUtteranceListSelectedItems(List<PresentationEventsEntity> utterances) {

		ArrayList<String> utteranceLabels = new ArrayList<String>();
		for (PresentationEventsEntity utterance: utterances) {
			utteranceLabels.add(utterance.getEvent().getSS3GlossWindowItem().getLabel());
		}
		
		ArrayList<Integer> indexList = new ArrayList<Integer>();
		
		ListModel<String> listModel = utteranceList.getModel();
		int size = listModel.getSize();
		
		for (int i = 0; i < size; i++) {
			String elementLabelWithText = listModel.getElementAt(i);

			String elementLabel = elementLabelWithText.contains(" ")
                ? elementLabelWithText.substring(0, elementLabelWithText.indexOf(" "))
                : elementLabelWithText;

			for (String utteranceLabel: utteranceLabels) {
				if (elementLabel.equals(utteranceLabel)) {
					indexList.add(i);
				}
			}
		}
		
		// code necessary to create an int array from an Integer arraylist
		int indiciesListSize = indexList.size();
		// indiciesArray must be final to be used in inner Runnable class below
		final int[] indiciesArray = new int[indiciesListSize];
		for (int i = 0; i < indiciesListSize; i++) {
			indiciesArray[i] = indexList.get(i);
		}

		utteranceList.clearSelection();
		
		if (indiciesListSize > 0) {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					
					// first remove the listener from the utteranceList to avoid an infinite action firing
					utteranceList.getSelectionModel().removeListSelectionListener(utteranceListSelectionListener);

					utteranceList.setSelectedIndices(indiciesArray);

					// then re-add the listener after we have manually set the indicies
					utteranceList.getSelectionModel().addListSelectionListener(utteranceListSelectionListener);

					utteranceList.ensureIndexIsVisible(indiciesArray[0]);
				}
			});
		}
	}

	public int getSelectedUtteranceIndex() {
	    return selectedUtteranceIndex;
    }
	
	public PresentationEventsEntity getSelectedUtterance() {
		if (getSegment().getParentField().getEntities().size() == 0 || selectedUtteranceIndex == -1) {
			return null;
		}
		return getSegment().getParentField().getEntities().get(selectedUtteranceIndex);
	}
	
	public ArrayList<PresentationEventsEntity> getVisibleUtterances() {
		ArrayList<PresentationEventsEntity> visibleUtterances = new ArrayList<PresentationEventsEntity>();
		
		int[] visibleIndices = utteranceList.getSelectedIndices();
		for (int i = 0; i < visibleIndices.length; i++) {
			int visibleIndex = visibleIndices[i];
			visibleUtterances.add(getSegment().getParentField().getEntities().get(visibleIndex));
		}
		
		return visibleUtterances;
	}

	public void setSelectedUtteranceIndex(int i) {
	    selectedUtteranceIndex = i;
	    selectAndScrollToUtteranceByIndex(i);
    }

	public boolean setSelectedUtterance(EventsEntity utterance) {
        segmentTemplate.selectTemplate();

		ArrayList<PresentationEventsEntity> utterances = getSegment().getParentField().getEntities();
		for (int i=0 ; i < utterances.size(); i++) {
			if (utterances.get(i) == utterance) {
				selectedUtteranceIndex = i;
				selectAndScrollToUtteranceByIndex(i);
				return true;
			}
		}

		selectedUtteranceIndex = -1;
        selectAndScrollToUtteranceByIndex(-1);

		return false;
	}
	
	public boolean setSelectedUtterance(String id) {
		ArrayList<PresentationEventsEntity> utterances = getSegment().getParentField().getEntities();
		for (int i=0 ; i < utterances.size(); i++) {
			if (utterances.get(i).getEvent().getID().equals(id)) {
				selectedUtteranceIndex = i;
				selectAndScrollToUtteranceByIndex(i);
				return true;
			}
		}
		return false;
	}
	
	public boolean offsetSelectedUtterance(int offset) {
		int newIndex = selectedUtteranceIndex == -1 ? 0 : selectedUtteranceIndex + offset;
		int listSize = getUtteranceList().getModel().getSize();
		if (newIndex >= listSize || newIndex < 0) {
			return false;
		}
		selectedUtteranceIndex = newIndex;

        EventsEntity utterance = getSegment().getParentField().getEntities().get(selectedUtteranceIndex);

        if (!segmentPanel.zoomLocked())
            segmentPanel.fitZoom(utterance.interval().duration());

        var time = utterance.getStartTimeInfo();
        SS3Singleton.getMediaPlayer().seek(time);
        segmentPanel.scroll(time);

        selectAndScrollToUtteranceByIndex(selectedUtteranceIndex);

        segmentPanel.getToolBarPanels().getMediaToolBar().selectEvent(null, null, null);
        segmentPanel.getUtteranceView().getNonmanualFieldsView().getFieldWrappersCollection().unselectAll();
        segmentPanel.getUtteranceView().getCustomView().getFieldWrappersCollection().unselectAll();
        segmentPanel.getUtteranceView().getGlossNRelatedView().getFieldWrappersCollection().unselectAll();
        segmentPanel.getUtteranceView().getSpeechView().getFieldWrappersCollection().unselectAll();

        segmentPanel.getUtteranceView().showAllFields();
		
		return true;
	}
	
	public JComboBox<String> getTemporalPartitionComboBox() {
		return partitionComboBox;
	}
	
	public String getCurrentPartitionString() {
		return currentPartitionString;
	}
	
	public void setCurrentPartitionString(String currentPartitionString) {
		this.currentPartitionString = currentPartitionString;
	}
	
	public SS3TemporalPartition getCurrentPartition() {
		return currentPartition;
	}
	
	public void setCurrentPartition(SS3TemporalPartition currentPartition) {
		this.currentPartition = currentPartition;
		
		if (currentPartition != null)
			setCurrentPartitionString(currentPartition.getLabel());
		else
			setCurrentPartitionString(null);
	}
	
	public JComboBox<String> getSegmentComboBox() {
		return segmentComboBox;
	}
	
	public String getCurrentSegmentString() {
		return currentSegmentString;
	}
	
	public void setCurrentSegmentString(String currentSegmentString) {
		this.currentSegmentString = currentSegmentString;
	}
	
	public SS3GlossWindowSegment getCurrentSegment() {
		return this.currentSegment;
	}
	
	public void setCurrentSegment(SS3GlossWindowSegment currentSegment) {
		this.currentSegment = currentSegment;
		
		if (currentSegment != null) {
			String currentParticipantLabel = currentSegment.getParticipantLabel();
			String currentSegmentLabel = currentSegment.getLabel();
			String combinedLabel = currentParticipantLabel + Util.SEGMENT_TIER_SEPARATOR + currentSegmentLabel;	

			if (combinedLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR) == 0)
				combinedLabel = combinedLabel.substring(1);
			
			setCurrentSegmentString(combinedLabel);
		}
		else
			setCurrentSegmentString(null);
	}
	
	public JList<String> getUtteranceList() {
		return utteranceList;
	}

	public void moveUtteranceListSelection(boolean up) {
		offsetSelectedUtterance(up ? -1 : 1);
	}
	
	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}
	
	public Segment getSegment() {
		return segment;
	}

	public SS3Participant getSelectedParticipant() {
		return selectedParticipant;
	}

	public void setSelectedParticipant(SS3Participant selectedParticipant) {
		this.selectedParticipant = selectedParticipant;
	}

	public SS3Participant getSelectedAnnotator() {
		return selectedAnnotator;
	}

	public void setSelectedAnnotator(SS3Participant selectedAnnotator) {
		this.selectedAnnotator = selectedAnnotator;
	}
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		switch (Integer.parseInt(str)) {
		case NOTES_BTN:
			if (notesDialog == null)
				createNotesDialog();
			notesDialog.setVisible(true);
			break;
		}
	}

	private void createNotesDialog() {
		notesDialog = new JDialog();
		notesDialog.setTitle("Note Editor");
		notesDialog.setModal(true);

		notesPanel = new NotesPanel( getCurrentSegment() != null ? getCurrentSegment().getNote() : null,
				notesDialog);

		JPanel panel1 = new JPanel(new BorderLayout());
		panel1.add(notesPanel, BorderLayout.CENTER);
		panel1.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		Container notesContentPane = notesDialog.getContentPane();
		notesContentPane.setLayout(new BorderLayout());
		notesContentPane.add(notesPanel, BorderLayout.CENTER);

		notesDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				notesPanel.save();
			}
		});

		notesDialog.setSize(400, 350);
		notesDialog.setPreferredSize(new Dimension(400, 350));
		notesDialog.setLocation(150, 150);
		notesDialog.pack();
	}

	public void discard() {
		if (notesDialog != null) {
			notesPanel.discard();

			WindowListener[] listeners = notesDialog.getWindowListeners();
			for (int i = 0; i < listeners.length; i++) {
				WindowListener listener = listeners[i];
				notesDialog.removeWindowListener(listener);
			}

			notesPanel = null;
			notesDialog = null;
		}

	}
	
	public JPanel getTop() {
		return top;
	}
	
	public void setRadioButtonSelected(boolean val) {
		radioButton.setSelected(val);
	}

	public void focusUtteranceList() {
		utteranceList.requestFocus();
	}
	
	/**
	 * Based on an index into a list of utterances, reframe the visible viewbox on-screen to the start of the indexed utterance.
	 * This will: 1) Automatically update the utterance list in this SegmentControlPanel; 2) Set a new movie time; 
	 * 3) Move the slider to this new movie time; 4) refresh the visible non-manual fields in every segment tier.
	 * @param i The index of the utterance to select, based on the parent field of this SegmentControlPanel
	 */
	private void selectAndScrollToUtteranceByIndex(int i) {
        PresentationField parentField = getSegment().getParentField();
        ArrayList<PresentationEventsEntity> utterances = parentField.getEntities();
        
	    if (i != -1 && ! utterances.isEmpty()) {
            PresentationEventsEntity utterance = utterances.get(i);

            SS3Singleton.getMediaToolBar().enterUtterance(utterance);
        } else {
	        SS3Singleton.getMediaToolBar().exitUtterance();
        }

        segmentTemplate.getNonmanView()
            .getCSFieldDialog()
            .updateCurrentUtterance();
    }

    public SegmentTemplate getSegmentTemplate() {
        return segmentTemplate;
	}
	
	@Override
	public void mouseClicked(MouseEvent e) {
		segmentTemplate.selectTemplate();
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
}

class PrevNextUtteranceListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	
	public PrevNextUtteranceListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
        var player = SS3Singleton.getMediaPlayer();
        if (player.playing())
            player.pause();

        // just check the utterance is not out of bounds for either prev/next
		String str = e.getActionCommand();		
		switch (Integer.parseInt(str)) {
		case SegmentControlPanel.PREV_BTN:
			
			if (!this.segmentControlPanel.offsetSelectedUtterance(-1)) {
				String msg = "<HTML>Warning: Previous utterance does not exist <BR>";
				msg = msg + "Please use \"Next Utterance\" to view other utterances</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resource Update", msg);
				return;
			}
			break;
		case SegmentControlPanel.NEXT_BTN:
			if (!this.segmentControlPanel.offsetSelectedUtterance(1)) {
				String msg = "<HTML>Warning: Next utterance does not exist <BR>";
				msg = msg + "Please use \"Prev Utterance\" to view other utterances</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resource Update", msg);
				return;
			}
			break;
		}

        // unselect events, and unhighlight everything so that a new look at an utterance plays that utterance
		segmentControlPanel.getSignStreamSegmentPanel().getToolBarPanels().getMediaToolBar().selectEvent(null, null, null);
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getFieldWrappersCollection().unselectAll();
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getCustomView().getFieldWrappersCollection().unselectAll();
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getGlossNRelatedView().getFieldWrappersCollection().unselectAll();
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getSpeechView().getFieldWrappersCollection().unselectAll();

		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().showAllFields();
        
		// gets the focus off of the utterance list and allows the up/down to move the zoom slider
//		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getHeader().getTimelineView()
//				.getUpperLeftCorner().getZoomInOut().requestFocus();
	}
}

class UtteranceListMouseListener extends MouseAdapter {
    private final SegmentControlPanel segmentControlPanel;
    private final SignStreamSegmentPanel segmentPanel;
    private final JList<String> list;

    public UtteranceListMouseListener(SegmentControlPanel segmentControlPanel) {
        this.segmentControlPanel = segmentControlPanel;
        this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
        this.list = segmentControlPanel.getUtteranceList();
    }

    private void consume(MouseEvent e) {

        if (e.isMetaDown()) {
            e.consume();

            var player = SS3Singleton.getMediaPlayer();
            if (player.playing())
                player.pause();

            segmentControlPanel.setSelectedUtteranceIndex(-1);
        } else if (list.getSelectedIndices().length == 1) {
//            var i = list.getSelectedIndex();
//
//            var utterance = segmentControlPanel.getSegment()
//                .getParentField()
//                .getEntities()
//                .get(i);
//
//            if (!segmentPanel.zoomLocked())
//                segmentPanel.fitZoom(utterance.interval().duration());
//
//            var time = utterance.getStartTimeInfo();
//            SS3Singleton.getMediaPlayer().seek(time);
//            segmentPanel.scroll(time);
//
//            segmentControlPanel.setSelectedUtteranceIndex(i);
        }
    }

    @Override public void mousePressed(MouseEvent e) { consume(e); }
    @Override public void mouseReleased(MouseEvent e) { consume(e); }

    @Override public void mouseClicked(MouseEvent e) { e.consume(); }
}

class UtteranceListSelectionListener implements ListSelectionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public UtteranceListSelectionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();		
	}
	@Override
	public void valueChanged(ListSelectionEvent e) {
        ListSelectionModel utteranceListSelectionModel = (ListSelectionModel) e.getSource();

        // this will proceed once the value is no longer adjusting, meaning the mouse click is released
		if (e.getValueIsAdjusting())
			return;

        if (utteranceListSelectionModel.isSelectionEmpty()) {
        		return;
        } else {
            var player = SS3Singleton.getMediaPlayer();
            if (player.playing())
                player.pause();

	    		// go to selected utterance
            int utteranceListIndex = utteranceListSelectionModel.getMaxSelectionIndex();
            EventsEntity utterance = segmentControlPanel.getSegment().getParentField().getEntities().get(utteranceListIndex);

            if (!segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().locked())
                segmentPanel.fitZoom(utterance.interval().duration());

            var time = utterance.getStartTimeInfo();
            SS3Singleton.getMediaPlayer().seek(time);
            segmentPanel.scroll(time);

            // unselect events, and unhighlight everything so that a new look at an utterance plays that utterance
            segmentControlPanel.getSignStreamSegmentPanel().getToolBarPanels().getMediaToolBar().deselect();

            segmentControlPanel.setSelectedUtterance(utterance);

            segmentPanel.getUtteranceView()
                .getSegmentTemplate()
                .getNonmanView()
                .getCSFieldDialog()
                .updateCurrentUtterance();

            segmentPanel.getUtteranceView().showAllFields();
        }
	}
	
}

class SegmentActionListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	private String currentValue = "";
	
	public SegmentActionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
	}
	
	protected void setCurrentValue(String value) {
		this.currentValue = value;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		// first check that this segment tier is not visible in any other segment in the utterance view
		String selectedTemporalPartitionName = 
				(String) segmentControlPanel.getTemporalPartitionComboBox().getSelectedItem();		
		var partition = segmentPanel.getSS3Database()
	            .getSS3GlossWindow()
	            .getTemporalPartitionByLabel(selectedTemporalPartitionName);

		String selectedSegmentTierNameWithParticipant = (String) segmentControlPanel.getSegmentComboBox().getSelectedItem();
		
		String selectedSegmentTierName = selectedSegmentTierNameWithParticipant.substring(selectedSegmentTierNameWithParticipant.indexOf(Util.SEGMENT_TIER_SEPARATOR) + 1);

        var selectedSegments = segmentPanel.getUtteranceView().getSegments().stream().map(s -> s.getParentField().getSS3GlossWindowSegment()).collect(Collectors.toSet());

        // Determine if the segment selected to change to is already selected
		var segment = partition.getTemporalPartitionSegments().stream()
            .filter(s -> s.getLabel().equals(selectedSegmentTierName) && selectedSegments.contains(s))
            .findFirst().orElse(null);
		
		if (segment == null) {
			// switch to that segment if it's not already selected
			segmentPanel.getUtteranceView().addUtterancePane(segmentControlPanel.getSegment(), selectedTemporalPartitionName, selectedSegmentTierName, null);
			setCurrentValue(selectedTemporalPartitionName);
		} else {
			// do not switch to that segment, and revert the combobox selection back to what it was before attempting to change it
			segmentControlPanel.getSegmentComboBox().setSelectedItem(this.currentValue);
		}
	}
	
}

class TemporalPartitionActionListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	private String currentValue = "";
	
	public TemporalPartitionActionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
	}
	
	protected void setCurrentValue(String value) {
		this.currentValue = value;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String partitionName = (String) segmentControlPanel.getTemporalPartitionComboBox().getSelectedItem();
		var partition = segmentPanel.getSS3Database()
            .getSS3GlossWindow()
            .getTemporalPartitionByLabel(partitionName);
		
		if (partition.getLabel().equals(this.currentValue))
			return;

        var selectedSegments = segmentPanel.getUtteranceView().getSegments().stream()
            .map(s -> s.getParentField()
                .getSS3GlossWindowSegment())
            .collect(Collectors.toSet());

        // Select a segment for the partition that isn't already selected
		var segment = partition.getTemporalPartitionSegments().stream()
            .filter(s -> !selectedSegments.contains(s))
            .findFirst().orElse(null);
		
		if (segment != null) {
			var segmentName = segment.getLabel();

			segmentPanel.getUtteranceView().addUtterancePane(segmentControlPanel.getSegment(), partitionName, segmentName, null);
			setCurrentValue(partitionName);
		} else {
			segmentControlPanel.getTemporalPartitionComboBox().setSelectedItem(this.currentValue);
		}
	}
	
}
