package edu.bu.signstream.media;

import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;

import java.net.URI;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;

public interface MediaPlayer {
    // Controls //

    void preroll();

    boolean playing();
    void play();
    void pause();

    Time time();
    Interval range();
    void seek(Time time);
    void seek(Interval segment);
    void seek(Interval segment, double rate);

    double rate();
    void rate(double rate);

    void close();

    // Media //

    List<Media> media();
    Optional<Media> media(URI uri);
    void remove(Media m);
    void offset(Media m, Duration d);

    List<Video> videos();
    Optional<Video> video(URI uri);
    Video addVideo(URI uri, int order);

    List<Audio> audios();
    Optional<Audio> audio(URI uri);
    Audio addAudio(URI uri, int order);

    Time start();
    Time end();
    Interval interval();
    Duration duration();

    // Listeners //

    void addTimeListener(Consumer<Time> fn);
    void removeTimeListener(Consumer<Time> fn);
    void addStateListener(Consumer<Boolean> fn);
    void removeStateListener(Consumer<Boolean> fn);
    void addRateListener(Consumer<Double> fn);
    void removeRateListener(Consumer<Double> fn);
}
