package edu.bu.signstream.media.gstreamer;

import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeUnit;
import org.freedesktop.gstreamer.*;
import org.freedesktop.gstreamer.query.DurationQuery;

import java.net.URI;
import java.util.HashMap;
import java.util.concurrent.CountDownLatch;

public class GstProbe {
    private final CountDownLatch latch = new CountDownLatch(2);
    private final Pipeline pipeline = new Pipeline();

    protected HashMap<String, Object> caps = new HashMap<>();

    public static HashMap<String, Object> probe(URI uri, String format) {
        return new GstProbe(uri, format).await();
    }

    private GstProbe(URI uri, String format) {
        var src = ElementFactory.make("uridecodebin", "decode");
        src.set("uri", uri.toString());

        src.connect(new Element.PAD_ADDED() {
            @Override
            public void padAdded(Element element, Pad pad) {
                var s = pad.getCurrentCaps().getStructure(0);
                System.out.println("PAD: " + pad);
                System.out.println("CAPS: " + s);
                if (s.getName().equals(format)) {
                    for (int i = 0; i < s.getFields(); i++) {
                        var key = s.getName(i);
                        if ("width".equals(key))
                            caps.put(key, s.getInteger(key));
                        else if ("height".equals(key))
                            caps.put(key, s.getInteger(key));
                        else if ("framerate".equals(key))
                            caps.put(key, s.getFraction(key));
                        else if ("rate".equals(key))
                            caps.put(key, s.getInteger(key));
                    }

                    src.disconnect(this);
                    latch.countDown();
                }
            }
        });
        pipeline.getBus().connect(new Bus.ASYNC_DONE() {
            @Override
            public void asyncDone(GstObject source) {
                var query = new DurationQuery(Format.TIME);
                src.query(query);
                caps.put("duration", new Duration(query.getDuration(), TimeUnit.NANOSECOND));

                pipeline.getBus().disconnect(this);
                latch.countDown();
            }
        });

        pipeline.getBus().connect((Bus.INFO) (arg0, arg1, arg2) ->
            System.err.println("<probe:info> " + arg0.getName() + " [" + arg1 + "]: " + arg2));
        pipeline.getBus().connect((Bus.ERROR) (arg0, arg1, arg2) ->
            System.err.println("<probe:error> " + arg0.getName() + " [" + arg1 + "]: " + arg2));

        pipeline.add(src);
    }

    private HashMap<String, Object> await() {
        pipeline.play();

        try {
            if (!latch.await(500, java.util.concurrent.TimeUnit.MILLISECONDS)) {
                throw new RuntimeException("Failed to find a compatible stream!");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
            System.out.println("<probe> Interrupted!");
        }

        pipeline.stop();
        pipeline.close();

        System.out.println("<probe> CAPS:");
        for (var e : caps.entrySet()) {
            System.out.println("  " + e.getKey() + ": " + e.getValue());
        }

        return caps;
    }
}
