package edu.bu.signstream.media.gstreamer.util;

import java.util.concurrent.*;

public class Executor {
    private final ExecutorService exec;
    private final String threadName;

    public Executor(ExecutorService exec, String threadName) {
        this.exec = exec;
        this.threadName = threadName;
    }

    public Executor(String threadName) {
        this(Executors.newSingleThreadExecutor(r -> new Thread(r, threadName)), threadName);
    }

    public <V> Future<V> callAsync(Callable<V> fn) {
        if (Thread.currentThread().getName().contains(threadName))
            throw new IllegalCallerException("Nested executor submissions!");

        return exec.submit(fn);
    }
    public Future<Void> runAsync(Runnable fn) {
        return callAsync(() -> {
            fn.run();
            return null;
        });
    }

    public <V> V call(Callable<V> fn) {
        try {
            return callAsync(fn)
                .get(10, TimeUnit.SECONDS);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    public void run(Runnable fn) {
        try {
            runAsync(fn)
                .get(10, TimeUnit.SECONDS);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void shutdown() {
        exec.shutdownNow();
    }
}
