package edu.bu.signstream.ui.panels;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.logging.Level;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;

import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.panels.newDatabase.ParticipantsPaneScrolledView;

public class AnnotatorSelectionPanel extends JPanel {
	
	private static final long serialVersionUID = 1L;
	
	private JComboBox annotatorSelectionComboBox = null;	
	private JLabel addNewAnnotatorLabel = null;	
	private JButton addNewAnnotatorButton = null;
	private JButton annotatorSelectionButton = null;	
	
	private AnnotatorSelectionDialog parentDialog;
	private ArrayList<SS3Participant> annotatorList;
	private AnnotatorsDialog annotatorsDialog = null;
	
	public AnnotatorSelectionPanel(AnnotatorSelectionDialog annotatorSelectionDialog, 
			ArrayList<SS3Participant> annotatorList, AnnotatorsDialog annotatorsDialog) {
		this.parentDialog = annotatorSelectionDialog;
		this.annotatorList = annotatorList;
		this.annotatorsDialog = annotatorsDialog;
		
		// dropdown to select an annotator from the current session
		ArrayList<String> annotatorStringList = getAllAnnotators();		
		annotatorSelectionComboBox = new JComboBox(annotatorStringList.toArray());
		annotatorSelectionComboBox.setMaximumSize(new Dimension(100, 40));
		annotatorSelectionComboBox.setPreferredSize(new Dimension(100, 40));
		
		// label to guide lost users
		String cantFindName = isParticipant() ? "a participant's" : "an annotator's";
		addNewAnnotatorLabel = new JLabel(
				"<html><body> Don't see " + cantFindName + " name?<br>Add it by clicking here: </body></html>");

		// button to the screen to add new annotator
		AddNewAnnotatorListener addNewAnnotatorListener = new AddNewAnnotatorListener(this);
		String addNew = isParticipant() ? "Add New Participant" : "Add New Annotator";
		addNewAnnotatorButton = new JButton(addNew);
		addNewAnnotatorButton.addActionListener(addNewAnnotatorListener);		
		
		// button to select the annotator chosen from the dropdown
		annotatorSelectionButton = new JButton("Confirm Selection");
		SelectAnnotatorListener selectAnnotatorListener = new SelectAnnotatorListener(this);
		annotatorSelectionButton.addActionListener(selectAnnotatorListener);

		setLayout(new GridLayout(0, 1));
		add(annotatorSelectionComboBox);
		add(addNewAnnotatorLabel);
		add(addNewAnnotatorButton);
		add(new JSeparator());
		add(annotatorSelectionButton);
	}
	
	public AnnotatorSelectionDialog getParentDialog() {
		return parentDialog;
	}
	
	public AnnotatorsDialog getAnnotatorsDialog() {
		return annotatorsDialog;
	}
	
	public ArrayList<String> getAllAnnotators() {
		ArrayList<String> ret = new ArrayList<String>();
		
		for (SS3Participant pv: annotatorList) {
			ret.add(pv.getLabel());
		}
		
		Collections.sort(ret);
		
		return ret;		
	}
	
	public SS3Participant getSelectedAnnotator() {
		String selectedAnnotatorLabel = (String) annotatorSelectionComboBox.getSelectedItem();
		SS3Participant selectedAnnotator = null;
		
		for (SS3Participant pv: annotatorList) {
			if (selectedAnnotatorLabel.equals(pv.getLabel())) {
				selectedAnnotator = pv;
				break;
			}
		}
		
		return selectedAnnotator;
	}
	
	protected ArrayList<SS3Participant> getAnnotatorList() {
		return annotatorList;
	}
	
	public void addAndSetComboBoxSelectedTo(String currentSelectedAnnotator) {
		annotatorSelectionComboBox.addItem(currentSelectedAnnotator);
		annotatorSelectionComboBox.setSelectedItem(currentSelectedAnnotator);
		
		revalidate();
		repaint();
	}
	
	public boolean isParticipant() {
		return annotatorsDialog.getAnnotatorsPanel().getParticipantsPaneScrolledView().isParticipant();
	}
}

class SelectAnnotatorListener implements ActionListener {
	private AnnotatorSelectionPanel panel = null;
	
	public SelectAnnotatorListener(AnnotatorSelectionPanel parent) {
		this.panel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		SS3Participant annotator = panel.getSelectedAnnotator();
		if (annotator == null) {
			// can make this a dialog here to let user know to add someone before selecting
			return;
		}

		String currentlySelectedAnnotator = annotator.getLabel();

		// do nothing for participants
		// if the selected item is needed, then call panel.getSelectedAnnotator() afterwards
		if ( !panel.isParticipant()) {
			// singleton select annotator
			SS3Singleton.setCurrentAnnotator(currentlySelectedAnnotator);
			
			ArrayList<String> newAnnotatorList = panel.getAllAnnotators();
			MediaToolBar mtb = SS3Singleton.getMediaToolBar(SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel());
			mtb.updateAnnotatorList(newAnnotatorList, currentlySelectedAnnotator);
		}	
		
		
		panel.getParentDialog().dispose();
	}	
}

class AddNewAnnotatorListener implements ActionListener {
	private AnnotatorSelectionPanel panel = null;
	
	public AddNewAnnotatorListener(AnnotatorSelectionPanel parent) {
		this.panel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		panel.getAnnotatorsDialog().setLocation(250, 250);
		panel.getAnnotatorsDialog().setVisible(true);
		panel.getAnnotatorsDialog().setModal(true);
		
		// dialog will be modal here, waiting to be completed by user
		
		// then, after user closes the dialog, proceed to add that new annotator (if any) to the scrolled view		
		AnnotatorsPanel annotatorsPanel = panel.getAnnotatorsDialog().getAnnotatorsPanel();		
		SS3Participant annotatorJustCreated = annotatorsPanel.getNewSelectedParticipant();
		
		// if an annotator was selected, and it has not been added to the list already
		if (annotatorJustCreated != null && !panel.getAllAnnotators().contains(annotatorJustCreated.getLabel())) {
			boolean proceed = true;
			
	        // set annotator or participant in database
			if (panel.isParticipant())
				proceed = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().addSelectedParticipant(annotatorJustCreated);
			else
				proceed = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().addSelectedAnnotator(annotatorJustCreated);
			
			if (proceed) {				
				ParticipantsPaneScrolledView annotatorsPaneScrolledView = annotatorsPanel.getParticipantsPaneScrolledView();
				annotatorsPaneScrolledView.addParticipant(annotatorJustCreated);
		        
		        // write annotator or participant to disk
				edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
		        try {
		        		if (panel.isParticipant())
		        			writer.saveParticipantsLibrary(annotatorsPaneScrolledView.getParticipantLibrary());
		        		else
		        			writer.saveAnnotatorsLibrary(annotatorsPaneScrolledView.getParticipantLibrary());
		        } catch (Exception ex) {
		            ex.printStackTrace();
					SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		        }
				
				// update combobox with new list of annotators
				panel.addAndSetComboBoxSelectedTo(annotatorJustCreated.getLabel());
				panel.getAnnotatorList().add(annotatorJustCreated);
			}
			
			SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().update(
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database());
		}
	}
}
