package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.*;

import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.CustomFieldsView;
import edu.bu.signstream.grepresentation.view.GestureNRelatedView;
import edu.bu.signstream.grepresentation.view.GlossNRelatedView;
import edu.bu.signstream.grepresentation.view.NonmanualFieldsView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.frame.Blockable;
import edu.bu.signstream.ui.frame.Blocker;

public class CSFieldDialog extends JDialog implements Blockable {

	private static final long serialVersionUID = 1L;
	
	private CSFieldPanel csFieldPanel = null;
	private final Blocker blocker;
	
	public CSFieldDialog(SignStreamSegmentPanel segmentPanel, Segment segment,
						 NonmanualFieldsView nonManualView, GlossNRelatedView glossNRelatedView, 
						 CustomFieldsView customView, GestureNRelatedView gestureView, PresentationField parentField) {

		super(SS3Singleton.getSignStreamApplication(), false);	// sets modal to true
		setTitle("Field Editor");

		csFieldPanel = new CSFieldPanel(segmentPanel, segment, nonManualView, this,
				glossNRelatedView, customView, gestureView, parentField);
		csFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		blocker = new Blocker(csFieldPanel, Blocker.Darken());

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(blocker, BorderLayout.CENTER);

		addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent we) {
				csFieldPanel.discard();
				dispose();
			}
		});

		setSize(
				getPreferredSize().width + 15,
				getPreferredSize().height + 50);
		setLocation(150, 150);
		pack();
	}
	
	public CSFieldPanel getCSFieldPanel() {
		return csFieldPanel;
	}

	public void setSelectedField(String fieldID, boolean selected) {
		csFieldPanel.setSelectedField(fieldID, selected);
	}
	
	public void apply() {
	    csFieldPanel.apply();
	}
	
	public Segment getSegment() {
		return csFieldPanel.getSegment();
	}

    @Override
    public void dispose() {
        super.dispose();
        SS3Singleton.getSignStreamApplication().unblock();
    }

    public void updateCurrentUtterance() {
        csFieldPanel.updateCurrentUtterance();
        csFieldPanel.discard();
    }

    public void setVisible(boolean b, String fieldID) {
		if (b)
            SS3Singleton.getSignStreamApplication().block();
		else
            SS3Singleton.getSignStreamApplication().unblock();

		updateCurrentUtterance();
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				csFieldPanel.scrollToAndSelectFieldByID(fieldID);
			}
		});

		super.setVisible(b);
	}

    @Override public void block() { blocker.block(); }
    @Override public void unblock() { blocker.unblock(); }
    @Override public boolean blocked() { return blocker.blocked(); }
}
