package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.ArrayList;

import javax.swing.*;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.panels.DocumentModelListener;

public class NewCSValuePanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private JDialog popup = null;

	private JLabel title = new JLabel("Field Specification Editor");
	private JLabel newValueNameLbl = new JLabel("New Value Name:");
	private JLabel newValueLabelLbl = new JLabel("New Value Label:");
	private JTextField newValueNameTxt = new JTextField(15);
	private JTextField newValueLabelTxt = new JTextField(15);

	private final int CANCEL = 1;
	private final int OK = 2;

	private JButton btnCancel = new JButton("Cancel");
	private JButton btnOk = new JButton("Done");

	private String fieldID = "";
	private ChainedEvent event = null;
	private SignStreamSegmentPanel panel = null;
	
	JPanel centerPanel = new JPanel();
	GridBagConstraints gbc = new GridBagConstraints();		

	public NewCSValuePanel(String fieldID, ChainedEvent event,
			SignStreamSegmentPanel panel, JDialog popup) {
		this.popup = popup;
		this.fieldID = fieldID;
		this.event = event;
		this.panel = panel;
		
		this.popup.addWindowFocusListener(new WindowFocusListener() {
			@Override public void windowLostFocus(WindowEvent e) {
				NewCSValuePanel.this.popup.setVisible(false);
			}

			@Override public void windowGainedFocus(WindowEvent e) {}
		});

		UserInterfaceUtil.setLabelLNF(newValueNameLbl);
		UserInterfaceUtil.setLabelLNF(newValueLabelLbl);
		UserInterfaceUtil.setTextFieldLNF(newValueNameTxt);
		UserInterfaceUtil.setTextFieldLNF(newValueLabelTxt);
		
		btnOk.setEnabled(false); 
		newValueNameTxt.getDocument().addDocumentListener(
				new DocumentModelListener(newValueNameTxt, btnOk));
		newValueLabelTxt.getDocument().addDocumentListener(
				new DocumentModelListener(newValueLabelTxt, btnOk));

		Dimension d = btnCancel.getPreferredSize();
		btnOk.setPreferredSize(d);
		btnOk.setMaximumSize(d);

		centerPanel.setLayout(new GridBagLayout());		
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridwidth = 3;
		gbc.gridx = 0;
		gbc.gridy = 0;
		centerPanel.add(title, gbc);

		gbc.gridy = 2;
		gbc.gridx = 0;
		gbc.gridwidth = 1;
		centerPanel.add(newValueNameLbl, gbc);
		centerPanel.add(Box.createHorizontalStrut(10));
		gbc.gridx = 2;
		centerPanel.add(newValueNameTxt, gbc);
		centerPanel.add(Box.createHorizontalStrut(10));
		gbc.gridx = 4;
		centerPanel.add(btnOk, gbc);

		gbc.gridy = 3;
		gbc.gridx = 0;
		centerPanel.add(newValueLabelLbl, gbc);
		centerPanel.add(Box.createHorizontalStrut(10));
		gbc.gridx = 2;
		centerPanel.add(newValueLabelTxt, gbc);
		centerPanel.add(Box.createHorizontalStrut(10));
		gbc.gridx = 4;
		centerPanel.add(btnCancel, gbc);

		btnOk.setActionCommand(OK + "");
		btnOk.setToolTipText("Add new Value to local coding schema, set Value to selected Event");
		btnOk.addActionListener(this);

		btnCancel.setActionCommand(CANCEL + "");
		btnCancel.addActionListener(this);
		
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 0));
		add(centerPanel, BorderLayout.CENTER);

		SwingUtilities.invokeLater( new Runnable() { 
			@Override
			public void run() { 
				newValueNameTxt.requestFocus(); 
			} 
		}); 
	}	

	private boolean isUnique(String valueName, String valueLabel, SS3Field field) {
		ArrayList<SS3FieldValue> values = field.getValuesAsList();
		for(SS3FieldValue value:values) {
			if(valueName.equalsIgnoreCase(value.getName()) || valueLabel.equalsIgnoreCase(value.getLabel())) {
				return false;
			}
		}
		return true;
	}
	
	boolean expanded = false; 
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case CANCEL:
			popup.setVisible(false);
			break;
		case OK:		
			SS3FieldValue newValue = new SS3FieldValue();
			String valueName = newValueNameTxt.getText();
			String valueLabel = newValueLabelTxt.getText();
			
			if(valueName.length()>0 && valueLabel.length()==0)
				valueLabel = valueName;
			
			if(valueName.length()==0 && valueLabel.length()>0) 
				valueName = valueLabel; 

			SS3CodingScheme dbScheme = panel.getSS3Database().getSS3CodingScheme(); 
			SS3Field field = dbScheme.getField(fieldID);
			
			boolean unique = isUnique(valueName, valueLabel, field);
			
			if(unique) {
				newValue.setId("" + Util.getLongUniqueNumber());
				newValue.setName(newValueNameTxt.getText());
				newValue.setLabel(newValueLabelTxt.getText());
		
				field.addValue(newValue);				
				
				event.setID(newValue.getId());
				
				popup.setVisible(false);
				panel.repaint();
			} else if(!expanded) {
				expanded = true;
				
				JLabel msgLabel1 = new JLabel("The selected value name and/or label is already in use by another");
				JLabel msgLabel2 = new JLabel("value within this field. Please change it or use the Cancel button"); 
				JLabel msgLabel3 = new JLabel("to exit the editor.");
				
				UserInterfaceUtil.setLabelLNF(msgLabel1);
				UserInterfaceUtil.setLabelLNF(msgLabel2);
				UserInterfaceUtil.setLabelLNF(msgLabel3);

				gbc.gridwidth = 5;
				gbc.gridx = 0;
				gbc.gridy = 5;
				centerPanel.add(msgLabel1, gbc);				
				gbc.gridy = 6;
				centerPanel.add(msgLabel2, gbc);				
				gbc.gridy = 7;
				centerPanel.add(msgLabel3, gbc);			
				
				popup.pack();
			}
			break;
		}
	}
}
