package edu.bu.signstream.ui.panels.search;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.zip.ZipFile;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.filechooser.FileFilter;

import edu.bu.signstream.common.files.FilesLocation;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationFieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.RequestHandler;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentTemplate;
import edu.bu.signstream.grepresentation.view.UtteranceView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Zoomer;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SearchPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	private JLabel searchDomain = null;
	private JButton changeDomainButton = null;
	
	private JLabel searchQueryBuilderLabel = new JLabel("Query Builder: \n");
	private SearchQueryBuilderPanel searchQueryBuilderPanel = null;
	
	private JLabel searchQueryTextLabel = new JLabel("Query: \n");
	private JTextArea searchQueryTextPanel = null;
	
	private JButton clearButton = null;
	private JButton searchButton = null;
	private JButton returnOriginalState = null;
	
	private JSplitPane splitPane = null;
	
	private JList<String> resultsList = null;
	private JScrollPane resultsListScroller = null;
	private ResultsListSelectionListener resultsListSelectionListener = null;
	
	private JButton saveSearchButton = null;
	private JButton openSavedSearchButton = null;
	
	private SearchEngine searchEngine = null;
	
	public SearchSavedState originalSavedState;
	public SearchSavedState mostRecentSavedState;
	private HashMap<Integer, SearchSavedState> searchSavedStates;
	private String currentResultVisible = null;
	private int currentResultIndexVisible = -1;
	
	private SS3SignStreamApplication application;
	private SS3Database database;
	
	private final int CHANGE_DOMAIN = 10;
	private final int SEARCH = 20;
	private final int RETURN_ORIGINAL_STATE = 30;
	private final int CLEAR = 40;
	private final int SAVE_SEARCH = 50;
	private final int OPEN_SEARCH = 60;
	
	private final String SEARCH_FILE_EXTENSION = ".search";
	
	public SearchPanel(SS3SignStreamApplication application) {
		this.application = application;
		
		application.getSignStreamSegmentPanel().updateLoadedDatabase();
		
		if (application.getSignStreamSegmentPanel() != null) {
			if (application.getSignStreamSegmentPanel().getSS3Database() != null) {
				this.database = application.getSignStreamSegmentPanel().getSS3Database();
				
				// only save original state once per each search window session
				if (application.getSignStreamSegmentPanel().isUtteranceLoaded()) {
					mostRecentSavedState = application.getSignStreamSegmentPanel().getUtteranceView().saveCurrentState();
					originalSavedState = application.getSignStreamSegmentPanel().getUtteranceView().saveCurrentState();
				}
			}
		}

		searchEngine = new SearchEngine(database);
		
		searchSavedStates = new HashMap<Integer, SearchSavedState>();
		
		createUI();
	}
	
	private void createUI() {
		SignStreamSegmentPanel segmentPanel = application.getSignStreamSegmentPanel();
		if (segmentPanel == null) {
			return;
		}
		
		String dName = database.getDirectory();
		if (dName.startsWith(SS3Database.tempDirectoryPrefix)) {
			dName = dName.substring(SS3Database.tempDirectoryPrefix.length());
		}
//		searchNote = new JTextArea("Note: While the Search Window is open, you will be temporarily unable to perform some menu functions, such as directly opening and closing collections from the main menu, or returning to Database View.");
		searchDomain = new JLabel("Search Domain: " + dName);
		
		changeDomainButton = new JButton("Change Search Domain");
		changeDomainButton.addActionListener(this);
		changeDomainButton.setActionCommand("" + CHANGE_DOMAIN);
		
		searchQueryBuilderPanel = new SearchQueryBuilderPanel(database);
		
		searchQueryTextPanel = new JTextArea();
		
		searchButton = new JButton("Perform Search");
		searchButton.addActionListener(this);
		searchButton.setActionCommand("" + SEARCH);
		
		clearButton = new JButton("Clear Search");
		clearButton.addActionListener(this);
		clearButton.setActionCommand("" + CLEAR);
		
		returnOriginalState = new JButton("Return to Original State");
		returnOriginalState.addActionListener(this);
		returnOriginalState.setActionCommand("" + RETURN_ORIGINAL_STATE);
		
		resultsList = new JList<String>();
		resultsListScroller = new JScrollPane(resultsList);
		resultsListScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		
		resultsListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		resultsList.setDragEnabled(false);
		resultsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		resultsListSelectionListener = new ResultsListSelectionListener(this, resultsList);
		resultsList.getSelectionModel().addListSelectionListener(resultsListSelectionListener);
		
		JPanel top = new JPanel();
		JPanel bottom = new JPanel();
		
		top.setLayout(new GridBagLayout());
		bottom.setLayout(new GridBagLayout());

		saveSearchButton = new JButton("Save Search");
		saveSearchButton.addActionListener(this);
		saveSearchButton.setActionCommand("" + SAVE_SEARCH);
		
		openSavedSearchButton = new JButton("Open Search");
		openSavedSearchButton.addActionListener(this);
		openSavedSearchButton.setActionCommand("" + OPEN_SEARCH);
		
		GridBagConstraints c = new GridBagConstraints();

		c.gridy = 0;
		
		c.weightx = 1.0;
		c.gridwidth = 1;
		
		c.gridx = 0;
		c.anchor = GridBagConstraints.NORTHWEST;
		top.add(searchDomain, c);
		
		c.gridx = 1;
		c.anchor = GridBagConstraints.NORTHEAST;
		top.add(changeDomainButton, c);
		
		c.gridx = 2;
		c.anchor = GridBagConstraints.EAST;
//		top.add(returnOriginalState, c);	// NOTE: just re-add this if we want to re-enable return to original - it worked fine prior to commenting this
		
		c.gridx = 0;
		c.gridwidth = 3;		
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.BOTH;

		c.gridy++;
		JSeparator sep = new JSeparator(JSeparator.HORIZONTAL);
		sep.setPreferredSize(new Dimension(5, 5));
		top.add(sep, c);
		
		c.gridy++;
		top.add(searchQueryBuilderLabel, c);
		c.weighty = 0.8;
		c.gridy++;
		top.add(searchQueryBuilderPanel, c);

		c.weighty = 0;
		c.gridwidth = 1;
		
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.WEST;
		bottom.add(searchButton, c);
		
		c.gridx = 1;
		c.anchor = GridBagConstraints.CENTER;
		bottom.add(clearButton, c);
		
		c.gridx++;
		c.anchor = GridBagConstraints.EAST;
		bottom.add(saveSearchButton, c);
		c.gridx++;
		bottom.add(openSavedSearchButton, c);
		
		c.gridx = 0;

		c.gridy++;
		c.weighty = 0.2;
		c.anchor = GridBagConstraints.SOUTH;
		c.fill = GridBagConstraints.BOTH;
		c.gridwidth = 5;
		
		bottom.add(resultsListScroller, c);
		
		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setTopComponent(top);
		splitPane.setBottomComponent(bottom);
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(splitPane);
	}
	
	private void saveCurrentState() {
		UtteranceView utteranceView = application.getSignStreamSegmentPanel().getUtteranceView();
		if (utteranceView == null) 
			return;
		
		SearchSavedState tempState = utteranceView.saveCurrentState();
		
		if (currentResultIndexVisible > -1) {
			searchSavedStates.put(currentResultIndexVisible, tempState);
		}
	}
	
	private void revertToSavedState(SearchSavedState searchSavedState) {		
		SignStreamSegmentPanel panel = application.getSignStreamSegmentPanel();
		
		SS3Database savedStateDatabase = searchSavedState.getDatabase();
		
//		if ( !savedStateDatabase.equals(database)) {
			
			// must perform this first - in case we don't actually want to switch then stop here
			boolean success = changeDomain(searchSavedState);
//		}

		UtteranceView utteranceView = this.application.getSignStreamSegmentPanel().getUtteranceView();
		if (utteranceView == null)
			return;
		
//		this.application.getSignStreamSegmentPanel().setSS3Database(database);

		utteranceView.parseSegmentStringAndReloadView(searchSavedState.getSegmentLabel(), searchSavedState.getTemporalPartitionLabel(), searchSavedState.getSelectedUtterance());
	}
	
	private void loadSearchResult() {
		SignStreamSegmentPanel panel = application.getSignStreamSegmentPanel();

//		if (panel.getSS3Database().equals(this.database)) {
			StringTokenizer tokSemicolon = new StringTokenizer(this.currentResultVisible, ";");
			String partitionLabel = tokSemicolon.nextToken();
			String segmentLabel = tokSemicolon.nextToken();
			String utteranceLabelAndGlosses = tokSemicolon.nextToken();

			StringTokenizer tokSpace = new StringTokenizer(utteranceLabelAndGlosses, " ");
			String utteranceLabel = tokSpace.nextToken();

			PresentationEventsEntity utterance = panel.getUtteranceView().getUtteranceFromString(partitionLabel, segmentLabel, utteranceLabel);
			
			UtteranceView utteranceView = panel.getUtteranceView();
			if (utteranceView == null)
				return;
			
			utteranceView.parseSegmentStringAndReloadView(segmentLabel, partitionLabel, utteranceLabel);
			
			// Scroll to the utterance
            var time = utterance.getStartTimeInfo();
			panel.getUtteranceView().getSignStreamSegmentPanel().scroll(time);

	        panel.fitZoom(utterance.interval().duration());	        
            SS3Singleton.getMediaPlayer().seek(time);
            
			panel.getUtteranceView().showAllFields();

			return;
//		}
	}
	
	private ArrayList<SS3GlossWindowItem> search(SearchTreeNode searchTreeNode) {
		return searchEngine.search(searchTreeNode);
	}

	private boolean isValidZipFile(File file) {
		try {
			ZipFile zipFile = new ZipFile(file);
			zipFile.close();
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	private boolean doesCollectionExistForDatabase(SS3Database database) {
		String directoryPath = database.getParentDirectoryPath();
		String collectionName = database.getDirectory();

		if (collectionName.startsWith(SS3Database.tempDirectoryPrefix)) {
			collectionName = collectionName.substring(SS3Database.tempDirectoryPrefix.length());
		}


		File collectionPath = new File(directoryPath + File.separator + collectionName);

		if (collectionPath.exists() && this.isValidZipFile(collectionPath)) {
			return true;
		} else if (collectionPath.exists() && !this.isValidZipFile(collectionPath)) {
			return false;
		}

		return false;
	}

	private DefaultMutableTreeNode getDBNavNode() {
		Object root = this.application.getSignStreamSegmentPanel().getDbNavigationPanel().getJTree().getModel()
				.getRoot();

		if (root == null)
			return null;

		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
		int childCount = rootNode.getChildCount();
		DefaultMutableTreeNode node = null;

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		return node;
	}

	private boolean forceSaveCollection(SS3Database database, int save_type) {
		if (save_type != SS3SignStreamApplication.SAVE_AS && save_type != SS3SignStreamApplication.SAVE) {
			return false;
		}

		int result = JOptionPane.NO_OPTION;
		
		if (save_type == SS3SignStreamApplication.SAVE) {
			result = JOptionPane.OK_OPTION;
		} else {
		
			// need to first grab the DBnav node if we want to save the database
			String forceSaveCollectionText = "This collection requires saving before proceeding before changing the search domain.\n\n" +
					"Would you like to save the collection now?";
	
			String forceSaveCollectionTitle = "Save Collection";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.WARNING_MESSAGE;
			String options[] = {"Save", "Do Not Save"};
	
			try {
				result = JOptionPane.showOptionDialog(this, forceSaveCollectionText, forceSaveCollectionTitle,
						optionType, messageType, null, options, options[1]);
			} catch (HeadlessException he) {
				SS3SignStreamApplication.logger.log(Level.INFO, he.getMessage());
			}
		}

		// if user chose to save the open database
		if (result == JOptionPane.OK_OPTION) {
			// need to first grab the DBnav node if we want to save the database

			DefaultMutableTreeNode node = this.getDBNavNode();

			if (node == null) {
				application.confirmationDialog("Save Collection Warning",
											   "Collection not available");
				return false;
			}

			// save currently open database
			File savedFile = this.application.saveCollection(save_type);
			database.setDirectory(savedFile.getName());
			database.setParentDirectoryPath(savedFile.getParent() + File.separator);
			this.application.getSignStreamSegmentPanel().setSS3Database(database);

			return true;
		} else {
			return false;
		}
	}
	
	private boolean changeDomain(SearchSavedState searchSavedState) {
		SS3Database optionalDatabase = null;
		if (searchSavedState != null) {
			optionalDatabase = searchSavedState.getDatabase();
		}
		
		SS3Database applicationDatabase = this.application.getSignStreamSegmentPanel().getSS3Database();		// current database
		if ( (optionalDatabase == null && !this.doesCollectionExistForDatabase(applicationDatabase))
				|| SS3Singleton.getApplicationStateController().isMediaAssociatedRequiredSaving()) {
			// this is necessary to do, for cases like where the user just associated the videos to their collection and has to re-save the collection
			boolean success = this.forceSaveCollection(applicationDatabase, SS3SignStreamApplication.SAVE_AS);
			if (!success) {
				return true;		// means there was a cancel, so it's fine
			}
		}
		
		if (isInOriginalState() && searchSavedState == null && applicationDatabase.equals(database)) {
				
			String warningDialogText =
					"<html>"
					+ "You are about to open a new SignStream collection, keeping the search window active.<br><br>"
							+ "The search settings you have defined will remain unchanged.<br><br>"
							+ "You will be given the chance to save any local changes made to utterances before changing collections.<br>"
//							+ "<br>It is possible to return to your original collection and utterance by clicking \"Revert to Original State\""
							+ "</html>";
			
			application.confirmationDialog("Search Warning",
					warningDialogText);			
		}
		
		// save the state before switching...if error then it reverts to this
		mostRecentSavedState = application.getSignStreamSegmentPanel().getUtteranceView().saveCurrentState();

		// switch databases
		if (optionalDatabase != null) {			
			this.database = optionalDatabase;
			boolean result = this.application.openCollectionForSearch(new File(this.database.getParentDirectoryPath() + File.separator + this.database.getDirectory()), this);
			if (result == false) {
				return false;
			}			
		} else {			
			boolean result = this.application.openCollectionForSearch(null, this);
			if (result == false) {
				return false;
			}
			this.database = application.getSignStreamSegmentPanel().getSS3Database();			
		}

		String dName = this.database.getDirectory();
		if (dName.startsWith(SS3Database.tempDirectoryPrefix)) {
			dName = dName.substring(SS3Database.tempDirectoryPrefix.length());
		}
        this.searchDomain.setText("Search Domain: " + dName);

        this.updateQueryBuilderPanel();
		
		application.getSignStreamSegmentPanel().getDbNavigationPanel().enterUtteranceView();

		// snap to the first utterance of default segment
		var segment = application.getSignStreamSegmentPanel().getUtteranceView().getSegment();
		if (segment != null) {
			try {
				// try catch because there might not be utterances, which means index out of bounds below
		        EventsEntity utterance = segment.getParentField().getEntities().get(0);
		        application.getSignStreamSegmentPanel().fitZoom(utterance.interval().duration());
			} catch (Exception e) {
				
			}
		}

		searchEngine = new SearchEngine(database);		

		clearResultPanel();
		
		return true;
	}
	
	public void saveSearch() {
		SearchTreeNode root = getQueryBuilderPanelRootNode();
		
		try {
			var frame = SS3Singleton.getSignStreamApplication();
			
			JFileChooser fc = new JFileChooser();
			fc.setDialogTitle("Save Search As...");
			fc.setFileFilter(new SavedSearchFileFilter());
			fc.setAcceptAllFileFilterUsed(false);
						
			// open the location of the last opened database
			FilesLocation lastLocation = SS3Singleton.getFilesLocation();
			File lastOpened = new File(lastLocation.getLastOpenedDatabaseLocation());			
			fc.setCurrentDirectory(lastOpened);
			
			int result = fc.showSaveDialog(frame);
			
			if (result == JFileChooser.APPROVE_OPTION) {
				File selectedFile = fc.getSelectedFile();
				if (selectedFile != null) {
					String filePath = selectedFile.getAbsolutePath();
					if ( ! filePath.endsWith(SEARCH_FILE_EXTENSION)) {
						filePath = filePath + SEARCH_FILE_EXTENSION;
					}

					FileOutputStream fos = new FileOutputStream(filePath);
					ObjectOutputStream oos = new ObjectOutputStream(fos);
					oos.writeObject(root);
					oos.close();
					fos.close();

					JOptionPane.showMessageDialog(frame, "Search saved successfully!");
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void openSearch() {		
		SearchTreeNode savedSearchTreeNode = null;

		try {
			var frame = SS3Singleton.getSignStreamApplication();
			
			JFileChooser fc = new JFileChooser();
			fc.setDialogTitle("Open Saved Search...");
			fc.setFileFilter(new SavedSearchFileFilter());
			fc.setAcceptAllFileFilterUsed(false);
			
			// open the location of the last opened database
			FilesLocation lastLocation = SS3Singleton.getFilesLocation();
			File lastOpened = new File(lastLocation.getLastOpenedDatabaseLocation());
			fc.setCurrentDirectory(lastOpened);

			int result = fc.showOpenDialog(frame);
			
			if (result == JFileChooser.APPROVE_OPTION) {
	            File selectedFile = fc.getSelectedFile();
				
				if (selectedFile != null) {	
					FileInputStream fis = new FileInputStream(selectedFile);
					ObjectInputStream ois = new ObjectInputStream(fis);
					savedSearchTreeNode = (SearchTreeNode) ois.readObject();
					ois.close();
					fis.close();

					searchQueryBuilderPanel.loadSavedSearchTreeNode(savedSearchTreeNode);

					clearResultPanel();
				}
			}
		} catch (IOException i) {
			i.printStackTrace();
			return;
		} catch (ClassNotFoundException c) {
			System.out.println("SearchTreeNode class not found");
			c.printStackTrace();
			return;
		}				
	}
	
	public SearchTreeNode getQueryBuilderPanelRootNode() {
		return searchQueryBuilderPanel.getRootSearchTreeNode();
	}
	
	public void updateQueryBuilderPanel() {
		searchQueryBuilderPanel.updateQuery(database);
	}
	
	public String getQueryTextPanelText() {
		return searchQueryTextPanel.getText();
	}
	
	private void updateResultPanel(ArrayList<SS3GlossWindowItem> resultItems) {
		ArrayList<String> utterances = new ArrayList<String>();
		
		for (int i = 0; i < resultItems.size(); i++) {
			SS3GlossWindowItem item = resultItems.get(i);
			String fieldID;
			SS3TemporalPartition partition = database.getSS3GlossWindow()
					.getTemporalPartitionByLabel(item.getPartitionLabel());
			SS3GlossWindowSegment segment = partition.getTemporalPartitionSegmentByLabel(item.getSegmentLabel());
			if (segment.getParticipant() == null || segment.getParticipant().isDominantHandRight()) {
				fieldID = Util.DOM_GLOSS_FIELD_ID;
			} else {
				fieldID = Util.NDOM_GLOSS_FIELD_ID;
			}
			StringBuilder combinedTextLabels = new StringBuilder("  ");
			
//			if (item.getSS3StatementField(fieldID) != null) {
			if (segment.getSS3StatementField(fieldID) != null) {
				// entities are in order by startTime
				ArrayList<SS3Entity> ss3Entities = segment.getSS3StatementField(fieldID).getSS3Entities();
				
				for (SS3Entity ss3Entity: ss3Entities) {
					if (ss3Entity.getParentEntityIDs() != null) {
						boolean partiallyWithinUtterance = false;
						for (String parentEntityID: ss3Entity.getParentEntityIDs()) {
							if (item.getId().equalsIgnoreCase(parentEntityID)) {
								partiallyWithinUtterance = true;
							}
						}
						if (! partiallyWithinUtterance)
							continue;
					}
					ArrayList<SS3Item> entItems = ss3Entity.getItems();
					for (SS3Item entItem: entItems) {
						if (entItem.isTextOnly())	// means it's just entered text (i.e. pre-gloss existing)
							continue;
						
						try {
							Integer id = Integer.valueOf(entItem.getId());
							if (Constants.PHASES_CODES.contains(id))
								continue;
						} catch (NumberFormatException e) {
							// do nothing - just means the ID was too large and is likely just a label, so include it below
						}
						
						combinedTextLabels.append(entItem.getText()).append(" ");
					}
				}
			}
			
			String temporalPartitionLabel = item.getPartitionLabel();
			String segmentLabel = item.getSegmentLabel();
			String utteranceLabel = item.getLabel();
			
			String combinedItemLabel = temporalPartitionLabel + ";" + segmentLabel + ";" + utteranceLabel;
			
			utterances.add(combinedItemLabel + combinedTextLabels);
		}
		
		resultsList.setListData(Arrays.copyOf(utterances.toArray(), utterances.size(), String[].class));
		
		// clear out the old
		searchSavedStates.clear();
		setCurrentResultVisible(null);
		setCurrentResultIndexVisible(-1);
	}
	
	private void executeSearch() {
		updateResultPanel(search(getQueryBuilderPanelRootNode()));
	}
	
	private void clearResultPanel() {
		// pass in empty arraylist to clear the entire results list
        this.updateResultPanel(new ArrayList<>());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case CHANGE_DOMAIN:
			
//			application.block();
			
			boolean success = changeDomain(null);

			if ( ! success) {
				String errorText =
						"<html>"
						+ "There was an error opening this collection.<br>"
								+ "Please ensure the collection being opened is fully functional and is correctly associated with any required videos.<br><br>"
								+ "The original file will be displayed again, in the original state in which you left the search window.<br>"
								+ "The search settings you have defined will remain unchanged."
								+ "</html>";
				
				application.confirmationDialog("Collection Error",
						errorText);		
				
				this.revertToSavedState(mostRecentSavedState);
			}
			
//			application.unblock();
			
			executeSearch();

			break;
			
		case SEARCH:	
			executeSearch();
			
			break;
			
		case RETURN_ORIGINAL_STATE:
			if (this.originalSavedState != null) {
				this.saveCurrentState();
				this.revertToSavedState(this.originalSavedState);

				this.resultsList.clearSelection();
				this.resultsList.setListData(new String[0]);
//				this.searchQueryBuilderPanel.getOuterSearchGroup().clear();
			}
			
			// need to reset these here in case we are switching back to original state from within the same database
			// in that case, we shouldn't mistake the original state for an element of the searchSavedStates map
			setCurrentResultVisible(null);
			setCurrentResultIndexVisible(-1);
			
			executeSearch();
			
			break;
		case SAVE_SEARCH:
			saveSearch();
			break;
		case OPEN_SEARCH:
			openSearch();
			break;
		
		case CLEAR:
			clearResultPanel();
		    searchQueryBuilderPanel.getOuterSearchGroup().clear();

		}
	}
	
	public void setCurrentResultVisible(String currentResultVisible) {
		this.currentResultVisible = currentResultVisible;
	}
	
	public String getCurrentResultVisible() {
		return currentResultVisible;
	}
	
	public void setCurrentResultIndexVisible(int currentResultIndexVisible) {
		this.currentResultIndexVisible = currentResultIndexVisible;
	}
	
	public int getCurrentResultIndexVisible() {
		return currentResultIndexVisible;
	}
	
	public boolean isInOriginalState() {
		SS3Database savedStateDatabase = originalSavedState.getDatabase();

		return savedStateDatabase.equals(database);
	}

	private class ResultsListSelectionListener implements ListSelectionListener {
		private SearchPanel searchPanel = null;
		private JList resultsList = null;
	
		public ResultsListSelectionListener(SearchPanel searchPanel, JList resultsList) {
			this.searchPanel = searchPanel;
			this.resultsList = resultsList;
		}
	
		@Override
		public void valueChanged(ListSelectionEvent e) {
			// this will proceed once the value is no longer adjusting, meaning the mouse click is released
			if (e.getValueIsAdjusting())
				return;
			
//			application.block();
			
	        ListSelectionModel resultsListSelectionModel = (ListSelectionModel) e.getSource();
	        
	        if (resultsListSelectionModel.isSelectionEmpty()) {
	        		return;
	        } else {
	        		int resultsListIndex = resultsListSelectionModel.getMaxSelectionIndex();
	        		String resultsListString = (String) resultsList.getModel().getElementAt(resultsListIndex);
	        		
	        		int currentIndexVisible = searchPanel.getCurrentResultIndexVisible();
	        		
	        		if (resultsListIndex == currentIndexVisible) {
	        			// do nothing, since we have selected the result we are already viewing
	        			return;
	        		} else {
	        			searchPanel.saveCurrentState();
	        			
	        			searchPanel.setCurrentResultIndexVisible(resultsListIndex);
	        			searchPanel.setCurrentResultVisible(resultsListString);
	        			
	        			searchPanel.loadSearchResult();	        			
	        		}
	        }
	        
//	        application.unblock();
			
		}
		
	}

	class SavedSearchFileFilter extends FileFilter {
		String ext;
	
		public SavedSearchFileFilter() {
			this.ext = SEARCH_FILE_EXTENSION;
		}

		@Override
		public boolean accept(File pathname) {
			if (pathname.getAbsolutePath().endsWith(ext))
				return true;
			else
				return false;
		}

		@Override
		public String getDescription() {
			return "SignStream Search Files";
		}
	}

}

