package edu.bu.signstream.ui.panels.search;

import java.awt.BorderLayout;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;

public class SearchQueryBuilderPanel extends JPanel {
	
	private static final long serialVersionUID = 1L;
	
	private JScrollPane scrollPane = null;
	private SearchGroup outerSearchGroup = null;
	private SearchTreeNode rootSearchTreeNode = null;
	private SS3Database database = null;
	
	// create query builder panel from scratch
	public SearchQueryBuilderPanel(SS3Database database) {
		this.database = database;
		
		outerSearchGroup = new NonBinarySearchGroup(database, null);
		rootSearchTreeNode = outerSearchGroup.getSearchTreeNode();		
		scrollPane = new JScrollPane(outerSearchGroup);
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
	}
	
	// create query builder panel from saved search
	public SearchQueryBuilderPanel(SearchTreeNode rootSearchTreeNode) {
		// TODO
	}
	
	public SearchTreeNode getRootSearchTreeNode() {
		return rootSearchTreeNode;
	}
	
	// create query builder panel from saved search
	public void loadSavedSearchTreeNode(SearchTreeNode rootSearchTreeNode) {
		this.rootSearchTreeNode = rootSearchTreeNode;
		
		outerSearchGroup = new NonBinarySearchGroup(database, null);
		outerSearchGroup.loadSavedSearchTreeNode(rootSearchTreeNode);
		scrollPane = new JScrollPane(outerSearchGroup);

		removeAll();
		
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
		
//		updateQuery(database); // GREG: need?
		
		revalidate();
		repaint();
	}
	
	// update query panels and their tree nodes from a newly switched database
	public void updateQuery(SS3Database database) {
		this.database = database;
		
		updateSearchExpression(outerSearchGroup);
	}
	
	public SearchGroup getOuterSearchGroup() {
		return outerSearchGroup;
	}
	
	private void updateSearchExpression(SearchExpression expression) {
		expression.setDatabase(database);
		
		if (expression instanceof SearchRule) {
			SearchRule searchRule = (SearchRule) expression;
			searchRule.updateSearchRuleAndTreeNode();
		} else if (expression instanceof SearchGroup) {
			SearchGroup searchGroup = (SearchGroup) expression;
			ArrayList<SearchExpression> expressionsList = searchGroup.getAllExpressions();
			for (SearchExpression exp: expressionsList) {
				updateSearchExpression(exp);
			}
		}
	}
}
