package edu.bu.signstream.ui.popup;


import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;
import java.util.function.Consumer;

public class FieldValueSelectPopup extends Popup {
    private boolean clicked = false;

    public FieldValueSelectPopup(Field field, SignStreamSegmentPanel segmentPanel, Consumer<String> onSelect, Runnable onCancel) {
        setUndecorated(true);

        var panel = new JPanel();
        panel.setBackground(UIPalette.getBackground());

        panel.setLayout(new GridBagLayout());
        var gbc = new GridBagConstraints();
        gbc.weightx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = GridBagConstraints.REMAINDER;

        var dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

        var values = new HashMap<String, SS3FieldValue>();
        
        var defaultField = dbScheme.getField(field.getFieldID());
        if (defaultField != null) {
        	for (SS3FieldValue val: defaultField.getValuesAsList()) {
        		values.put(val.getId(), val);
        	}
        }

        for (var v : values.values()) {
            var label = new JLabel(v.getName());
            label.addMouseListener(new MouseAdapter() {
                @Override
                public void mousePressed(MouseEvent e) {
                    clicked = true;
                    onSelect.accept(v.getId());
                    setVisible(false);
                }

                @Override
                public void mouseEntered(MouseEvent e) {
                    label.setForeground(Color.RED);
                }

                @Override
                public void mouseExited(MouseEvent e) {
                    label.setForeground(Color.BLACK);
                }
            });
            panel.add(label, gbc);
        }

        var addNew = new JLabel("Add New...");
        addNew.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                clicked = true;

                var popup = new FieldValueCreatePopup(field, segmentPanel, onSelect, onCancel);
                popup.setLocation(FieldValueSelectPopup.this.getLocation());
                popup.setVisible(true);

                setVisible(false);
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                addNew.setForeground(Color.RED);
            }

            @Override
            public void mouseExited(MouseEvent e) {
                addNew.setForeground(Color.BLACK);
            }
        });
        panel.add(addNew, gbc);

        addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                var key = e.getKeyCode();

                if (key == KeyEvent.VK_ESCAPE) {
                    onCancel.run();
                    setVisible(false);
                }
            }
        });

        addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
                if (clicked) return;
                onCancel.run();
                setVisible(false);
            }

            @Override public void focusGained(FocusEvent e) {}
        });

        add(panel);
        pack();
        SwingUtilities.invokeLater(this::requestFocus);
    }
}
