package edu.bu.signstream.ui.textgrid;

import edu.bu.signstream.common.textgrid.TextGrid;
import edu.bu.signstream.common.util.Pair;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalField;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalFieldWrapper;
import edu.bu.signstream.grepresentation.fields.customField.CustomPointField;
import edu.bu.signstream.grepresentation.fields.customField.CustomPointFieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldPanel;
import edu.bu.signstream.ui.popup.Popup;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

public class ImportTextGridDialog extends Popup {
    public ImportTextGridDialog(CSFieldPanel csFieldPanel, TextGrid grid) {
        setAlwaysOnTop(true);

        var panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        var gbc = new GridBagConstraints();
        gbc.weightx = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = GridBagConstraints.REMAINDER;

        var groups = new String[]{Constants.SPEECH_GROUP_NAME, Constants.GENERIC_GROUP_NAME};
        var groupSelect = new JComboBox<>(groups);
        var groupIndex = csFieldPanel.getCustomGroup(csFieldPanel.getSelectedNode());
        if (groupIndex != CSFieldPanel.BAD_GROUP_NUMBER)
            groupSelect.setSelectedIndex(groupIndex);
        else
            groupSelect.setSelectedIndex(1);
        panel.add(new JLabel("Field Group"), gbc);
        panel.add(groupSelect, gbc);

        var ranges = new ArrayList<TimeRange>();
        if (SS3Singleton.getRealVideoTimeRange() != null)
            ranges.add(SS3Singleton.getVideoTimeRange());
        if (SS3Singleton.getAudioTimeRange() != null)
            ranges.add(SS3Singleton.getAudioTimeRange());
        for (var g : SS3Singleton.getMediaToolBar().segmentPanel().getSS3Database().getGraphs())
            ranges.add(g.range());
        ranges.add(TimeRange.ABSOLUTE);
        var rangeSelect = new JComboBox<>(ranges.toArray(TimeRange[]::new));
        if (SS3Singleton.getAudioTimeRange() != null)
            rangeSelect.setSelectedIndex(1);
        else
            rangeSelect.setSelectedIndex(0);
        panel.add(new JLabel("Time Range"), gbc);
        panel.add(rangeSelect, gbc);

        panel.add(new JLabel("Interval Tiers"), gbc);
        var intervalPanel = new JPanel();
        intervalPanel.setLayout(new BoxLayout(intervalPanel, BoxLayout.Y_AXIS));
        var intervalTiers = new ArrayList<Pair<JCheckBox, TextGrid.IntervalTier>>();
        for (var tier : grid.intervalTiers) {
            var cb = new JCheckBox(tier.name);
            cb.setSelected(true);
            intervalPanel.add(cb);
            intervalTiers.add(Pair.of(cb, tier));
        }
        panel.add(new JScrollPane(intervalPanel), gbc);

        panel.add(new JLabel("Text Tiers"), gbc);
        var textPanel = new JPanel();
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.Y_AXIS));
        var textTiers = new ArrayList<Pair<JCheckBox, TextGrid.TextTier>>();
        for (var tier : grid.textTiers) {
            var cb = new JCheckBox(tier.name);
            cb.setSelected(true);
            textPanel.add(cb);
            textTiers.add(Pair.of(cb, tier));
        }
        panel.add(new JScrollPane(textPanel), gbc);

        var done = new JButton("Done");
        done.addActionListener(l -> {
            var range = (TimeRange)rangeSelect.getSelectedItem();
            var group = groupSelect.getSelectedIndex() == Constants.SPEECH_GROUP_ID
                ? Constants.SPEECH_GROUP_NAME_CAPS
                : Constants.GENERIC_GROUP_NAME_CAPS;

            var segment = csFieldPanel.getSegment();

            for (var pair : textTiers)
                if (pair.first.isSelected())
                    addTextTier(pair.second, range, group, segment);

            for (var pair : intervalTiers)
                if (pair.first.isSelected())
                    addIntervalTier(pair.second, range, group, segment);

            SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

            dispose();
            csFieldPanel.dialog().unblock();

            csFieldPanel.apply();
            csFieldPanel.discard();
            csFieldPanel.close();
        });
        panel.add(done, gbc);

        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                csFieldPanel.dialog().unblock();
            }
        });

        add(panel);
    }

    private SS3Field addFieldToCodingScheme(String name, String type, String group, Segment segment) {
        var db = segment.getSignStreamSegmentPanel().getSS3Database();

		SS3CodingScheme dbScheme = db.getSS3CodingScheme();

        var field = new SS3Field();
        field.setId(Util.getLongUniqueNumber());
        field.setName(name);
        field.setType(type);
        field.setLabel(name);
        field.setPrefix("");
        field.setGroup(group);

        dbScheme.addFieldAndOrder(field);

        return field;
    }

    private void addIntervalTier(TextGrid.IntervalTier tier, TimeRange range, String group, Segment segment) {
        var schemeField = addFieldToCodingScheme(tier.name, Constants.TYPE_CUSTOM_INTERVAL, group, segment);

        var track = new SS3Track();
        track.setId(schemeField.getId());

        for (var pair : tier.intervals) {
            var text = pair.first;
            var interval = pair.second;

            var entity = new SS3Entity();
            var item = new SS3Item();
            item.setStartTimeInfo(interval.t0.in(range));
            item.setEndTimeInfo(interval.t1.in(range));
            item.setText(text);
            entity.addItem(item);

            track.addEntity(entity);
        }

        var segmentPanel = segment.getSignStreamSegmentPanel();

        var field = new CustomIntervalField(track, segmentPanel, segment.getParentField());
        field.setName(tier.name);

        var wrapper = new CustomIntervalFieldWrapper(field, segmentPanel, range);

        var parent = segment.getRootLabelObject().getCustomLabelObject()
            .getChildren()[group.equals(Constants.SPEECH_GROUP_NAME_CAPS) 
                           ? Constants.SPEECH_GROUP_ID : Constants.GENERIC_GROUP_ID];

        var label = new LabelObject(parent, field.getFieldID(), field.getName());
        label.setWrapper(wrapper);
        label.setPermanentlyVisibleAcrossSegmentTier(true);

        parent.addChild(label);
    }

    private void addTextTier(TextGrid.TextTier tier, TimeRange range, String group, Segment segment) {
        var schemeField = addFieldToCodingScheme(tier.name, Constants.TYPE_CUSTOM_POINT, group, segment);

        var track = new SS3Track();
        track.setId(schemeField.getId());

        for (var pair : tier.points) {
            var text = pair.first;
            var time = pair.second.in(range);

            var entity = new SS3Entity();
            var item = new SS3Item();
            item.setStartTimeInfo(time);
            item.setEndTimeInfo(time);
            item.setText(text);
            entity.addItem(item);

            track.addEntity(entity);

            var value = new SS3FieldValue();
            value.setName(text);
            value.setLabel(text);
            schemeField.addValue(value);
        }

        var segmentPanel = segment.getSignStreamSegmentPanel();

        var field = new CustomPointField(track, segmentPanel, segment.getParentField());
        field.setName(tier.name);

        var wrapper = new CustomPointFieldWrapper(field, segmentPanel, range);

        var parent = segment.getRootLabelObject().getCustomLabelObject()
            .getChildren()[group.equals(Constants.SPEECH_GROUP_NAME_CAPS) 
                           ? Constants.SPEECH_GROUP_ID : Constants.GENERIC_GROUP_ID];

        var label = new LabelObject(parent, field.getFieldID(), field.getName());
        label.setWrapper(wrapper);
        label.setPermanentlyVisibleAcrossSegmentTier(true);

        parent.addChild(label);
    }
}
